#! /usr/bin/python

# Package imports
import os.path
import argparse

# UPC/C specific parsing package
import upc_parse.parser

# Process commandline arguments
arg_parser = argparse.ArgumentParser(description="SSTmacro UPC preprocessor. Converts UPC code into C++ code executable in the SSTmacro simulator.")
arg_parser.add_argument("files", metavar="FILE", nargs='+', help="UPC source code file(s).")
arg_parser.add_argument("-v", "--verbose", action="count", default=0, help="increase output information")
arg_parser.add_argument("-r", "--replicate", action="store_true", help="prints the original line above the modified line in the output file")
args = arg_parser.parse_args()

# Loop over input files.
for filename in args.files:
    
    # Open input file
    input_fname = filename
    upc_ifile = open(input_fname, 'r')
        
    # Create output file
    output_fname = os.path.splitext(input_fname)[0] + '.cc'
    upc_ofile    = open(output_fname, 'w')

    # Setup the parser.
    state = upc_parse.parser.new_state(verbosity=args.verbose, replicate=args.replicate)

    # Initial output.
    print "Reading UPC source code from " + input_fname

    # Perform parsing line by line.
    for line in iter(upc_ifile.readline, ''):
        code_tokens = []
        comment_tokens = []
        directive_tokens = []
        state = upc_parse.parser.get_tokens(state, line, code_tokens, comment_tokens, directive_tokens)
        state = upc_parse.parser.apply_rules(state, code_tokens, comment_tokens, directive_tokens)
        state = upc_parse.parser.writeline(state, line, code_tokens, comment_tokens, directive_tokens, upc_ofile)

    # Close input and output files
    upc_ifile.close()
    upc_ofile.close()
    
    # Per-file output.
    print "Writing output to " + output_fname
    print str(state.modified_count()) + " lines modified.\n"

# Final output.
print str(len(args.files)) + " file(s) translated."