/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_LIB_COMPUTE_MINIMD_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_LIB_COMPUTE_MINIMD_H_INCLUDED

#include <sstmac/software/libraries/compute/lib_compute_time.h>
#include <sstmac/sstmacro.h>
#include <sstmac/software/libraries/compute/lib_compute_loops.h>
#include <tools/eiger/api/sstmac_compute_eiger.h>

namespace mini
{
  class eiger_metrics {
  public:
    eiger_metrics(std::string prefix) : prefix(prefix) {
      add("delmaxrss", 0);
      add("minflt", 0);
      add("majflt", 0);
      add("nvcsw", 0);
      add("nivcsw", 0);
    }
    std::map<std::string,double> m;
    std::string prefix;
    void add(std::string name, int value) {
      add(name, double(value));
    }
    void add(std::string name, double value) {
      m[prefix+"_"+name] = value;
    }
  };

  class lib_compute_minimd : public sstmac::sw::lib_compute_time
  {
    // ------- constructor / boost stuff -------------//
  private:

  protected:
    lib_compute_minimd(softwareid id);

  public:
    typedef boost::intrusive_ptr<lib_compute_minimd> ptr;

    virtual
    ~lib_compute_minimd();

    static lib_compute_minimd::ptr
    construct(softwareid t);
    // --------------------------------------//

  public:

    void
    compute(std::string func, const boost::intrusive_ptr<ptr_type> &a1,
        const boost::intrusive_ptr<ptr_type> &a2, int index = 0);

    void
    compute(timestamp t)
    {
      lib_compute_time::compute(t);
    }

    int
    get_int(const std::string& func, int index = 0);

  protected:
    sstmac::sw::softwareid id_;
    std::map<std::string, timestamp> funcs_;
    bool using_loops_;
    sstmac::sw::lib_compute_loops::ptr libloop_;
  };

}
#endif
