/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_ATOM_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_ATOM_H_INCLUDED

#include <minimd.h>

namespace mini
{

  // Ignored types.
  class lib_compute_minimd;

  /**
   * Take over the responsibilities of class Atom in miniMD.
   */
  class minimd::atom : public sstmac::ptr_type
  {
  public:

    virtual std::string
    to_string() const
    {
      return "atom";
    }

    typedef boost::intrusive_ptr<atom> ptr;
    struct Box
    {
      double xprd, yprd, zprd;
      double xlo, xhi;
      double ylo, yhi;
      double zlo, zhi;
      void
      set_prd(double x, double y, double z)
      {
        xprd = x;
        yprd = y;
        zprd = z;
      }
    };

    int natoms;
    std::vector<int> neighatoms; // Added to keep track of everybodies nlocal.
    int nlocal;
    int nghost;
    int nmax;
    // numbers derivable from real executions
    int ratio_neighbors; // average neighbors/atom for thermo energy calc
    double frac_scat; // probability of scatter in force loop.
    // lib_compute_loop numlines coefficients (constant or linear)
    double Fclear_nl;
    double Nbinatoms_nl;
    double Nbuild_nl1;
    double Nbuild_nl2;
    double Nbuild_nl3;
    double Ttemperature_nl;
    double Tpressure_nl;
    double Imove_nl;
    double Iapply_nl;
    double Fscatter_offset;
    double Fscatter_coef;
    double Tenergy_offset;
    double Tenergy_coef;

    Box box;

    void
    init(const lib_compute_minimd::ptr&, const mpiapi::ptr&)
    {
      // all atom:: operations are rolled into runtime overhead for other calls.
    }
  };

}

#endif
