/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_IN_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_IN_H_INCLUDED

#include <minimd.h>

namespace mini {

  // Ignored types.
  class lib_compute_minimd;

  /**
   * Take over the responsibilities of struct In in miniMD.
   */
  class minimd::in  : public ptr_type {
  public:
    virtual std::string to_string() const{
            return "minimd:in";
          }

    typedef boost::intrusive_ptr<in> ptr;

    /// Problem size.
    int nx, ny, nz;
    /// Target temperature.
    double t_request;
    /// Desired density.
    double rho;
    /// avg atom nghost
    int nghostavg;
    /// avg neighbors per local atom for thermo
    int ratio_neighbors;
    /// force scatter loop probability
    double frac_scat;

    void init(const lib_compute_minimd::ptr&, const mpiapi::ptr&) {
      // all in:: operations are rolled into runtime overhead for other calls.
    }
  };

} 

#endif
