/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_TIMER_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_TIMER_H_INCLUDED

#include <minimd.h>
#include <sstmac/software/api/mpi/mpiapi.h>
#include <sstmac/common/sstmac-config.h>

namespace mini {

  /**
   * A stand-in for the miniMD timer functions.
   */
  class minimd::timer  : public ptr_type {
    mpiapi::ptr mpi_;
  public:
    virtual std::string to_string() const{
            return "timer";
          }

    typedef boost::intrusive_ptr<timer> ptr;

    enum ttag {
      TIME_TOTAL=0, TIME_COMM=1, TIME_FORCE=2, TIME_NEIGH=3, TIME_N=4
    };
    static int ttagInt(ttag t) {
      switch(t) {
      case TIME_TOTAL: return 0;
      case TIME_COMM: return 1;
      case TIME_FORCE: return 2;
      case TIME_NEIGH: return 3;
      case TIME_N: return 4;
      default: exit(1);
      }
    }
    static std::string ttagString(ttag t) {
      switch(t) {
      case TIME_TOTAL: return std::string("TIME_TOTAL");
      case TIME_COMM: return std::string("TIME_COMM");
      case TIME_FORCE: return std::string("TIME_FORCE");
      case TIME_NEIGH: return std::string("TIME_NEIGH");
      case TIME_N: return std::string("TIME_N");
      default: exit(1);
      }
    }
    timestamp previous_time;
    std::vector<timestamp> array;

    void init(const lib_compute_minimd::ptr&, const mpiapi::ptr& mpi)
    {
      mpi_ = mpi;
      array.resize(ttagInt(TIME_N));
    }
    
    void stamp() {
      previous_time = mpi_->wtime();
      // std::cerr << "stamp() previous_time =  " << previous_time.sec() <<std::endl;
    }
    
    void stamp(ttag tag) {

      timestamp ts = mpi_->wtime();
      array.at(ttagInt(tag)) += (ts - previous_time);
#if 0
      std::cerr << "stamp(" << ttagString(tag) << ") += " 
	<< (ts.sec() - previous_time.sec()) <<  
	" --> " << array.at(ttagInt(tag)).sec() << 
	" prev= " << previous_time.sec() << "ts= " << ts.sec() << std::endl; 
#endif
      previous_time = mpi_->wtime(); // do not count all that io in next timer
    }

    void barrier_start(ttag tag) {
//      std::cerr << "barrier_start " << tag <<std::endl;
      mpi_->barrier(mpi_->comm_world());
      array.at(ttagInt(tag)) = mpi_->wtime();
//      std::cerr << "barrier_start time " << array.at(ttagInt(tag)).sec() <<std::endl;
      stamp();
    }

    void barrier_stop(ttag tag) {
      mpi_->barrier(mpi_->comm_world());
      timestamp ts = mpi_->wtime();
      array.at(ttagInt(tag)) = ts - array.at(ttagInt(tag));
//      std::cerr << "barrier_stop " << tag <<std::endl;
//      std::cerr << "barrier_stop time " << ts.sec() << " delta " << array.at(ttagInt(tag)).sec() <<std::endl;
    }
  };

} 

#endif
