#!/usr/bin/env python

import sys
simdir=""

class column:
  def __init__(self, name):
    import array
    self.name = name
    self.data = array.array('d')

  def append(self,val):
    self.data.append(val)

  def len(self):
    return len(self.data)

  def avg(self):
    n=len(self.data)
    x=sum(self.data)
    return x/n
 
  def nr(self):
    n=len(self.data)
    return n

class dataset:
	""" holder of parsed csv data columns"""
	def __init__(self, pathname):
		import csv
		allf=open(pathname, 'rb')
		self.dname=allf.readline().strip()
		# print "reading", self.dname
		f=csv.reader(allf, delimiter=',', quotechar='"', skipinitialspace=True)
		self.colnames=f.next()
		self.cols = dict()
		self.rows = 0
		for name in self.colnames:
			if len(name)==0:
				continue
			# print name
			self.cols[name] = column(name)
		junkfirst=f.next() ;# discard 1st sample in each data set: allocation activity
    		for row in f:
			self.rows += 1
			i=0
    			for name in self.colnames:
				if len(name)==0:
					continue
				# print row[i]
				self.cols[name].append(float(row[i]))
				i+=1
      
    
def dotwo(simdir,realdir):
	import os
	import fnmatch
	basefiles=[]
	for file in os.listdir(simdir):
		if fnmatch.fnmatch(file, 'tesla.*.log'):
			basefiles.append(file)

	for fname in basefiles:
		dr =dataset(realdir+fname)
		ds =dataset(simdir+fname)
		atr =dr.cols['wtime'].avg()
		ats =ds.cols['wtime'].avg()
		sr=ats/atr
		print sr, atr, ats, fname

def doall(simdirs, realdirs, runsizes, dataname):
	import os
	import fnmatch
	basefiles=[]
	for file in os.listdir(simdirs[0]):
		if fnmatch.fnmatch(file, 'all_tesla.*.log'):
			basefiles.append(file)

	
	for fname in sorted(basefiles):
		print fname
		for size in range(len(runsizes)):
			dr =dataset(realdirs[size]+fname)
			ds =dataset(simdirs[size]+fname)
			atr =dr.cols['wtime'].avg()
			ats =ds.cols['wtime'].avg()
			try:
				anall =dr.cols['nall'].avg()
				atrDanall=atr/anall
				atsDanall=ats/anall
			except:
				atrDanall=0
				atsDanall=0
			try:
				anloc =dr.cols['nlocal'].avg()
				atsDanloc=ats/anloc
				atrDanloc=atr/anloc
			except:
				atrDanloc =0
				atsDanloc =0
			sr=ats/atr
			print runsizes[size], sr, atr, ats, atrDanloc, atsDanloc, atrDanall, atsDanall


def getname(name):
	"""convert file name to loopnest name. e.g. tesla.Ttemperature_mpi_1.0.log to Ttemperature"""
	vname=name.split('.')[1].split('_')[0]
	return vname

def plotscatter(name,actual,prediction, dataname):
	""" plot scatter of actual vs prediction """
	import matplotlib.pyplot as plt
	plt.figure()
	plt.plot([x for x in actual], [y for y in prediction], 'b.')
	xmin=min(x for x in actual)
	xmax=max(x for x in actual)
	ymin=min(y for y in prediction)
	ymax=max(y for y in prediction)
	pmin=min(xmin,ymin)
	#pmax=max(xmax,ymax)
	pmax=xmax
	plt.plot([pmin,pmax],[pmin,pmax],'k-')
	plt.title('miniMD '+name+", "+dataname)
	plt.xlabel('Walltime of experiment')
	plt.ylabel('SST lib_eiger Walltime')
	plt.yscale('log')
	plt.xscale('log')
	#plt.axes().set_aspect('equal')
	plt.savefig(dataname+"_"+name+'_scatter_sst.pdf', format="pdf")
	plt.close()
	#plt.show()

def plotlines(name,actual,prediction, dataname):
	""" plot values of actual & prediction vs sample#"""
	import matplotlib.pyplot as plt
	plt.figure()
	plt.plot(prediction, 'b,' , actual, 'r,')
	#plt.plot([x for x in actual], 'b.')
	xmin=min(x for x in actual)
	xmax=max(x for x in actual)
	ymin=min(y for y in prediction)
	ymax=max(y for y in prediction)
	pmin=min(xmin,ymin)
	pmax=max(xmax,ymax)
	plt.plot([0,pmin],[0,pmax],'k-')
	plt.title('miniMD '+name+", "+dataname)
	plt.xlabel('sample')
	plt.ylabel('walltime')
	plt.yscale('log')
	# plt.xscale('log')
	#plt.axes().set_aspect('equal')
	plt.savefig(dataname+"_"+name+'_walltimes.pdf', format="pdf")
	plt.close()
	#plt.show()


def plotall(simdirs, realdirs, runsizes, dataname):
	import os
	import fnmatch
	basefiles=[]
	for file in os.listdir(simdirs[0]):
		if fnmatch.fnmatch(file, 'all_tesla.*.log'):
			basefiles.append(file)

	
	for fname in sorted(basefiles):
		print fname
		prediction=[]
		actual=[]
		for size in range(len(runsizes)):
			dr =dataset(realdirs[size]+fname)
			ds =dataset(simdirs[size]+fname)
			atr =dr.cols['wtime'].avg()
			ats =ds.cols['wtime'].avg()
			actual += dr.cols['wtime'].data
			prediction += ds.cols['wtime'].data
		plotscatter(getname(fname), actual, prediction, dataname)
		plotlines(getname(fname), actual, prediction, dataname)


def collectnames(d):
	import os
	import fnmatch
	names = dict()
	for file in os.listdir(d):
		if fnmatch.fnmatch(file, 'tesla.*.log'):
			names[getname(file)]=1
	return sorted(names.keys())

def mergefiles(workdir,files,site,commsize):
	import os
	outname="all_tesla."+site+"_mpi_"+str(commsize)+".log"
	outpath=os.path.join(workdir,outname)
	print outpath
	# print files
	# print commsize
	name=os.path.join(workdir,files[0])
	out=open(outpath,"w")
	first=open(name, 'rb')
	first.readline().strip() ; # throw it away; replace
	print >> out, "\""+outname+"\"" ; # copy row header
	print >> out, first.readline() ,
	first.close()
	for fn in sorted(files):
		name=os.path.join(workdir,fn)
		f=open(name,'rb') ; # toss headers
		f.readline()
		f.readline()
		for line in f:
			print >>out, line ,
		f.close()
	out.close()

def mergedirs(dirs, commsize):
	import os
	import fnmatch
	for d in dirs:
		print d
		names=collectnames(d)
		basefiles=[]
		for site in names:
			# print site
			for file in os.listdir(d):
				pat="tesla."+site+"_mpi_"+str(commsize)+".*.log"
				if fnmatch.fnmatch(file, pat):
					basefiles.append(file)
			workdir=os.path.dirname(d)
			mergefiles(workdir, basefiles, site,commsize)
			basefiles=[]

	
def mergeall(simdirs, realdirs, runsizes, commsize):
	mergedirs(simdirs,commsize)
	mergedirs(realdirs,commsize)


realdirs=[
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.10.10.10.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11short/text/",
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.20.20.20.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11short/text/",
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.30.30.30.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11short/text/",
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.40.40.40.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11short/text/",
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.50.50.50.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11short/text/"
]
simdirs=[
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-10/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-20/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-30/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-40/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-50/"
]
eigerdirs=[
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-eiger-10/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-eiger-20/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-eiger-30/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-eiger-40/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-eiger-50/"
]
runsizes=[10,20,30,40,50]

eightdirs=[
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.20.20.20.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11eight/text/",
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.30.30.30.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11eight/text/",
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.40.40.40.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11eight/text/",
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.50.50.50.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11eight/text/",
"/home/baallan/sst/mantevo/miniMD-0.1/dumps.60.60.60.100.10.10.10.0.00462.1.444.0.8442.20.2.5.2.8.1.u11eight/text/"
]
eightsimdirs=[
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-20/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-30/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-40/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-50/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-60/"
]
eighteigerdirs=[
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-eiger-20/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-eiger-30/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-eiger-40/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-eiger-50/",
"/home/baallan/sst/sstmacro/skeletons/minimd-cpu/log-8-eiger-60/"
]
eightsizes=[20,30,40,50,60]

#mergeall(eigerdirs, realdirs, runsizes, 1)
#doall(eigerdirs, realdirs, runsizes, "x5550-1-eiger")
#plotall(eigerdirs, realdirs, runsizes, "x5550-1-eiger")

mergeall(eighteigerdirs, eightdirs, eightsizes, 8)
doall(eighteigerdirs, eightdirs, eightsizes,"x5550-8-eiger")
plotall(eighteigerdirs, eightdirs, eightsizes,"x5550-8-eiger")
