/*
 * hpccg_matrix.h
 *
 *  Created on: Nov 7, 2012
 *      Author: ghendry
 */

#ifndef HPCCG_MATRIX_STUBS_H_
#define HPCCG_MATRIX_STUBS_H_

#include "HPCCG/hpccg_matrix.h"
#include <sstmac/software/api/hpx/components/hpx_stubs.h>
#include <sstmac/software/api/hpx/lcos/eager_future.h>
#include "HPCCG/buffer_wrapper.h"

namespace hpx_hpccg
{

  class hpccg::matrix::matrix_stubs : public hpx::components::hpx_stubs
  {
  protected:
    matrix_stubs(hpx::globalid gid) :
      hpx_stubs(gid)
    {

    }

  public:

    typedef boost::intrusive_ptr<matrix_stubs> ptr;

    static ptr
    construct(hpx::globalid gid)
    {
      return ptr(new matrix_stubs(gid));
    }

    virtual
    ~matrix_stubs()
    {
    }

    // -------------------
    hpx::lcos::promise<int>::ptr
    init_async(std::size_t objectid, config_params::ptr par,
        const std::vector<stub_ptr>& stubs)
    {
      matrix::init_paramtype::ptr params = matrix::init_paramtype::construct(
          objectid, par, stubs, sizeof(size_t), par->get_byte_length(),
          stubs.size() * sizeof(hpx::globalid));

      return hpx::lcos::eager_future<int>::construct(
          matrix::get_action_name(matrix::INIT), gid_, params);
    }

    /// Initialize the \a gtc::server::point instance with the
    /// given point file.
    void
    init(std::size_t objectid, config_params::ptr const& par,
        const std::vector<stub_ptr>& stubs)
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise.
      init_async(objectid, par, stubs)->get();
    }

    hpx::lcos::promise<buffer_wrapper<int> >::ptr
    get_start_row()
    {
      matrix::get_start_row_paramtype::ptr params;
      return hpx::lcos::eager_future<buffer_wrapper<int> >::construct(
          matrix::get_action_name(matrix::GET_START_ROW), gid_, params);
    }

    hpx::lcos::promise<buffer_wrapper<int> >::ptr
    get_tmp_neighbors()
    {
      matrix::get_start_row_paramtype::ptr params;
      return hpx::lcos::eager_future<buffer_wrapper<int> >::construct(
          matrix::get_action_name(matrix::GET_TMP_NEIGHBORS), gid_, params);
    }

    hpx::lcos::promise<int>::ptr
    add_send_neighbor(int val)
    {
      matrix::add_send_neighbor_paramtype::ptr params =
          matrix::add_send_neighbor_paramtype::construct(val, sizeof(int));
      return hpx::lcos::eager_future<int>::construct(
          matrix::get_action_name(matrix::ADD_SEND_NEIGHBOR), gid_, params);
    }

    hpx::lcos::promise<int>::ptr
    add_send_length(int val)
    {
      matrix::add_send_length_paramtype::ptr params =
          matrix::add_send_length_paramtype::construct(val, sizeof(int));
      return hpx::lcos::eager_future<int>::construct(
          matrix::get_action_name(matrix::ADD_SEND_LENGTH), gid_, params);
    }

    hpx::lcos::promise<int>::ptr
    add_send_element(buffer_wrapper<int> val)
    {
      matrix::add_send_element_paramtype::ptr params =
          matrix::add_send_element_paramtype::construct(val,
              val.get_byte_length());
      return hpx::lcos::eager_future<int>::construct(
          matrix::get_action_name(matrix::ADD_SEND_ELEMENT), gid_, params);
    }

    hpx::lcos::promise<int>::ptr
    add_elements(buffer_wrapper<double> val)
    {
      matrix::add_elements_paramtype::ptr params =
          matrix::add_elements_paramtype::construct(val, val.get_byte_length());
      return hpx::lcos::eager_future<int>::construct(
          matrix::get_action_name(matrix::ADD_ELEMENTS), gid_, params);
    }

    hpx::lcos::promise<int>::ptr
    HPCCG_Solve(int iter, double tol)
    {
      matrix::hpccg_solve_paramtype::ptr params =
          matrix::hpccg_solve_paramtype::construct(iter, tol);
      return hpx::lcos::eager_future<int>::construct(
          matrix::get_action_name(matrix::HPCCG_SOLVE), gid_, params);
    }

    hpx::lcos::promise<double>::ptr
    get_ddot_local()
    {
      matrix::get_ddot_local_paramtype::ptr params;
      return hpx::lcos::eager_future<double>::construct(
          matrix::get_action_name(matrix::GET_DDOT_LOCAL), gid_, params);
    }

  };
}

#endif
