/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SKELETONS_PARALLEX_GTC_GTC_APP_H_INCLUDED
#define SKELETONS_PARALLEX_GTC_GTC_APP_H_INCLUDED

#include <sstmac/software/api/hpx/hpx_app.h>
#include <sstmac/software/libraries/filesystem/nppfs.h>

#include <gtc/gtc_params.h>

#include <sstmac/sstmacro.h>

#include <sstmac/software/api/hpx/lcos/and_gate.h>

namespace hpx_gtc
{

  class gtc_app : public hpx::hpx_app
  {

  protected:

    config_parameters::ptr gtcparams_;


    class gtc_point;
    class gtc_particle;

    virtual void
    register_actions();

    virtual void
    register_components();

    void
    setup();

    void
    read_input_params();

    gtc_app(const sstmac::sim_parameters::ptr& params)
        : hpx::hpx_app(params)
    {
    }

  public:
    typedef boost::intrusive_ptr<gtc_app> ptr;

    gtc_app(sstmac::sw::softwareid id) :
        hpx_app(id)
    {

    }

    virtual
    ~gtc_app()
    {
    }

    virtual void
    hpx_main();

    static ptr
    construct(sstmac::sw::softwareid id)
    {
      ptr retval(new gtc_app(id));
      return retval;
    }

    static ptr
    construct(const sstmac::sim_parameters::ptr& params){
        return new gtc_app(params);
    }

    virtual app::ptr
    clone(sstmac::sw::softwareid id)
    {
      return construct(id);
    }

    virtual void
    consume_params(const sstmac::sim_parameters::ptr &params);

    virtual std::string
    to_string() const
    {
      return "gtc_app";
    }
  };

}

#endif /* GTC_APP_H_ */
