/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_PARALLEX_GTC_GTC_PARTICLE_STUBS_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_PARALLEX_GTC_GTC_PARTICLE_STUBS_H_INCLUDED

#include <gtc/gtc_particle.h>
#include <sstmac/software/api/hpx/components/hpx_stubs.h>
#include <sstmac/software/api/hpx/lcos/promise.h>

namespace hpx_gtc
{

  class gtc_app::gtc_particle::gtc_particle_stubs : public hpx::components::hpx_stubs
  {

  protected:
    gtc_particle_stubs(hpx::globalid gid) :
        hpx_stubs(gid)
    {

    }

  public:
    typedef boost::intrusive_ptr<gtc_particle_stubs> ptr;

    static ptr
    construct(hpx::globalid gid)
    {
      return ptr(new gtc_particle_stubs(gid));
    }

    virtual
    ~gtc_particle_stubs()
    {
    }

    hpx::lcos::promise<int>::ptr
    init_async(std::size_t objectid, double posx, double posy, double posz);

    void
    init(std::size_t objectid, double posx, double posy, double posz);

    hpx::lcos::promise<double>::ptr
    distance_async(double posx, double posy, double posz);

    double
    distance(double posx, double posy, double posz);
  };

}

#endif
