/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SKELETONS_PARALLEX_GTC_GTC_POINT_STUBS_H_INCLUDED
#define SKELETONS_PARALLEX_GTC_GTC_POINT_STUBS_H_INCLUDED

#include <gtc/gtc_point.h>
#include <sstmac/software/api/hpx/components/hpx_stubs.h>
#include <sstmac/software/api/hpx/lcos/eager_future.h>

namespace hpx_gtc
{

  class gtc_app::gtc_point::gtc_point_stubs : public hpx::components::hpx_stubs
  {

  protected:
    gtc_point_stubs(hpx::globalid gid) :
        hpx_stubs(gid)
    {

    }

  public:
    typedef boost::intrusive_ptr<gtc_point_stubs> ptr;

    static ptr
    construct(hpx::globalid gid)
    {
      return ptr(new gtc_point_stubs(gid));
    }

    virtual
    ~gtc_point_stubs()
    {
    }

    /// Initialize the \a gtc::server::point instance with the
    /// given point file.
    hpx::lcos::promise<int>::ptr
    init_async(std::size_t objectid, config_parameters::ptr const& par,
        const std::vector<stub_ptr>& stubs)
    {
      gtc_point::init_paramtype::ptr params =
          gtc_point::init_paramtype::construct(objectid, par, stubs,
              sizeof(size_t), par->get_byte_length(),
              stubs.size() * sizeof(hpx::globalid));

      return hpx::lcos::eager_future<int>::construct(
          gtc_point::get_action_name(gtc_point::INIT), gid_, params);
    }

    /// Initialize the \a gtc::server::point instance with the
    /// given point file.
    void
    init(std::size_t objectid, config_parameters::ptr const& par,
        const std::vector<stub_ptr>& stubs)
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise.
      init_async(objectid, par, stubs)->get();
    }

    hpx::lcos::promise<int>::ptr
    load_async()
    {
      gtc_point::load_paramtype::ptr params;

      return hpx::lcos::eager_future<int>::construct(
          gtc_point::get_action_name(gtc_point::LOAD), gid_, params);

    }

    void
    load()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise.
      load_async()->get();
    }

    hpx::lcos::promise<int>::ptr
    chargei_async(std::size_t istep)
    {
      chargei_paramtype::ptr params = chargei_paramtype::construct(istep);

      return hpx::lcos::eager_future<int>::construct(
          gtc_point::get_action_name(CHARGEI), gid_, params);
    }

    void
    chargei(std::size_t istep, config_parameters::ptr const& par)
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise.
      chargei_async(istep)->get();
    }

    hpx::lcos::promise<fake_array<double> >::ptr
    get_densityi_async()
    {

      get_densityi_paramtype::ptr params;
      return hpx::lcos::eager_future<fake_array<double> >::construct(
          gtc_point::get_action_name(GET_ZONALI),
          gid_,
          params,
          hpx::parcels::vector1_depayloadizer<double, fake_array<double> >::construct());
    }

    fake_array<double>
    get_densityi()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_densityi_async()->get();
    }

    hpx::lcos::promise<fake_array<double> >::ptr
    get_zonali_async()
    {
      get_zonali_paramtype::ptr params;
      return hpx::lcos::eager_future<fake_array<double> >::construct(
          gtc_point::get_action_name(GET_ZONALI),
          gid_,
          params,
          hpx::parcels::vector1_depayloadizer<double, fake_array<double> >::construct());
    }

    fake_array<double>
    get_zonali()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_zonali_async()->get();
    }

    hpx::lcos::promise<int>::ptr
    smooth_async(std::size_t iflag, std::size_t idiag)
    {
      smooth_paramtype::ptr params = smooth_paramtype::construct(iflag, idiag);
      return hpx::lcos::eager_future<int>::construct(
          gtc_point::get_action_name(SMOOTH), gid_, params);
    }

    void
    smooth(std::size_t iflag, std::size_t idiag)
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise.
      smooth_async(iflag, idiag)->get();
    }

    hpx::lcos::promise<fake_array<double> >::ptr
    get_phi_async(std::size_t depth)
    {
      get_phi_paramtype::ptr params = get_phi_paramtype::construct(depth);
      return hpx::lcos::eager_future<fake_array<double> >::construct(
          gtc_point::get_action_name(GET_PHI),
          gid_,
          params,
          hpx::parcels::vector1_depayloadizer<double, fake_array<double> >::construct());
    }

    fake_array<double>
    get_phi(std::size_t depth)
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_phi_async(depth)->get();
    }

    hpx::lcos::promise<fake_array<double> >::ptr
    get_eachzeta_async()
    {
      get_eachzeta_paramtype::ptr params;
      return hpx::lcos::eager_future<fake_array<double> >::construct(
          gtc_point::get_action_name(GET_EACHZETA),
          gid_,
          params,
          hpx::parcels::vector1_depayloadizer<double, fake_array<double> >::construct());
    }

    fake_array<double>
    get_eachzeta()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_eachzeta_async()->get();
    }

    hpx::lcos::promise<int>::ptr
    field_async()
    {
      field_paramtype::ptr params;
      return hpx::lcos::eager_future<int>::construct(gtc_point::get_action_name(FIELD),
          gid_, params);
    }

    void
    field(hpx::globalid const& gid,
        const std::vector<hpx::globalid>& point_components,
        config_parameters::ptr const& par)
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise.
      field_async()->get();
    }

    hpx::lcos::promise<fake_array<double> >::ptr
    get_evector_async(std::size_t depth, std::size_t extent)
    {
      get_evector_paramtype::ptr params = get_evector_paramtype::construct(depth, extent);
      return hpx::lcos::eager_future<fake_array<double> >::construct(
          gtc_point::get_action_name(GET_EVECTOR), gid_, params,
          hpx::parcels::vector1_depayloadizer<double, fake_array<double> >::construct());
    }

    fake_array<double>
    get_evector(std::size_t depth, std::size_t extent)
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_evector_async(depth, extent)->get();
    }

    hpx::lcos::promise<int>::ptr
    pushi_async(std::size_t irk, std::size_t istep, std::size_t idiag)
    {

    	pushi_paramtype::ptr params = pushi_paramtype::construct(irk, istep, idiag);
    	return hpx::lcos::eager_future<int>::construct(gtc_point::get_action_name(PUSHI), gid_, params);

    }

    void
    pushi(std::size_t irk, std::size_t istep, std::size_t idiag)
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise.
       pushi_async(irk, istep, idiag)->get();
    }

    hpx::lcos::promise<fake_array<double> >::ptr
    get_dden_async()
    {
      get_dden_paramtype::ptr params;
      return hpx::lcos::eager_future<fake_array<double> >::construct(gtc_point::get_action_name(GET_DDEN), gid_, params,
          hpx::parcels::vector1_depayloadizer<double, fake_array<double> >::construct());
    }

    fake_array<double>
    get_dden()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_dden_async()->get();
    }

    hpx::lcos::promise<fake_array<double> >::ptr
    get_dtem_async()
    {
    	get_dtem_paramtype::ptr params;
      return hpx::lcos::eager_future<fake_array<double> >::construct(gtc_point::get_action_name(GET_DTEM), gid_, params,
    		  hpx::parcels::vector1_depayloadizer<double, fake_array<double> >::construct());
    }

    fake_array<double>
    get_dtem()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_dtem_async()->get();
    }

    hpx::lcos::promise<int>::ptr
    shifti_async()
    {
      shifti_paramtype::ptr params;
      return hpx::lcos::eager_future<int>::construct(gtc_point::get_action_name(SHIFTI), gid_, params);
    }

    void
    shifti()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise.
      shifti_async()->get();
    }

    hpx::lcos::promise<std::size_t>::ptr
    get_msend_async()
    {
      get_msend_paramtype::ptr params;
      return hpx::lcos::eager_future<size_t>::construct(gtc_point::get_action_name(GET_MSEND), gid_, params);
    }

    std::size_t
    get_msend()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_msend_async()->get();
    }

    hpx::lcos::promise<fake_array<std::size_t> >::ptr
    get_msendright_async()
    {
      get_msendright_paramtype::ptr params;
      return hpx::lcos::eager_future<fake_array<size_t> >::construct(gtc_point::get_action_name(GET_MSENDRIGHT), gid_, params);
    }

    fake_array<std::size_t>
    get_msendright()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_msendright_async()->get();
    }

    hpx::lcos::promise<fake_array<double> >::ptr
    get_sendright_async()
    {
      get_sendright_paramtype::ptr params;
      return hpx::lcos::eager_future<fake_array<double> >::construct(gtc_point::get_action_name(GET_SENDRIGHT), gid_, params,
                       hpx::parcels::vector1_depayloadizer<double, fake_array<double> >::construct());
    }

    fake_array<double>
    get_sendright()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_sendright_async()->get();
    }

    hpx::lcos::promise<fake_array<std::size_t> >::ptr
    get_msendleft_async()
    {
      get_msendleft_paramtype::ptr params;
      return hpx::lcos::eager_future<fake_array<size_t> >::construct(gtc_point::get_action_name(GET_MSENDLEFT), gid_, params);
    }

    fake_array<std::size_t>
    get_msendleft()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
      return get_msendleft_async()->get();
    }

    hpx::lcos::promise<fake_array<double> >::ptr
    get_sendleft_async()
    {
      get_sendleft_paramtype::ptr params;
      return hpx::lcos::eager_future<fake_array<double> >::construct(gtc_point::get_action_name(GET_SENDLEFT), gid_, params,
          hpx::parcels::vector1_depayloadizer<double, fake_array<double> >::construct());
    }

    fake_array<double>
    get_sendleft()
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise
     return get_sendleft_async()->get();
    }

    hpx::lcos::promise<int>::ptr
    poisson_async(std::size_t iflag, std::size_t istep, std::size_t irk)
    {
      poisson_paramtype::ptr params = poisson_paramtype::construct(iflag, istep, irk);
       return hpx::lcos::eager_future < int>::construct(gtc_point::get_action_name(POISSON), gid_, params);
    }

    void
    poisson(std::size_t iflag, std::size_t istep, std::size_t irk)
    {
      // The following get yields control while the action above
      // is executed and the result is returned to the promise.
      poisson_async(iflag, istep, irk)->get();
    }

  };

}

#endif
