//#if HAVE_CONFIG_H
# include "sstmac/common/sstmac-config.h"
//#endif

#include "gacpp.h"
#include "ftypes.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <sstmac/software/api/mpi/sstmac_mpi.h>

extern long random();
extern void srandom(unsigned);

static DoublePrecision gai_drand_(Integer *flag)
{
#ifdef CRAY_YMP
    return ran((unsigned int)*flag);
#else
    if (*flag)
        srandom((unsigned) *flag);

    return ((DoublePrecision) random()) * 4.6566128752458e-10;
#endif
}

#define drand_ SSTMAC_F77_FUNC_(drand,DRAND)
DoublePrecision drand_(Integer *flag)
{
    return (gai_drand_(flag));
}

/************
TODO:
ga_add
ga_duplicate
nga_acc
nga_access
nga_release
***************/

#define ga_add_ SSTMAC_F77_FUNC_(ga_add, GA_ADD)
void FATR ga_add_(DoublePrecision* alpha, Integer* g_a,
		  DoublePrecision* beta, Integer* g_b, Integer* g_c)
{
  gaAdd(alpha[0], g_a[0], beta[0], g_b[0], g_c[0]);
}

#define tascel_init_ SSTMAC_F77_FUNC_(tascel_init, TASCEL_INIT)
void FATR tascel_init_()
{
  tascelInit();
}

#define tascel_finalize_ SSTMAC_F77_FUNC_(tascel_finalize, TASCEL_FINALIZE)
void FATR tascel_finalize_()
{
  tascelFinalize();
}

#define ga_allocate_ SSTMAC_F77_FUNC_(ga_allocate, GA_ALLOCATE)
Logical FATR ga_allocate_(Integer *g_a)
{
  // Set data actually does the allocation right now
  return 1;
}

#define ga_copy_ SSTMAC_F77_FUNC_(ga_copy, GA_COPY)
void FATR ga_copy_(Integer *g_a, Integer *g_b)
{
  gaCopy(g_a[0], g_b[0]);
}

#define ga_create_handle_ SSTMAC_F77_FUNC_(ga_create_handle, GA_CREATE_HANDLE)
Integer FATR ga_create_handle_()
{
  return gaCreateHandle();
}

#define ga_destroy_ SSTMAC_F77_FUNC_(ga_destroy, GA_DESTROY)
Logical FATR ga_destroy_(Integer *g_a)
{
  gaDestroy(g_a[0]);
  return 1;
}

#define ga_dgemm_ SSTMAC_F77_FUNC_(ga_dgemm,GA_DGEMM)
void FATR ga_dgemm_(
#if F2C_HIDDEN_STRING_LENGTH_AFTER_ARGS
        char *transa, char *transb,
        Integer *m, Integer *n, Integer *k,
        DoublePrecision *alpha, Integer *g_a, Integer *g_b,
        DoublePrecision *beta, Integer *g_c, int talen, int tblen
#else
        char *transa, int talen, char *transb, int tblen,
        Integer *m, Integer *n, Integer *k,
        DoublePrecision *alpha, Integer *g_a, Integer *g_b,
        DoublePrecision *beta, Integer *g_c
#endif
        )
{
  assert(*m == *n && *n == *k);
  gaDgemm(transa, transb, m[0], n[0], k[0], alpha, g_a[0], g_b[0],
	  beta, g_c[0], talen, tblen);
}

#define ga_dgop_ SSTMAC_F77_FUNC_(ga_dgop,GA_DGOP)
void FATR ga_dgop_(Integer *type, DoublePrecision *x, Integer *n, char *op, int len)
{
  if (len == 4) {
    DEBUG_PRINT("ga_dgop: type = %d, x = [%f,%f,%f,%f], n = %d, op = %c, len = %d\n",
		*type, x[0], x[1], x[2], x[3], *n, *op, len);
  } else {
    DEBUG_PRINT("ga_dgop: type = %d, x = [%f], n = %d, op = %c, len = %d\n",
		*type, *x, *n, *op, len);
  }

  amBarrier();

  DoublePrecision* ret = alloca(sizeof(DoublePrecision) * *n);
  if (strncmp(op, "+", 1) == 0) {
    MPI_Allreduce(x, ret, *n, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
  } else if (strncmp(op, "max", 3) == 0) {
    MPI_Allreduce(x, ret, *n, MPI_DOUBLE, MPI_MAX, MPI_COMM_WORLD);
  } else {
    fprintf(stderr, "error: unsupported reduction operation: \"%s\"\n", op);
    exit(1);
  }

  memcpy(x, ret, sizeof(DoublePrecision) * *n);
}

#define ga_diag_seq_ SSTMAC_F77_FUNC_(ga_diag_seq, GA_DIAG_SEQ)
void FATR ga_diag_seq_(Integer *g_a, Integer *g_s, Integer *g_v, DoublePrecision *eval)
{
  gaDiagSeq(g_a[0], g_s[0], g_v[0], eval);
}

#define ga_diag_std_seq_ SSTMAC_F77_FUNC_(ga_diag_std_seq, GA_DIAG_STD_SEQ)
void FATR ga_diag_std_seq_(Integer* g_a, Integer* g_v, DoublePrecision *eval)
{
  gaDiagStdSeq(g_a[0], g_v[0], eval);
}

#define ga_duplicate_ SSTMAC_F77_FUNC_(ga_duplicate, GA_DUPLICATE)
Logical FATR ga_duplicate_( Integer *g_a, Integer *g_b, char *array_name, int slen)
{
  assert(0);
  return 0;
}

#define ga_error_ SSTMAC_F77_FUNC_(ga_error, GA_ERROR)
void FATR ga_error_(
#if F2C_HIDDEN_STRING_LENGTH_AFTER_ARGS
    char *string, Integer *icode, int slen
#else
    char *string, int slen, Integer *icode
#endif
    )
{
  fprintf(stderr, "error occurred: ga_error\n");
}

#define ga_igop_ SSTMAC_F77_FUNC_(ga_igop,GA_IGOP)
void FATR ga_igop_(Integer *type, Integer *x, Integer *n, char *op, int len)
{
  DEBUG_PRINT("ga_igop: type = %d, x = %d, n = %d, op = %c, len = %d\n",
	      *type, *x, *n, *op, len);
  assert(*op == '+');
  Integer ret;
  if (sizeof(Integer) == sizeof(int))
    MPI_Allreduce(x, &ret, *n, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
  else if (sizeof(Integer) == sizeof(long))
    MPI_Allreduce(x, &ret, *n, MPI_LONG, MPI_SUM, MPI_COMM_WORLD);
  *x = ret;
}

#define ga_initialize_ SSTMAC_F77_FUNC_(ga_initialize, GA_INITIALIZE)
void FATR ga_initialize_()
{
}

#define ga_inquire_memory_ SSTMAC_F77_FUNC_(ga_inquire_memory, GA_INQUIRE_MEMORY)
Integer FATR ga_inquire_memory_()
{
  return 0;
}

#define ga_nblock_ SSTMAC_F77_FUNC_(ga_nblock, GA_NBLOCK)
void FATR ga_nblock_(Integer *g_a, Integer *nblock)
{
}

#define ga_nnodes_ SSTMAC_F77_FUNC_(ga_nnodes, GA_NNODES)
Integer FATR ga_nnodes_()
{
  return getNProc();
}

#define ga_nodeid_ SSTMAC_F77_FUNC_(ga_nodeid, GA_NODEID)
Integer ga_nodeid_()
{
  return getNodeID();
}

#define ga_set_block_cyclic_proc_grid_ SSTMAC_F77_FUNC_(ga_set_block_cyclic_proc_grid, GA_SET_BLOCK_CYCLIC_PROC_GRID)
void FATR ga_set_block_cyclic_proc_grid_(Integer *g_a, Integer *dims, Integer *proc_grid)
{
}

#define ga_set_data_ SSTMAC_F77_FUNC_(ga_set_data, GA_SET_DATA)
void FATR ga_set_data_(Integer* g_a, Integer* ndim, Integer *dims, Integer* type)
{
  if (ndim[0] == 1) {
    DEBUG_PRINT("setData han = %d, ndim = %d, dims[0] = %d, type = %d\n",
		*g_a, *ndim, dims[0], *type);
  } else {
    DEBUG_PRINT("setData han = %d, ndim = %d, dims = [%d,%d], type = %d\n",
		*g_a, *ndim, dims[0], dims[1], *type);
  }
  gaCreate(g_a[0], ndim[0], dims, type[0]);
}

#define ga_symmetrize_ SSTMAC_F77_FUNC_(ga_symmetrize, GA_SYMMETRIZE)
void FATR ga_symmetrize_(Integer *g_a)
{
  gaSymmetrize(g_a[0]);
}

#define ga_sync_ SSTMAC_F77_FUNC_(ga_sync, GA_SYNC)
void FATR ga_sync_()
{
  amBarrier();
}

#define ga_terminate_ SSTMAC_F77_FUNC_(ga_terminate, GA_TERMINATE)
void FATR ga_terminate_()
{
}

#define ga_zero_ SSTMAC_F77_FUNC_(ga_zero, GA_ZERO)
void FATR ga_zero_(Integer *g_a)
{
  gaZero(g_a[0]);
}

#define nga_acc_ SSTMAC_F77_FUNC_(nga_acc, NGA_ACC)
void FATR nga_acc_(Integer *g_a, Integer *lo, Integer *hi,
                   void *buf, Integer *ld, void *alpha)
{
  assert(0);
}

#define nga_access_ SSTMAC_F77_FUNC_(nga_access,NGA_ACCESS)
void FATR nga_access_(Integer* g_a, Integer lo[], Integer hi[],
                      AccessIndex* index, Integer ld[])
{
  assert(0);
}

#define nga_distribution_ SSTMAC_F77_FUNC_(nga_distribution, NGA_DISTRIBUTION)
void FATR nga_distribution_(Integer *g_a, Integer *proc, Integer *lo, Integer *hi)
{
  assert(0);
}

#define nga_get_ SSTMAC_F77_FUNC_(nga_get, NGA_GET)
void FATR nga_get_(Integer *g_a, Integer *lo, Integer *hi,
                   void *buf, Integer *ld)
{
  //DEBUG_PRINT("nga_get_: g_a = %d, lo = [%d,%d], hi = [%d,%d], ld = %d\n",
  //g_a[0], lo[0], lo[1], hi[0], hi[1], ld[0]);
  gaGet(g_a[0], lo, hi, buf, ld);
}

#define nga_put_ SSTMAC_F77_FUNC_(nga_put, NGA_PUT)
void FATR nga_put_(Integer *g_a, Integer *lo,
                   Integer *hi, void *buf, Integer *ld)
{
  //DEBUG_PRINT("nga_put_: g_a = %d, lo = [%d,%d], hi = [%d,%d], ld = %d\n",
  //g_a[0], lo[0], lo[1], hi[0], hi[1], ld[0]);
  gaPut(g_a[0], lo, hi, buf, ld);
}

#define nga_read_inc_ SSTMAC_F77_FUNC_(nga_read_inc, NGA_READ_INC)
Integer FATR nga_read_inc_(Integer *g_a, Integer *subscript,
                           Integer *inc)
{
  return gaReadInc(g_a[0], subscript[0], inc[0]);
}

#define nga_release_ SSTMAC_F77_FUNC_(nga_release, NGA_RELEASE)
void FATR nga_release_(Integer *g_a, Integer *lo, Integer *hi)
{
}

#define ma_init_ SSTMAC_F77_FUNC_(ma_init, MA_INIT)
Integer FATR ma_init_(Integer datatype, Integer stack, Integer heap)
{
   return 0;
}
