/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


/*
 * The major contents of this file was taken from http://sushpa.wordpress.com/2008/05/20/simple-matrix-multiplication-on-mpi/
 * and formatted to fit your screen (made into SST/macro skeleton)
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_SYSTOLICMXM_SIMPLEMXM_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_SYSTOLICMXM_SIMPLEMXM_H_INCLUDED

#include <sstmac/sstmacro.h>
#include <compute/lib_compute_matrix.h>
#include <math.h>
#include <stdlib.h>

using sstmac::sw::lib_compute_matrix;

namespace matrix
{

    /**
     * Torus-wrap Matrix Multiply.
     */
    class simplemxm : public sstmac::sw::mpiapp
    {
    public:
      long matrix_order_;
      lib_compute_matrix::ptr compute_matrix_;

    private:

      simplemxm(softwareid id);

      simplemxm(const sstmac::sim_parameters::ptr& params) : sstmac::sw::mpiapp(params) {}

      long getRowCount(long rowsTotal, long mpiRank, long mpiSize);


    public:
      typedef boost::intrusive_ptr<simplemxm> ptr;
      static ptr
      construct(softwareid id);

      static ptr
      construct(const sstmac::sim_parameters::ptr& params)
      {
          return new simplemxm(params);
      }

      ~simplemxm() throw ();

      app::ptr
      clone(softwareid id);

      void
      skeleton_main();

      virtual void
      consume_params(const sim_parameters::ptr &params);

      virtual std::string
      to_string() const
      {
        return "simplemxm";
      }

    };

}

#endif

