/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_IP_QUEUE_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_IP_QUEUE_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/backends/native/sim_partition.h>
#include <sstmac/common/sstevent.h>
#include <sstmac/common/timestamp.h>
#include <sstmac/backends/native/ipc_event.h>
#include <sstmac/common/debug.h>

namespace sstmac
{
  namespace hw
  {
    class interconnect;
  }
  namespace native
  {
    class ip_queue : public ptr_type
    {
    protected:

      ip_queue(lppath coords);

      lppath path_;

      static debug dbg_;

      typedef std::multimap<timestamp, ipc_event::ptr> queue_type;
      queue_type queue_;

    public:
      typedef boost::intrusive_ptr<ip_queue> ptr;

      static timestamp bigtime;

      lppath
      get_coords() const
      {
        return path_;
      }

      virtual std::string
      to_string() const
      {
        return "ip_queue";
      }

      virtual ipc_event::ptr
      do_one_event() = 0;

      virtual void
      get_front(timestamp &t, event::ptr &e) = 0;

      virtual void
      insert_event(const timestamp &t, ipc_event::ptr ev) = 0;

      virtual bool
      is_empty() = 0;

      virtual bool
      check_queue(const timestamp& now, const timestamp &t,
          const ip_queue::ptr &outq, bool &added) = 0;

      virtual void
      sync(const timestamp &t) = 0;

      virtual void
      ending_simulation() = 0;

      virtual void
      set_switches(const boost::intrusive_ptr<hw::interconnect> &fab);

      virtual bool
      is_done()
      {
        return false;
      }
      virtual void
      reset_events()
      {

      }

      virtual int
      check_recv(bool wait) = 0;

      virtual
      ~ip_queue()
      {
      }
    };

  }
}
#endif
