/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_MPIPARALLEL_MPI_SERIALIZER_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_MPIPARALLEL_MPI_SERIALIZER_H_INCLUDED

#include <sstmac/common/messages/sst_serializer.h>
#include <sstmac/common/debug.h>
namespace sstmac
{
  namespace native
  {

#define MPIPACKER_DECL_PACK(thetype) virtual void pack(thetype t);
    class mpi_ser_packer : public sstmac::ser_packer
    {
    protected:
      template<class T>
        void
        do_pack(T t);

      mpi_ser_packer(char* b) :
        buffer_(b), at_(0)
      {

      }

    public:
      typedef boost::intrusive_ptr<mpi_ser_packer> ptr;

      static ptr
      construct(char* buf){
        return ptr(new mpi_ser_packer(buf));
      }

      virtual
      ~mpi_ser_packer(){ }


      MPIPACKER_DECL_PACK(int)
      MPIPACKER_DECL_PACK(float)
      MPIPACKER_DECL_PACK(short)
      MPIPACKER_DECL_PACK(double)
      MPIPACKER_DECL_PACK(long int)
      MPIPACKER_DECL_PACK(long long int)
      MPIPACKER_DECL_PACK(unsigned int)
      MPIPACKER_DECL_PACK(long unsigned int)
      MPIPACKER_DECL_PACK(long long unsigned int)
      MPIPACKER_DECL_PACK(char)
      MPIPACKER_DECL_PACK(bool)
      MPIPACKER_DECL_PACK(std::string)
      MPIPACKER_DECL_PACK(timestamp)

      size_t
      get_bytes_serialized() const{
        return at_;
      }

      virtual void pack_array(void* arr, size_t size);

    protected:
      char* buffer_;
      size_t at_;
    };
#undef MPIPACKER_DECL_PACK

#define MPIUNPACKER_DECL_UNPACK(thetype) virtual void unpack(thetype &t);
    class mpi_ser_unpacker : public sstmac::ser_unpacker
    {
    protected:

      template<class T>
        void
        do_unpack(T& t);

      mpi_ser_unpacker(char* b) :
        buffer_(b), at_(0)
      {

      }

    public:
      typedef boost::intrusive_ptr<mpi_ser_unpacker> ptr;

      static ptr
      construct(char* buf){
        return ptr(new mpi_ser_unpacker(buf));
      }

      virtual
      ~mpi_ser_unpacker() { }


      MPIUNPACKER_DECL_UNPACK(int)
      MPIUNPACKER_DECL_UNPACK(float)
      MPIUNPACKER_DECL_UNPACK(short)
      MPIUNPACKER_DECL_UNPACK(double)
      MPIUNPACKER_DECL_UNPACK(long int)
      MPIUNPACKER_DECL_UNPACK(long long int)
      MPIUNPACKER_DECL_UNPACK(unsigned int)
      MPIUNPACKER_DECL_UNPACK(long unsigned int)
      MPIUNPACKER_DECL_UNPACK(long long unsigned int)
      MPIUNPACKER_DECL_UNPACK(char)
      MPIUNPACKER_DECL_UNPACK(bool)
      MPIUNPACKER_DECL_UNPACK(std::string)
      MPIUNPACKER_DECL_UNPACK(timestamp)

      size_t
      get_bytes_deserialized() const
      {
        return at_;
      }

      virtual void unpack_array(void* arr, size_t size);

    protected:
      char* buffer_;
      size_t at_;
    };
#undef MPIPACKER_DECL_PACK

#define MPISIZER_DECL_PACK(thetype) virtual void size(thetype t);
    class mpi_ser_sizer : public sstmac::ser_sizer
    {
    protected:
      template<class T>
        void
        do_pack(T t);

      template<class T>
        void
        do_unpack(T& t);

      mpi_ser_sizer()
      {

      }

    public:
      typedef boost::intrusive_ptr<mpi_ser_sizer> ptr;

      static ptr
      construct(){
        return ptr(new mpi_ser_sizer());
      }

      virtual
      ~mpi_ser_sizer() { }


      MPISIZER_DECL_PACK(int)
      MPISIZER_DECL_PACK(float)
      MPISIZER_DECL_PACK(short)
      MPISIZER_DECL_PACK(double)
      MPISIZER_DECL_PACK(long int)
      MPISIZER_DECL_PACK(long long int)
      MPISIZER_DECL_PACK(unsigned int)
      MPISIZER_DECL_PACK(long unsigned int)
      MPISIZER_DECL_PACK(long long unsigned int)
      MPISIZER_DECL_PACK(char)
      MPISIZER_DECL_PACK(bool)
      MPISIZER_DECL_PACK(std::string)
      MPISIZER_DECL_PACK(timestamp)

    protected:

    };
#undef MPIPACKER_DECL_PACK
  }
}
#endif
