/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_MPIPARALLEL_IPC_EVENT_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_MPIPARALLEL_IPC_EVENT_H_INCLUDED

#include <sstmac/common/sstevent.h>
//#include <sstmac/common/messages/serializable.h>

namespace sstmac
{
  namespace native
  {
    // Events passed from interprocess links to logical process queues (and vice versa)
    // Needed for routing to switches that don't belong in an LP.
    class null_event : public event,
    public serializable_type<null_event>
    {
      ImplementSerializable(null_event)

    protected:

      null_event(const timestamp &t);
    public:
      typedef boost::intrusive_ptr<null_event> ptr;

      static ptr
      construct(const timestamp &t);

      // nodeaddress::ptr get_src_switch();

      virtual std::string
      to_string() const {
        return "null_event";
      }

      virtual void
      serialize_order(sst_serializer* ser);

      virtual void
      cancel() { }

      virtual bool
      is_cancelled() const {
        return false;
      }

      virtual void execute() { }


    };

  }
}

#endif
