/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_MPIPARALLEL_PAR_EVENTMAP_H_
#define SSTMAC_BACKENDS_NATIVE_MPIPARALLEL_PAR_EVENTMAP_H_

#include <sstmac/common/eventmanager.h>
#include <sstmac/common/sstevent.h>
#include <sstmac/common/timestamp.h>
#include <sstmac/backends/native/sim_partition.h>
#include <sstmac/hardware/network/interconnect/switchinterconnect.h>
#include <list>
#include <mpi.h>
#include <sstmac/backends/native/mpiparallel/par_q.h>

namespace sstmac
{
  namespace native
  {

    class par_eventmap : public eventmanager
    {
    protected:
      par_eventmap(const timestamp &tv, int lpid, int numlps,
          partition::ptr part, const timestamp &iplat);

      enum STATE
      {
        INIT, // either no events have been scheduled for run hasn't been called yet
        LOC_OK, // has events, good to go
        LOC_NOK, // has events but can't progress
        //   LOC_NOK_WAIT,  // has event but can't progress because of me (local lp)
        NOLOC, // doesn't have any local events
        FINISHED
      // the simulation is finished for all intents and purposes
      };
      boost::unordered_map<int, STATE> state_;
      boost::unordered_map<int, timestamp> last_update_time_;

      boost::unordered_map<int, timestamp> last_sent_time_;

      /* enum COMM_TAGS
       {
       STATE_UPDATE = 0, MESSAGE = 1, NEED_UPDATE = 3, LAST_COMM_STATE = 4
       };

       int state_buffer_;
       char* msg_buffer_;
       long int update_buffer_;
       char* sendbuf_;

       MPI_Request statereq_;
       MPI_Request messagereq_;
       MPI_Request updatereq_;

       int waiting_on_update_;

       timestamp last_update_sent_;

       size_t current_recv_size_;*/

      MPI_Op dlop_;



      boost::unordered_map<int, par_q> queues_;

      typedef std::multimap<timestamp, event::ptr> queue_t;
      queue_t localevents_;

      int lpid_;
      int numlps_;
      timestamp iplat_;

      std::list<ptr_type::ptr> finishers_;

      hw::switchinterconnect::ptr network_;
      partition::ptr part_;

    public:

      typedef boost::intrusive_ptr<par_eventmap> ptr;
      /// Hello.
      static par_eventmap::ptr
      construct(const timestamp &tv, int lpid, int numlps, partition::ptr part,
          const timestamp &iplat)
      {
        return ptr(new par_eventmap(tv, lpid, numlps, part, iplat));
      }

      virtual
      ~par_eventmap()
      {

      }

      virtual std::string
      to_string() const
      {
        return "par_eventmap";
      }

      //virtual void
      //clear(const timestamp &zero_time = timestamp(0));
      virtual void
      clear(const timestamp &zero_time = timestamp(0))
      {
        sst_throw(unimplementederror, "par_eventmap::clear()");
      }

      virtual void
      run();

      //virtual void
      //cancel_all_messages(const eventschedulerPtr& mod);
      virtual void
      cancel_all_messages(const eventschedulerPtr& mod)
      {
        sst_throw(unimplementederror, "par_eventmap::cancel_all_messages()");
      }

      //// ----- done
      virtual void
      ipc_schedule(nodeaddress::ptr goingto, event::ptr &ev);

      virtual int
      get_lpid() const
      {
        return lpid_;
      }

      virtual partition::ptr
      get_partition() const
      {
        return part_;
      }

      virtual void
      end_gracefully();

      virtual void
      step();

      virtual void
      schedule(const timestamp& start_time, const event::ptr& event);

      virtual void
      register_finish(ptr_type::ptr p);

      void
      set_switches(const hw::switchinterconnect::ptr& fab);

      //// --------------
    protected:
      void
      finish();

      void
      change_state(STATE s);

      void
      check_for_updates();

      void
      wait_for_updates();

      void
      send_update(int lp, timestamp theirs);

      void
      send_update2(int lp, timestamp theirs);

      void
      do_one_event();

      void
      update_state(STATE s, int lp);

      void
      dump_info();

      bool
      check_valid(timestamp min_t);

      void
      parse_message(par_message* p);

      void
      check_deadlock();

    };

  }
}

#endif /* PAR_EVENTMAP_H_ */
