/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_PARALLELPROCESS_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_PARALLELPROCESS_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/backends/native/sim_partition.h>
#include <sstmac/backends/native/mpiparallel/par_eventmap.h>

namespace sstmac
{
  namespace native
  {
    class parallelprocess : public ptr_type
    {
    protected:
      parallelprocess(const partition::ptr &part, int id, int num);

      partition::ptr part_;

      par_eventmap::ptr evman_;

      int myid_;
      int num_;

      static debug dbg_;

    public:
      typedef boost::intrusive_ptr<parallelprocess> ptr;

      static ptr
      construct(const sim_parameters::ptr &params, const partition::ptr &part,
          int id, int num)
      {
        return ptr(new parallelprocess(part, id, num));
      }
      virtual
      ~parallelprocess();

      virtual std::string
      to_string() const
      {
        return "parallelprocess";
      }

      timestamp
      run(timestamp until);

      eventmanager::ptr
      get_eventmanager() const
      {
        return evman_;
      }
    };

  }
}
#endif /* PARALLELPROCESS_H_ */
