/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#include <sstmac/backends/native/sim_partition.h>
#include <sstmac/common/fileio.h>


namespace sstmac
{

  namespace native
  {

    debug partition::dbg_;

    partition::partition(const std::string &partfile, int size)
    {
      if (!dbg_.is_real())
      {
        dbg_ = debug("<debug> partition | parallel");
        logger::register_logger(&dbg_);
      }

      int i;
      for (i = 0; i < size; i++)
      {
        std::list<int> part;
        std::pair<int, std::list<int> > key;
        key.first = i;
        key.second = part;
        insert(key);
      }

    }

    partition::ptr
    partition::construct(const std::string &partfile, int num)
    {
      ptr ret = ptr(new partition(partfile, num));
      ret->read_partition(partfile);
      return ret;
    }

    std::list<int>*
    partition::get_partition(int lpid)
    {
      if(lpid >= size()){
        sst_throw(ssterror, "partition::get_partition - lpid %d larger than size %d", lpid, size());
      }
      std::list<int>* retval = &(find(lpid)->second);
      return retval;
    }

    int
    partition::get_parent(int switch_id)
    {
      return parent_map_.find(switch_id)->second;
    }

    void
    partition::read_partition(const std::string &partfilename)
    {
      std::string thef = trim_str(partfilename);
      SSTMAC_DEBUG << "partition: reading partition file " << thef
            << "\n";
      std::ifstream is;
      SSTFileIO::open_file(is, thef);

      // Reads in METIS partition files.
      if (is.is_open())
      {
        int i = 0;
        std::string line;
        std::getline(is, line);
        while (is.good())
        {
          trim_str(line);
          // Insert switch into partition.
          int reg = atoi(line.c_str());
          get_partition(reg)->push_back(i);

          SSTMAC_DEBUG << "adding " << i << " to partition " << reg << "\n";

          // Add switch/parent pair to parent_map for convenience.
          std::pair<int, int> parent_pair;
          parent_pair.first = i;
          parent_pair.second = reg;
          parent_map_.insert(parent_pair);
          i++;
          std::getline(is, line);
        }

        is.close();
      }
      else
      {
        sst_throw(valueerror,
            "parallel::partition::read_partition - could not open file %s \n",
            partfilename.c_str());
      }

    }
  }

}
