/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_DRIVER_UTIL_H
#define SSTMAC_DRIVER_UTIL_H

#include <sstmac/common/debug.h>
#include <sstmac/common/errors.h>

#include <sstmac/software/services/launch/indexing/indexstrategy.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/software/services/launch/allocation/allocationstrategy.h>
#include <sstmac/hardware/network/interconnect/interconnect.h>
#include <sstmac/hardware/network/topology/topology.h>

#include <sstmac/common/sim_parameters.h>

#include <sstmac/common/sstmac-config.h>
#include <sstmac/common/timestamp.h>
#include <sstmac/common/units.h>
#include <ostream>
#include <boost/foreach.hpp>

#include <sstmac/common/vis/vis.h>

namespace sstmac
{

  namespace sw
  {
    class launcher;
    class launchinfo;
  }

  double
  get_positive_float(const char *value, bool &errorflag);

  int
  get_int(const char *value, bool &errorflag);

  int
  get_positive_int(const char *value, bool &errorflag);

  void
  get_intvec(const char *value, bool &errorflag, std::vector<int>& retval);

  std::vector<int>
  get_intvec(const char *value, bool &errorflag);

}
#endif
