/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/common/messages/eventdata.h>
#include <sstmac/common/errors.h>
#include <sstmac/software/process/thread.h>

namespace sstmac {

    const char eventdata::INST_FLOP[] = { "flop" };
    const char eventdata::INST_MEMOP[] = { "memop" };
    const char eventdata::INST_MEMOP_LOCAL[] = { "memop_local" };
    const char eventdata::INST_MEMOP_RAND[] = { "memop_rand" };

  eventdata::eventdata(const std::string &libn)
      : sst_message(sst_message::COMPUTE),
        library_message(libn), 
        core_(sw::thread::no_core_affinity),
        event_time_(0)
  {
  }

  eventdata::eventdata()
  {
  }

  eventdata::ptr 
  eventdata::construct(const std::string& libn)
  {
    eventdata::ptr retval(new eventdata(libn));
    return retval;
  }

  unsigned long long int
  eventdata::get_event_value(const std::string& key) const
  {
    data_map::const_iterator it = event_data_.find(key);
    if (it == event_data_.end())
        sst_throw(valueerror, "eventdata::get_event_value: no value for event %s found",
                  key.c_str());
    else
        return it->second;
  }

  unsigned long long int
  eventdata::get_optional_event_value(const std::string& key) const
  {
      data_map::const_iterator it = event_data_.find(key);
      if (it == event_data_.end())
          return 0;
      else
          return it->second;
  }

  void
  eventdata::set_event_value(const std::string& key, unsigned long long val)
  {
      event_data_[key] = val;
  }


  bool
  eventdata::has_event_value(const std::string& key) const
  {
    return event_data_.find(key) != event_data_.end();
  }

  void
  eventdata::set_event_time(const timestamp &t)
  {
      event_time_ = t;
  }

  const timestamp&
  eventdata::get_event_time() const
  {
      return event_time_;
  }

  void
  eventdata::set_core(int core)
  {
      core_ = core;
  }

  int
  eventdata::get_core() const
  {
      return core_;
  }

} // end of namespace sstmac
