/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_HARDWARE_NETWORK_MULTI_PREFIX_PARAMS_H_
#define SSTMAC_HARDWARE_NETWORK_MULTI_PREFIX_PARAMS_H_

#include <sstmac/common/sim_parameters.h>

namespace sstmac
{
  namespace hw
  {

    class prefix_params : public sim_parameters
    {

    protected:
      std::string prefix_;
      sim_parameters::ptr parent_;

      prefix_params(sim_parameters::ptr parent, std::string pre) :
        prefix_(pre), parent_(parent)
      {

      }

    public:
      typedef boost::intrusive_ptr<prefix_params> ptr;

      static ptr
      construct(sim_parameters::ptr parent, std::string pre)
      {
        return ptr(new prefix_params(parent, pre));
      }

      sim_parameters::ptr
      subspace_clone()
      {
        sst_throw(unimplementederror, "prefix params does not work with namespaces");
      }

      virtual sim_parameters::ptr
      clone()
      {
        return construct(parent_, prefix_);
      }

      virtual std::string
      to_string() const
      {
        return "prefix_params";
      }

      void
      set_prefix(std::string str)
      {
        prefix_ = str;
      }

      virtual void
      remove_param(const std::string & key)
      {
        std::string whole = prefix_ + key;
        parent_->remove_param(whole);
      }

      virtual void
      print_params(std::ostream &os = std::cerr) const
      {
        parent_->print_params(os);
      }

      virtual std::string
      get_param(const std::string& key) const
      {
        std::string whole = prefix_ + key;
        return parent_->get_param(whole);
      }

      virtual void
      add_param(const std::string& key, const std::string& val)
      {
        std::string whole = prefix_ + key;
        parent_->add_param(whole, val);
      }

      virtual void
      add_param_override(const std::string& key, const std::string& val)
      {
        std::string whole = prefix_ + key;
        parent_->add_param_override(whole, val);
      }

      virtual void
      combine_into(sim_parameters::ptr sp)
      {
        parent_->combine_into(sp);
      }

      virtual bool
      has_param(const std::string& key) const
      {
        std::string whole = prefix_ + key;
        return parent_->has_param(whole);
      }
    };

  }
}

#endif
