


/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */


#include <sstmac/common/ptr_type.h>
#include <sstmac/common/config.h>

namespace sstmac {

  ptr_type::~ptr_type()
  {
  }

  ptr_type::ptr_type() : references(0)
  {
  }

  long
  ptr_type::nreferences() const
  {
      return references;
  }

}



#include <sstmac/hardware/node/node.h>
#include <sstmac/common/stats/stats_common.h>
#include <sstmac/backends/native/eventmap.h>


#if DEBUG_TRACK_PTR_REFCOUNT
void* ptr_to_track = 0;
#endif

namespace boost {

#if DEBUG_TRACK_CLS_REFCOUNT || DEBUG_TRACK_PTR_REFCOUNT
static int nothing = 0;
#endif

void intrusive_ptr_add_ref(sstmac::ptr_type * p)
{

   ++(p->references);
#if DEBUG_TRACK_PTR_REFCOUNT
    if (p == ptr_to_track){
        ++nothing;
    }
#endif

#if DEBUG_TRACK_CLS_REFCOUNT
   DEBUG_TRACK_CLS_NAME* app = dynamic_cast<DEBUG_TRACK_CLS_NAME*>(p);
   if (app){
     ++nothing;
   }
#endif
}

void intrusive_ptr_release(sstmac::ptr_type * p)
{
#if DEBUG_TRACK_PTR_REFCOUNT
    if (p == ptr_to_track){
        ++nothing;
    }
#endif

#if DEBUG_TRACK_CLS_REFCOUNT
    DEBUG_TRACK_CLS_NAME* app = dynamic_cast<DEBUG_TRACK_CLS_NAME*>(p);
    if (app){
        ++nothing;
    }
#endif

  // decrement reference count, and delete object when reference count reaches 0
  if (--(p->references) == 0){
      delete p;
  }
}

void intrusive_ptr_add_ref(const sstmac::ptr_type * p)
{
  ++(p->references);

#if DEBUG_TRACK_PTR_REFCOUNT
    if (p == ptr_to_track){
        ++nothing;
    }
#endif

#if DEBUG_TRACK_CLS_REFCOUNT
    const DEBUG_TRACK_CLS_NAME* app = dynamic_cast<const DEBUG_TRACK_CLS_NAME*>(p);
    if (app){
        ++nothing;
    }
#endif
}

void intrusive_ptr_release(const sstmac::ptr_type * p)
{
#if DEBUG_TRACK_PTR_REFCOUNT
    if (p == ptr_to_track){
        ++nothing;
    }
#endif

#if DEBUG_TRACK_CLS_REFCOUNT
    const DEBUG_TRACK_CLS_NAME* app = dynamic_cast<const DEBUG_TRACK_CLS_NAME*>(p);
    if (app){
        ++nothing;
    }
#endif

   if (--(p->references) == 0){
       delete p;
   }
}

}
