/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstream>

#include <sstmac/common/sstevent.h>
#include <sstmac/common/eventmodule.h>

DeclareBaseClassSerializable(sstmac::event);
DeclareSerializable(sstmac::simple_event);
DeclareSerializable(sstmac::generic_event);

namespace sstmac
{

  event::event() :
    next(0), module_(0), time_(0)
  {

  }

  event::~event()
  {
  }

  const timestamp&
  event::time() const
  {
    return time_;
  }

  eventmodule::ptr
  event::get_module() const
  {
    return module_;
  }

  void
  event::set_time(const timestamp &t)
  {
    time_ = t;
  }

  void
  event::set_module(const eventmodule::ptr& mod)
  {
    module_ = mod;
  }

  std::string
  simple_event::to_string() const
  {
    std::stringstream ss;
    if (canceled_)
    {
      ss << "canceled";
    }
    else
    {
      ss << "sst_event(handler=" << (handler_ ? handler_->to_string() : "null")
          << ",  msg = " << (msg_to_deliver_ ? msg_to_deliver_->to_string()
          : "null") << ")";
    }
    return ss.str();
  }

  sst_message::ptr
  simple_event::get_message()
  {
    return msg_to_deliver_;
  }

  void
  simple_event::cancel()
  {
    if (msg_to_deliver_)
    {
      if (msg_to_deliver_->cancellable())
      {
        canceled_ = true;
      }
    }
  }

  void
  simple_event::execute()
  {
    if (!canceled_)
    {
      if (msg_to_deliver_)
        msg_to_deliver_->set_scheduled(false);
      handler_->handle(msg_to_deliver_);
    }
  }

  simple_event::simple_event(const sst_message::ptr &msg,
      const eventhandler::ptr &hand) :
    msg_to_deliver_(msg), handler_(hand), canceled_(false)
  {
  }

  simple_event::~simple_event()
  {
  }

  simple_event::ptr
  simple_event::construct(const sst_message::ptr &msg,
      const eventhandler::ptr &hand)
  {
    simple_event::ptr ret(new simple_event(msg, hand));
    return ret;
  }



} // end of namespace sstmac
