/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/stats/event_trace.h>

namespace sstmac
{

  void
  event_trace::simulation_finished(timestamp end)
  {
    if (is_active() && check_open())
    {
      myfile.close();
    }
  }

  event_trace::ptr
  event_trace::construct(const std::string& namesig, const std::string& filename)
  {
      timestamp start = sstmac_env::params->get_optional_time_param("event_trace_start", timestamp(0));
      //set the default max time to whatever fits into an int64
      double max_time = std::numeric_limits<int64_t>::max() * 1e-12;
      timestamp stop = sstmac_env::params->get_optional_time_param("event_trace_stop", timestamp(max_time));
      return new event_trace(namesig, filename, (long)start.ticks_int64(), (long)stop.ticks_int64());
  }

  void
  event_trace::collect(int event_typeid,
      const std::string& name,
      const nodeaddress::ptr& node,
      int aff, long threadid,
      int aid, int tid,
      long ticks_begin,
      long num_ticks)
  {
    if (is_active() && check_open() && ticks_begin >= start_ && (ticks_begin + num_ticks) <= stop_)
    {
      myfile << ticks_begin << ", " << num_ticks << ", " << name << ", "
          << threadid << ", " << event_typeid << ", " << node->unique_id() << ", " << aff << "\n";
    }
  }

}
