/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_STATS_EVENT_TRACE_H_
#define SSTMAC_COMMON_STATS_EVENT_TRACE_H_

#include <sstmac/common/stats/stat_logger.h>
#include <sstmac/software/process/key.h>
#include <sstmac/common/nodeaddress.h>
#include <sstmac/common/sstmac_env.h>

namespace sstmac
{

  class event_trace : public stat_logger
  {

  public:
    typedef boost::intrusive_ptr<event_trace> ptr;

  protected:

    long start_;
    long stop_;
    event_trace(const std::string& namesig, const std::string& filename, long start, long stop) :
      stat_logger(namesig, filename), start_(start), stop_(stop)
    {
    }

  public:
    static ptr
    construct(const std::string& namesig, const std::string& filename);


    virtual void
    simulation_finished(timestamp end);

    void
    collect(int key_typeid,
        const std::string& name,
        const nodeaddress::ptr& addr,
        int aff, long threadid,
        int aid, int tid,
        long ticks_begin,
        long num_ticks);

    virtual
    ~event_trace()
    {
    }

  };

}

#endif /* EVENT_TRACE_H_ */
