#include <sstmac/common/stats/stats_common.h>
#include <sstmac/common/lodepng.h>
#include <algorithm>

namespace sstmac
{

  void
  stat_spyplot::add_one(long source, long dest)
  {
    if (is_active())
    {
      /**
       if (vals_.find(source) == vals_.end())
       {
       vals_[source] = std::map();<long, long>
       for (long i = 0; i < num_; i++)
       {
       vals_[source][i] = 0;
       }
       }
       */
      //We don't need to fill. This by default will insert a zero element.
      vals_[source][dest] += 1;
    }
  }

  void
  stat_spyplot::add(long source, long dest, long num)
  {
    if (is_active())
    {
      /**
       if (vals_.find(source) == vals_.end())
       {
       vals_[source] = std::map();<long, long>
       for (long i = 0; i < num_; i++)
       {
       vals_[source][i] = 0;
       }
       }
       */
      //We don't need to fill. This by default will insert a zero element.
      vals_[source][dest] += num;
    }
  }

  void
  stat_spyplot::simulation_finished(timestamp end)
  {
    if (is_active() && check_open())
    {
      std::list<long> nodes;

      {
        nested_type<long, long, long>::map_iterator it, end = vals_.end();
        for (it = vals_.begin(); it != end; it++)
        {
          nodes.push_back(it->first);
        }
      }

      nodes.sort();

      std::list<long>::const_iterator it = nodes.begin(), end = nodes.end();
      for (; it != end; ++it)
      {
        std::map<long, long>& submap = vals_[*it];
        std::list<long>::const_iterator it2 = nodes.begin(), end2 = nodes.end();
        long datapoint = submap[*it2];
        myfile << datapoint;
        ++it2;
        for (; it2 != end2; ++it2)
        {
          long datapoint = submap[*it2];
          myfile << "," << datapoint;
        }
        myfile << "\n";
      }

      myfile.close();
    }
  }

  /**
   *
   * @param val - a number between 0 and 1
   * @return a 24-bit pixel
   */
  int
  get_pixel(double val)
  {
    /* if(val < 0.333333333){
     return 255 * 3 * val;
     }else if(val < 0.666666666){
     return ((int)(255 * 3 * (val - 0.333333333))) << 8;
     }else{
     return ((int)(255 * 3 * (val - 0.666666666))) << 16;
     }*/
    //return (1 << 23) * val;
    //return std::min((1<<23), (int)std::pow(10, 6.2 * val));
    //return ((int)(255 * (1.0 - val))) << 16;

    double quant = 1.0 / 3.0;
    int r = 0, g = 0, b = 0;
    /*if (val < quant) //blue
     {
     b = 255 * (val) / quant;
     }
     else if (val < 2 * quant) //blue-green
     {
     b = 255 * (1.0 - (val - quant)) / quant;
     g = ((int) (255 * ((val - quant)) / quant));
     }
     else if (val < 3 * quant)
     { //green
     g = 255 * (val) / quant;
     }
     else if (val < 4 * quant)
     {
     g = 255 * (1.0 - (val - 3 * quant)) / quant;
     r = ((int) (255 * ((val - 3 * quant)) / quant));
     }
     else
     {
     r = 255 * (val) / quant;
     }*/

    b = std::max(
        0,
        (int) ((val < quant) ? (255.0 * (val) / quant) : (255.0 * (quant - (val
            - quant)) / quant)));
    if (val < quant)
    {
      r = 0;
      g = 0;
    }
    else
    {
      g = std::max(
          0,
          (int) ((val < 2 * quant) ? (255.0 * (val - quant) / quant) : (255.0
              * (3 * quant - (val)) / quant)));

      if (val < 2 * quant)
      {
        r = 0;
      }
      else
      {
        r = (255 * (val - 2 * quant) / quant);
      }
    }

    return (r) | (g << 8) | (b << 16);
  }

  void
  stat_spyplot_png::simulation_finished(timestamp end)
  {
    if (is_active() && !disabled_)
    {

      long maxval = 0;
      long maxsrc = 0;
      long maxdest = 0;
      std::list<long> nodes;

      {
        nested_type<long, long, long>::map_iterator it, end = vals_.end();
        for (it = vals_.begin(); it != end; it++)
        {
          nodes.push_back(it->first);
          if (it->first > maxsrc)
          {
            maxsrc = it->first;
          }

          std::map<long, long>::const_iterator it2 = it->second.begin(), end2 =
              it->second.end();
          for (; it2 != end2; ++it2)
          {
            if (it2->first > maxdest)
            {
              maxdest = it2->first;
            }

            if (it2->second > maxval)
              maxval = it2->second;
          }
        }
      }

      nodes.sort();

      long pixels_per_point = 10;

      long dim = fill_ ? (std::max(maxsrc, maxdest) + 1) : vals_.size();

      long width = (dim + 2) * pixels_per_point;
      long height = dim * pixels_per_point;

      long imagesize = (width) * height * 4;
      std::vector<unsigned char> image;
      image.resize(imagesize);

      long cnt = 0;
      long heightcnt = height;

      if(vals_.size() > 1024){
        std::cout << "WARNING: disabling spyplot png " << fname_ << " because it would be too large\n";
        return;
      }

      for (long src = 0; src <= std::max(maxsrc, maxdest); src++)
      {
       // std::cout << "stat_spyplot_png(" << fname_ << "): considering src " << src << "...";
        if (fill_ || vals_.find(src) != vals_.end())
        {
        //  std::cout << "yep, doing it \n";
          for (int j = 0; j < pixels_per_point; j++)
          {
            if (fill_)
            {
              for (long dest = 0; dest <= std::max(maxsrc, maxdest); dest++)
              {

                int datapoint = get_pixel(
                    (double) vals_[src][dest] / (double) maxval);
                for (int i = 0; i < pixels_per_point; i++)
                {
                  image[cnt++] = datapoint & 0xFF;
                  image[cnt++] = (datapoint >> 8) & 0xFF;
                  image[cnt++] = (datapoint >> 16) & 0xFF;
                  image[cnt++] = 255; //alpha
                }
              }
            }
            else
            {
              std::map<long, long>& submap = vals_[src];
              std::list<long>::const_iterator it2 = nodes.begin(), end2 =
                  nodes.end();

              for (; it2 != end2; it2++)
              {
                int datapoint = get_pixel(
                    (double) submap[*it2] / (double) maxval);
                for (int i = 0; i < pixels_per_point; i++)
                {
                  image[cnt++] = datapoint & 0xFF;
                  image[cnt++] = (datapoint >> 8) & 0xFF;
                  image[cnt++] = (datapoint >> 16) & 0xFF;
                  image[cnt++] = 255; //alpha
                }
              }

            }

            //a clear space
            for (int i = 0; i < pixels_per_point; i++)
            {
              image[cnt++] = 0;
              image[cnt++] = 0;
              image[cnt++] = 0;
              image[cnt++] = 0; //alpha
            }

            int legend = get_pixel((double) heightcnt / (double) height);
            // std::cout << "legend is " << legend << "\n";
            for (int i = 0; i < pixels_per_point; i++)
            {
              image[cnt++] = legend & 0xFF;
              image[cnt++] = (legend >> 8) & 0xFF;
              image[cnt++] = (legend >> 16) & 0xFF;
              image[cnt++] = 255; //alpha
            }

            heightcnt--;

          }
        }
      }

      std::string fullpath = stats_dir_ + fname_;

      //Encode the image
      unsigned error = lodepng::encode(fullpath, image, width, height);

      //if there's an error, display it
      if (error)
        std::cout << "stat_spyplot_png: PNG encoder error " << error << ": "
            << lodepng_error_text(error) << std::endl;

    }
  }

} //end namespace

