#include <sstmac/common/test/params.h>
#include <sstmac/keywords/keyword_registration.h>

namespace sstmac
{

  test_parameters::test_parameters()
  {
  }

  test_parameters::test_parameters(const maptype &params)
    : params_(params)
  {
  }

  std::string
  test_parameters::get_param(const std::string& key) const
  {
    SSTMAC_DEBUG << "file_parameters: getting param " << key << "\n";
    maptype::const_iterator it = params_.find(key);
    if (it == params_.end())
    {
      sst_throw(sstmac::valueerror, "file_parameters: could not find parameter %s", key.c_str());
    }
    return it->second;
  }

  std::string&
  test_parameters::operator [](const std::string& key)
  {
    return params_[key];
  }

  bool
  test_parameters::has_param(const std::string& key) const
  {
    maptype::const_iterator it = params_.find(key);
    return (it != params_.end());
  }

  void
  test_parameters::add_param(const std::string& key, const std::string& val)
  {
    if (params_.find(key) != params_.end())
    {
      sst_throw( sstmac::valueerror,
          "file_parameters::add_param - key already in params: %s", key.c_str());
    }
    else
    {
      KeywordRegistration::validate_keyword(key, val);
      params_[key] = val;
    }
  }

  void
  test_parameters::add_param_override(const std::string& key,
      const std::string& val)
  {
    KeywordRegistration::validate_keyword(key, val);
    params_[key] = val;

  }

  void
  test_parameters::combine_into(sim_parameters::ptr sp)
  {
    BOOST_FOREACH(maptype::value_type it, params_)
          {
            sp->add_param_override(it.first, it.second);
          }
  }

  sim_parameters::ptr
  test_parameters::clone()
  {
    return new test_parameters(params_);
  }

  sim_parameters::ptr
  test_parameters::subspace_clone()
  {
    return new test_parameters;
  }

  void
  test_parameters::remove_param(const std::string & key)
  {
    params_.erase(key);
  }

  void
  test_parameters::print_params(std::ostream& os) const
  {
    sim_parameters::print_params(params_, os);
  }

  unittest_params::unittest_params()
    : boost::intrusive_ptr<test_parameters>(test_parameters::construct())
  {
  }

  std::string&
  unittest_params::operator[](const std::string& key)
  {
    test_parameters* ptr = get();
    return (*ptr)[key];
  }


}
