/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_VIS_VIS_VTK_H_INCLUDED
#define SSTMAC_COMMON_VIS_VIS_VTK_H_INCLUDED

#include <sstmac/common/vis/vis.h>

#include <vtkRenderWindow.h>
#include <vtkCamera.h>
#include <vtkActor.h>
#include <vtkRenderer.h>
#include <vtkMapper.h>
#include <vtkPolyDataAlgorithm.h>
#include <vtkTextActor.h>

namespace sstmac
{
  namespace vis
  {

    class vtk_obj : public vis_obj
    {
    public:
      typedef boost::intrusive_ptr<vtk_obj> ptr;

      static ptr
      construct(vtkPolyDataAlgorithm* al, vtkActor* ac, vtkMapper* ma)
      {
        return ptr(new vtk_obj(al, ac, ma));
      }

      virtual void
      set_activity(double val);

      virtual void
      set_color(VIS_COLOR col);

      vtkPolyDataAlgorithm*
      get_alg()
      {
        return alg_;
      }

      vtkActor*
      get_act()
      {
        return act_;
      }

    protected:

      vtkPolyDataAlgorithm* alg_;
      vtkActor* act_;
      vtkMapper* map_;

      void
      get_heatmap_color(double val, double &r, double &g, double &b);

      vtk_obj(vtkPolyDataAlgorithm* al, vtkActor* ac, vtkMapper* ma);

      virtual
      ~vtk_obj()
      {
        alg_->Delete();
        act_->Delete();
        map_->Delete();
      }

      virtual std::string
      to_string() const
      {
        return "vtk_obj";
      }
    };

    class vtk_engine : public vis_engine
    {
    public:
      typedef boost::intrusive_ptr<vtk_engine> ptr;

      static ptr
      construct(const sim_parameters::ptr& params)
      {
        return ptr(new vtk_engine());
      }

      virtual void
      init();

      virtual void
      draw();

      virtual void
      update(timestamp now);

      virtual void
      update_label(timestamp now);

      virtual vis_obj::ptr
      create_cube(double x, double y, double z, double len);

      virtual vis_obj::ptr
      create_cylinder(double x, double y, double z, double rad, double len,
          int dir);

      void
      write_to_file(const std::list<vis_obj::ptr> &objs, std::string fname);

      virtual void
      display_label(std::string text);

      void
      init_factory_params(const sim_parameters::ptr &params);

    protected:

      vtk_engine();

      virtual
      ~vtk_engine()
      {
        rend_->Delete();
        renWin_->Delete();
      }

      virtual std::string
      to_string() const
      {
        return "vtk_engine";
      }

      vtkRenderer *rend_;
      vtkRenderWindow *renWin_;

      std::list<vtk_obj::ptr> objs_;

      bool hasTurned_;

      timestamp update_;
      timestamp lastupdate_;
      timestamp start_;

      vtkTextActor* label_;
      std::string labeltext_;
    };

  }
}

#endif
