#ifndef CONNECTION_H
#define CONNECTION_H

#include <sstmac/common/eventhandler.h>

namespace sstmac
{
  namespace hw
  {

    class connectable :
        virtual public eventhandler
    {
    public:
      static const int any_port = -1;

      typedef enum
      {
        injection, ejection, network_link, global_link
      } connection_type_t;

#define connect_str_case(x) case x: return #x
      static const char*
      str(connection_type_t ty)
      {
        switch (ty)
        {
        connect_str_case(injection)
          ;
        connect_str_case(ejection)
          ;
        connect_str_case(network_link)
          ;
        connect_str_case(global_link)
          ;
        }
      }

#define connectable_type_invalid(ty) \
            sst_throw(valueerror, "invalid connectable type %s", connectable::str(ty))

      typedef boost::intrusive_ptr<connectable> ptr;

      virtual void
      connect(connection_type_t ty, const connectable::ptr& mod)
      {
        connect(any_port, ty, mod);
      }

      virtual void
      connect(long port, connection_type_t ty, const connectable::ptr& mod)
      {
        connect(port, ty, mod, 1.0, 1);
      }

      virtual void
      connect(const nodeaddress::ptr& addr, int port, connection_type_t ty,
          const connectable::ptr& mod)
      {
        connect(addr->unique_id(), port, ty, mod, 1.0, 1);
      }

      virtual void
      connect(const nodeaddress::ptr& addr, connection_type_t ty,
          const connectable::ptr& mod)
      {
        connect(addr->unique_id(), ty, mod, 1.0, 1);
      }

      virtual void
      connect(int dim, int port, connection_type_t ty,
          const connectable::ptr& mod, double weight, int red)
      {
        sst_throw(unimplementederror, "connectable does not implement weighted/redundant connect");
      }

      virtual void
      connect(long port, connection_type_t ty, const connectable::ptr& mod,
          double weight, int red)
      {
        sst_throw(unimplementederror, "connectable does not implement weighted/redundant connect");
      }



    };

  }
}

#endif // CONNECTION_H
