/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_HARDWARE_FAILURE_CONFIG_FAILURE_CONFIG_H_INCLUDED
#define SSTMAC_HARDWARE_FAILURE_CONFIG_FAILURE_CONFIG_H_INCLUDED


#include <sstmac/hardware/failure/failable.h>
#include <sstmac/backends/native/nodelist.h>

namespace sstmac
{
  namespace hw {
    class interconnect;
    typedef boost::intrusive_ptr<interconnect> interconnectPtr;
    class node;
    typedef boost::intrusive_ptr<node> nodePtr;

    class failure_config : public ptr_type {

    public:
      typedef boost::intrusive_ptr<failure_config> ptr;

      virtual ~failure_config() { }

    protected:
      failure_config(sim_parameters::ptr params);

      enum MODEL_TYPE {RATE, TRACE};

      MODEL_TYPE type_;
      sim_parameters::ptr params_;
      int nodes_per_ps_;
      std::string trace_name_;

    public:

      static ptr
      construct(sim_parameters::ptr params){
        return ptr(new failure_config(params));
      }

      void
      config_nodes(const std::vector<nodePtr>& list, eventmanager::ptr ev_man);

      void
      config_network(const interconnectPtr& net);

      virtual std::string
      to_string() const {
        return "failure_config";
      }

    };

  }

}
#endif
