/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/fastflower/fastflow.h>
#include <sstmac/hardware/fastflower/fastflow_message.h>

namespace sstmac
{
  namespace hw
  {
    long fastflow::FLOWIDCNT = 0;

    fastflow::fastflow(const sst_message::ptr &orig, long bytes) :
      unit_bw_(-1), real_head_bw_(0), flowbytes_(bytes)
    {
      flowid_ = FLOWIDCNT++;

      first_tail_ = true;
      bw_valid_ = true;

      bool singleton = fastflow::is_singleton(orig);

      fastflow_headinfo* h = new fastflow_headinfo();
      h->is_flow_ = !singleton;

      current_head_ = fastflow_message::construct(h, orig, ptr(this));

      if (h->is_flow_)
      {
        fastflow_tailinfo* t = new fastflow_tailinfo();
        current_tail_ = fastflow_message::construct(t, orig, ptr(this));
      }

      route_ = new saved_route();
    }
  }
}
