/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_FASTFLOWER_MESSAGE_H_
#define SSTMAC_HARDWARE_FASTFLOWER_MESSAGE_H_

#include <sstmac/hardware/fastflower/fastflow.h>
#include <sstmac/hardware/network/interconnect/interconnect_message.h>

namespace sstmac
{
  namespace hw
  {

    struct fastflow_headinfo
    {
      bool is_flow_;
    };

    struct fastflow_tailinfo
    {

    };

    /**
     * A message that represents either the head or the tail of a flow.
     */
    class fastflow_message :
        public interconnect_message
    {

    protected:

      fastflow_headinfo* head_;
      fastflow_tailinfo* tail_;

      fastflow::ptr flow_;

      fastflow_message(fastflow_headinfo* h,
          const sst_message::ptr &o,
          fastflow::ptr f) :
        interconnect_message(o),
        head_(h),
        flow_(f)
      {
        tail_ = NULL;
      }

      fastflow_message(fastflow_tailinfo* t,
          const sst_message::ptr &o,
          fastflow::ptr f) :
        interconnect_message(o),
        tail_(t),
        flow_(f)
      {
        head_ = NULL;
      }

    public:

      typedef boost::intrusive_ptr<fastflow_message> ptr;



      static ptr
      construct(fastflow_headinfo* h,
          const sst_message::ptr &o,
          fastflow::ptr f)
      {
        return ptr(new fastflow_message(h, o, f));
      }

      static ptr
      construct(fastflow_tailinfo* t,
          const sst_message::ptr &o,
          fastflow::ptr f)
      {

        return ptr(new fastflow_message(t, o, f));
      }

      fastflow_headinfo*
      get_headinfo()
      {
        return head_;
      }

      fastflow_tailinfo*
      get_tailinfo()
      {
        return tail_;
      }

      fastflow::ptr
      get_flow()
      {
        return flow_;
      }

      virtual bool
      is_head() const
      {
        if (head_)
          return true;
        return false;
      }

      virtual bool
      is_tail() const
      {
        return !is_head();
      }

      virtual
      ~fastflow_message()
      {
        if (head_)
        {
          delete head_;
        }

        if (tail_)
        {
          delete tail_;
        }
      }

      virtual std::string
      to_string() const
      {
        std::stringstream ss;
        ss << "fastflow_message(";
        ss << (is_head() ? "head" : "tail");
        ss << ", bytes=" << flow_->get_byte_length() << ",";
        ss << ", flowid=" << flow_->get_flow_id() << ")";
        return ss.str();
        //return "fastflow_message";
      }
    };

  }
}
#endif
