/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/fastflower/fastflower_memory.h>
#include <sstmac/hardware/node/node.h>

namespace sstmac
{
  namespace hw
  {
    SSTRegister("fastflower",memorymodel,fastflower_memory,
        "Implements a fastflower memory model that models memory congestion as a flow");

    //void
    // fastflower_memory::forward(fastflow_message::ptr fmsg, timestamp delay)
    // {
    /* if (nic_->get_parent_id()->equals(fmsg->toaddr()))
     {
     eventcallback::ptr cb = new_callback(nic_, &fastflower_nic::mem_done,
     fmsg);

     schedule(now() + delay, cb, fmsg);
     }
     else
     {
     schedule(now() + delay, inj_, fmsg);
     }*/

    //  }

    /**
     * This will get called with the injector and nic
     * @param ty
     * @param mod
     */
    void
    fastflower_memory::connect(connection_type_t ty,
        const connectable::ptr& mod)
    {
      /*switch (ty)
       {
       case injection:
       {
       injector::ptr inj = safe_cast(mod, injector);
       inj_ = inj;
       break;
       }
       case ejection:
       {
       fastflower_nic::ptr nic = safe_cast(mod, fastflower_nic);
       nic_ = nic;
       break;
       }
       default:
       connectable_type_invalid(ty);
       }*/
    }

    void
    fastflower_memory::do_access(const sst_message::ptr& msg,
        eventhandler::ptr cb)
    {
      fastflow_message::ptr fmsg =
          boost::dynamic_pointer_cast<fastflow_message>(msg);

      if (!fmsg)
      {
        sst_throw(ssterror, "fastflower_memory::do_access - can't cast %s to a fastflow_message", msg->to_string().c_str());
      }

      bool re;
      timestamp d = model(fmsg, re);
      if (re)
      {
        eventcallback::ptr e = msg_callback(ptr(this),
            &fastflower_memory::do_access, cb);
        schedule(now() + d, e, fmsg);
      }
      else
        schedule(now() + d, cb, fmsg);
    }

    void
    fastflower_memory::tail_checker(const sst_message::ptr& msg)
    {
      fastflow_message::ptr fmsg =
          boost::dynamic_pointer_cast<fastflow_message>(msg);

      if (!fmsg)
      {
        sst_throw(ssterror, "fastflower_memory::tail_checker - can't cast %s to a fastflow_message", msg->to_string().c_str());
      }
      if (fmsg->is_tail() || !fmsg->get_headinfo()->is_flow_)
      {
        SSTMAC_DEBUG << "fastflower_memory: delivering to node: "
              << fmsg->orig()->to_string() << " \n";
        fmsg->orig()->add_callback(done_);
        done_->handle(fmsg->orig());
        fmsg->get_flow()->done();
        //parent_node_->handle(fmsg->get_orig());
      }
    }

    void
    fastflower_memory::access(const sst_message::ptr& msg)
    {
      SSTMAC_DEBUG << "fastflower_memory: read \n";
      fastflow::ptr flow = fastflow::construct(msg);

      do_access(flow->get_head(), tailcheck_);
      if (flow->get_tail())
      {
        do_access(flow->get_tail(), tailcheck_);
      }
    }

  }
}
