/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_FASTFLOWER_MEMORY_H_
#define SSTMAC_HARDWARE_FASTFLOWER_MEMORY_H_

#include <sstmac/hardware/fastflower/fastflower_component.h>
#include <sstmac/hardware/memory/memorymodel.h>


namespace sstmac
{
  namespace hw
  {

    class fastflower_memory : virtual public fastflower_component, virtual public memorymodel
    {

    protected:
      fastflower_memory(double u, double c, timestamp t) : fastflower_component("memory")
      {

        tailcheck_ = msg_callback(ptr(this), &fastflower_memory::tail_checker);

        init_params(u, c, t);
      }

      eventcallback::ptr tailcheck_;

    public:
      typedef boost::intrusive_ptr<fastflower_memory> ptr;

      static ptr
      construct(const sim_parameters::ptr &params)
      {
        double u = params->get_bandwidth_param("memory_unit_bw");
        double c = params->get_bandwidth_param("memory_capacity_bw");
        timestamp t = params->get_time_param("memory_latency");

        return ptr(new fastflower_memory(u, c, t));
      }

      virtual
      ~fastflower_memory()
      {
      }

      virtual std::string
      to_string() const
      {
        return "fastflower_memory";
      }

      // virtual void
      // forward(fastflow_message::ptr fmsg, timestamp delay);

      /**
       * This will get called with the injector and nic
       * @param ty
       * @param mod
       */
      virtual void
      connect(connection_type_t ty, const connectable::ptr& mod);

      virtual void
      finish()
      {

        memorymodel::finish();
      }

      virtual void
      handle(const sst_message::ptr &msg)
      {
        sst_throw(ssterror, "fastflower_memory::handle - don't call this");
      }

      void
      tail_checker(const sst_message::ptr &msg);

      /**
       * Makes a flow, schedules it
       * @param msg
       */
      virtual void
      do_access(const sst_message::ptr& msg, eventhandler::ptr cb);

      virtual void
      access(const sst_message::ptr& msg);

    };

  }
}
#endif
