/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_FASTFLOWER_SWITCH_H_
#define SSTMAC_HARDWARE_FASTFLOWER_SWITCH_H_

#include <sstmac/hardware/network/switches/networkswitch.h>
#include <sstmac/hardware/fastflower/fastflower_component.h>

namespace sstmac
{
  namespace hw
  {

#define MIN_FLOW_SIZE 64.0

    class fastflower_switch : public discrete_network_switch,
        public fastflower_component
    {
    public:
      typedef boost::intrusive_ptr<fastflower_switch> ptr;
      typedef boost::intrusive_ptr<const fastflower_switch> const_ptr;

    protected:
      fastflower_switch() :
        fastflower_component("switch")
      {
      }

      void
      init_members(const fastflower_switch::const_ptr &sw);

      timestamp link_latency_;
      static int max_paths_;

    public:

      virtual
      ~fastflower_switch()
      {
      }

      virtual std::string
      to_string() const
      {
        return "fastflower_switch";
      }

      virtual void
      handle(const sst_message::ptr &msg);

      virtual injector::ptr
      get_new_injector(sim_parameters::ptr params);

      virtual ejector::ptr
      get_new_ejector(sim_parameters::ptr params);

      static ptr
      construct(const sim_parameters::ptr& params)
      {
        return new fastflower_switch;
      }

      virtual void
      finish()
      {
        fastflower_component::finish();
        discrete_network_switch::finish();
      }

      virtual void
      set_eventmanager(const eventmanager::ptr &m)
      {
        discrete_network_switch::set_eventmanager(m);

        boost::unordered_map<long, eventhandler::ptr>::iterator it, end =
            outports_.end();
        for (it = outports_.begin(); it != end; it++)
        {
          eventmodule::ptr ev = safe_cast(it->second, eventmodule);
          ev->set_eventmanager(m);
        }
      }

      virtual void
      connect(int dim, int port, connection_type_t ty,
          const connectable::ptr &comp, double weight, int red);

      virtual void
      init_factory_params(const sim_parameters::ptr& params);

      virtual networkswitch::ptr
      clone(router::ptr route, nodeaddress::ptr addr) const;

      virtual std::string
      get_interprocess_link_name() const
      {
        return "network_link_latency";
      }

      virtual int
      get_queue_length(int port) const;

    };

  }
}

#endif
