#ifndef SSTMAC_HARDWARE_FLOW_H
#define SSTMAC_HARDWARE_FLOW_H

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/eventmodule.h>
#include <sstmac/common/eventcallback.h>

namespace sstmac {
    namespace hw {

class flow :
    public eventhandler
{

    protected:
        flow(const sst_message::ptr& msg,
             const eventmodule::ptr& parent,
             const eventhandler::ptr& completion,
             int max_segments,
             long min_bytes_per_segment,
             const timestamp& idle_time,
             const event_step_callback::ptr& flow_update_cb);

      static debug dbg_;

    public:
        typedef boost::intrusive_ptr<flow> ptr;

        static const int default_max_nsegments = 10;

        static const long default_min_bytes_per_segment = 4096;

        class link :
            public ptr_type
        {
            private:
                double max_single_bw_;

                double max_bw_;

                double latency_;

                double bw_allocated_;

                int nflows_;

                link(double max_bw, double max_single_bw, const timestamp& latency);

                link(double max_bw);

            public:
                typedef boost::intrusive_ptr<link> ptr;

                static ptr
                construct(double max_bw, double max_single_bw, const timestamp& latency);

                static ptr
                construct(double max_bw);

                int nflows() const;

                double max_bw() const;

                double allocated_bandwidth() const;

                void allocate_bandwidth(double bw);

                double suggest_bandwidth();

                void deallocate_bandwidth(double bw);

                void register_flow();

                void unregister_flow();

                double latency() const;

                std::string to_string() const
                {
                    return "flow::link";
                }

        };


        typedef std::list<bool>::iterator alloc_iterator;

        typedef std::list<link::ptr>::const_iterator const_link_iterator;

        static ptr
        construct(const sst_message::ptr& msg,
                  const eventmodule::ptr& parent,
                  const eventhandler::ptr& completion,
                  int max_nsegments,
                  long min_nbytes_per_segment,
                  const timestamp& idle_time = timestamp(0),
                  const event_step_callback::ptr& flow_update_cb = 0);

        void add_link(const link::ptr& lnk);

        void start();

        void stop();

        void register_flow();

        void push();

        void handle(const sst_message::ptr &msg);

        std::string to_string() const;

    private:
        std::list<link::ptr> links_;

        std::list<bool> allocations_;

        double idle_time_per_segment_;

        long bytes_per_segment_;

        int nsegments_;

        int segment_number_;

        sst_message::ptr msg_;

        eventhandler::ptr completion_;

        eventmodule::ptr parent_;

        event_step_callback::ptr flow_update_cb_;

        double last_bw_;


};


    }
}

#endif // FLOW_H
