/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef sstmac_hardware_memory_mc_memorymodel_H_
#define sstmac_hardware_memory_mc_memorymodel_H_

#include <sstmac/hardware/memory/memorymodel.h>
#include <sstmac/hardware/node/node.h>
#include <sstmac/hardware/processor/computescheduler.h>

namespace sstmac
{
  namespace hw
  {

    class mc_memorymodel : public memorymodel
    {
    protected:
      mc_memorymodel(
          double mem_bw, timestamp mem_lat,
          double proclink_bw, timestamp proclink_lat,
          int num_mc, int num_socket);

      int num_mc_; //per socket
      int num_socket_;
      std::map<long, int> threadmcs_;
      std::map<int, int> mc_cnt_;

      std::map<int, link::ptr> memlinks_;
      std::map<int, link::ptr> proclinks_;

      timestamp
      get_delta_t(const sst_message::ptr& msg,
                  int sourcesocket, int destsocket,
                  int mc);

    public:
      typedef boost::intrusive_ptr<mc_memorymodel> ptr;

      static ptr
      construct(double mem_bw, timestamp mem_lat,
                double proclink_bw, timestamp proclink_lat,
                int num_mc, int num_socket)
      {
        return new mc_memorymodel(mem_bw, mem_lat,
                                  proclink_bw, proclink_lat,
                                  num_mc, num_socket);
      }

      virtual
      ~mc_memorymodel()
      {
      }

      virtual void
      handle(const sst_message::ptr& msg);

      virtual void
      access(const sst_message::ptr& msg);

      virtual void
      get_location(long threadid, int &sock, int &mc);
    };

  }
} /* namespace sstmac */
#endif /* mc_memorymodel_H_ */
