#ifndef SSTMAC_HARDWARE_NUMA_MEMORYMODEL_H
#define SSTMAC_HARDWARE_NUMA_MEMORYMODEL_H

#include <sstmac/hardware/memory/memorymodel.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/hardware/flow/flow.h>
#include <sstmac/hardware/node/node.h>

namespace sstmac {
    namespace hw {


  class numa_memorymodel :
    public memorymodel
  {
  protected:
    numa_memorymodel(const timestamp& activate_latency,
                     double north_bus_bw,
                     double link_mem_bw,
                     const timestamp& nic_link_lat,
                     double nic_link_bw,
                     int nsocket,
                     int flow_nsegments,
                     long flow_nbytes_per_segment);

    typedef std::map<int, flow::link::ptr> link_map;

    std::vector<link_map> socket_links_;

    std::vector<flow::link::ptr> bus_links_;

    flow::link::ptr rdma_link_;

    flow::link::ptr nic_link_;

    int nsocket_;

    int flow_nsegments_;

    long flow_nbytes_per_segment_;

    timestamp socket_to_socket_latency_;

    void do_access(const sst_message::ptr& msg);

    int get_socket(long threadid) const;

    sw::operatingsystem::ptr get_os() const;

    flow::ptr construct_flow(const sst_message::ptr& msg);

    void skip_access(const sst_message::ptr& msg);

    bool is_skippable(const sst_message::ptr& msg) const;

  public:
    typedef boost::intrusive_ptr<numa_memorymodel> ptr;

    static ptr
    construct(const sim_parameters::ptr& params);

    virtual void
    init(const node::ptr &parent_node, const computescheduler::ptr &sched);

    virtual
    ~numa_memorymodel();

    void set_eventmanager(const eventmanager::ptr &m);

    virtual void finish();

    virtual void handle(const sst_message::ptr &msg);

    virtual void access(const sst_message::ptr& msg);

    /**
      Ignore ht latency for now
    */
    void add_hypertransport_link(int src_socket, int dst_socket, double bw);

    void add_rdma_link(int closest_socket, const timestamp& latency, double link_bw, int nlinks);

  };

} } /* namespace sstmac */


#endif // NUMA_MEMORYMODEL_H
