/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/memory/simple_memorymodel.h>

namespace sstmac
{
  namespace hw
  {

    SSTRegister("simple",memorymodel,simple_memorymodel,
              "Implements a simple memory model that is just a single link");

    simple_memorymodel::simple_memorymodel(double mem_bw,
                                           timestamp mem_lat)
    {
      link_ = link::construct(mem_bw, mem_lat);
    }

    simple_memorymodel::~simple_memorymodel()
    {
      link_ = link::ptr();
    }

    void
    simple_memorymodel::handle(const sst_message::ptr &msg)
    {
      if(msg->get_type() == sst_message::FAILURE){
          failure_occurred(msg, ptr(this));
          return;
      }
      link_->access_done();

      msg->add_callback(done_);


      done_->handle(msg);
    }

    simple_memorymodel::ptr
    simple_memorymodel::construct(const sim_parameters::ptr &params)
    {
        /** sstkeyword { gui=100ns; } */
        timestamp memory_latency = params->get_time_param("node_mem_latency");
        /** sstkeyword { gui=4GB/s; } */
        double memory_bandwidth = params->get_bandwidth_param("node_mem_bandwidth");
        return new simple_memorymodel(memory_bandwidth, memory_latency);
    }

    void
    simple_memorymodel::finish()
    {
        if (finished_)
            return;

        memorymodel::finish();
        done_ = 0;
    }

/**
    eventhandler::ptr
    memorymodel::get_finish_handler(const sst_message::ptr& msg)
    {
        if (!msg->is_network_message())
        {
            // incoming network message
            if (msg->toaddr()->equals(parent_node_->get_id())){
                return to_node_;
            }
            else {
                return connections_[connectable::injection];
            }
        }
        else {
            return to_node_;
        }
    }

    void
    simple_memorymodel::handle(const sst_message::ptr &msg)
    {

    }
*/

    void
    simple_memorymodel::access(const sst_message::ptr &msg)
    {
        use_me();

        SSTMAC_DEBUG << "simple_memorymodel: doing access of " << msg->get_byte_length() << " \n";

        mem_bytes_->record(msg->get_byte_length());
        timestamp delta_t = link_->new_access(now(), msg->get_byte_length());
        delta_t += msg->get_event_time();
        schedule(now() + delta_t, ptr(this), msg);
    }

  }
} /* namespace sstmac */
