#include <sstmac/hardware/network/interconnect/analyticinterconnect.h>
#include <sstmac/hardware/nic/networkinterface.h>

namespace sstmac {
    namespace hw {

    SSTRegister("analytic", interconnect, analyticinterconnect,
        "Models the network interconnect via a simple analytic function of latency/bandwidth");


  analyticinterconnect::analyticinterconnect()
  {
  }

  void
  analyticinterconnect::init_factory_params(const sim_parameters::ptr &params)
  {
    bw_ = params->get_bandwidth_param("network_bandwidth");
    latency_ = params->get_time_param("network_latency");
    interconnect::init_factory_params(params);
  }

  void
  analyticinterconnect::handle(const sst_message::ptr &msg)
  {
    long dst_id = msg->toaddr()->unique_id();
    node::ptr dst_node = nodes_[dst_id];
    networkinterface::ptr dst_nic = dst_node->get_nic();
    double bw_term = msg->get_byte_length() / bw_;
    timestamp arrival = now() + latency_ + timestamp(bw_term);
    schedule(arrival, dst_nic, msg);

    if (msg->get_needs_ack()){
        sst_message::ptr acker = msg->clone(sst_message::SEND_ACK);
        node::ptr src_node = nodes_[msg->fromaddr()->unique_id()];
        schedule(now(), src_node->get_nic(), acker);
    }
  }

  void
  analyticinterconnect::connect(
      const nodeaddress::ptr& addr,
      connection_type_t ty,
      const connectable::ptr& mod)
  {
  }

  void
  analyticinterconnect::set_eventmanager(const eventmanager::ptr &m)
  {
      add_nodes(m->params_);
      init_failure_model(m->params_);

      std::vector<node::ptr>::iterator nodeit, nodeend = nodes_.end();
      for (nodeit = nodes_.begin(); nodeit != nodeend; nodeit++)
      {
        node::ptr the_node = *nodeit;
        connectable::ptr nic = the_node->get_nic();
        nic->connect(connectable::injection, ptr(this));
      }

      interconnect::set_eventmanager(m);
  }


    }
}
