/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_HARDWARE_NETWORK_MULTI_DIVIDER_H_
#define SSTMAC_HARDWARE_NETWORK_MULTI_DIVIDER_H_

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/errors.h>
#include <sstmac/common/factories/factory.h>
#include <sstmac/hardware/nic/networkinterface.h>

namespace sstmac
{
  namespace hw
  {
    class divider :
        public factory_type
    {
    protected:
      int num_;

      divider()
      {

      }

    public:
      typedef boost::intrusive_ptr<divider> ptr;

      virtual
      ~divider()
      {
      }

      virtual std::string
      to_string() const
      {
        return "divider";
      }

      virtual int
      which(sst_message::ptr msg) = 0;

      virtual void
      init(int num, const boost::unordered_map<int, networkinterface::ptr>& nics)
      {
        num_ = num;
      }

    };

    class size_divider : public divider
    {

    protected:

      long thresh_;

      size_divider(long thresh) :
        thresh_(thresh)
      {

      }

    public:
      typedef boost::intrusive_ptr<size_divider> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        long thresh = params->get_long_param("multi_sizedivider_threshold");

        return ptr(new size_divider( thresh));
      }

      virtual void
      init(int num)
      {
        if (num > 2)
          {
            sst_throw(ssterror,
                "size_divider::construct - i don't know what to do with more than 2 networks");
          }
        num_ = num;
      }

      virtual int
      which(sst_message::ptr msg)
      {
        if (msg->get_byte_length() < thresh_)
          {
            return 1;
          }
        else
          {
            return 2;
          }

      }
    };

    DeclareFactory(divider);
  }
}

#endif
