/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_HARDWARE_NETWORK_MULTI_MULTINETWORK_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_MULTI_MULTINETWORK_H_INCLUDED

#include <sstmac/hardware/network/interconnect/interconnect.h>
#include <sstmac/hardware/network/topology/fake_topology.h>
#include <sstmac/common/sim_parameters.h>

namespace sstmac
{
  namespace hw
  {

    class multinetwork : public interconnect
    {
    protected:
      multinetwork();

      boost::unordered_map<int, interconnect::ptr> networks_;

    public:
      typedef boost::intrusive_ptr<multinetwork> ptr;

      static ptr
      construct(const sim_parameters::ptr& params)
      {
        return new multinetwork;
      }

      void
      init_factory_params(const sim_parameters::ptr &params);

      virtual std::string
      to_string() const
      {
        return "multinetwork";
      }

      virtual
      ~multinetwork()
      {
      }

      interconnect::ptr
      get_network(int index)
      {
        if (networks_.find(index) == networks_.end())
        {
          std::cerr << "index: " << index << "\n";
          throw ssterror("multinetwork:get_network -- can't find index");
        }
        return networks_[index];
      }

      virtual long
      num_nodes() const
      {
        boost::unordered_map<int, interconnect::ptr>::const_iterator it =
            networks_.begin();
        return it->second->num_nodes();
      }

      virtual void
      finish();

      virtual void
      set_eventmanager(const eventmanager::ptr &m);

    };

  }
}
#endif /* MULTINETWORK_H_ */
