/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_SWITCHINTERCONNECT_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_SWITCHINTERCONNECT_H_INCLUDED

#include <sstmac/hardware/network/interconnect/interconnect.h>
#include <sstmac/hardware/network/switches/networkswitch.h>
#include <sstmac/hardware/network/switches/entryexit.h>
#include <sstmac/hardware/network/topology/topology.h>
#include <sstmac/hardware/network/topology/fake_topology.h>

#include <boost/unordered_map.hpp>
#include <sstmac/common/vis/vis.h>

#include <sstmac/common/driver_util.h>

namespace sstmac
{
  namespace hw
  {

    class switchinterconnect :
        public interconnect,
        public vis::vis_display
    {

    protected:

      switchinterconnect();

      topology::maptype switches_;

    public:
      typedef boost::intrusive_ptr<switchinterconnect> ptr;

      virtual void
      init_factory_params(const sim_parameters::ptr &params);

      virtual std::string
      to_string() const
      {
        return "switchinterconnect";
      }

      static ptr
      construct(const sim_parameters::ptr& params);

      virtual
      ~switchinterconnect()
      {

      }

      virtual std::string
      get_interprocess_link_name(){
        networkswitch::ptr switchtemp = get_template_switch(sstmac_env::params, topology_);
        return switchtemp->get_interprocess_link_name();
      }

      static networkswitch::ptr
      get_template_switch(const sim_parameters::ptr& params,
          const topology::ptr& top);

      virtual void
      finish();

      virtual void
      set_eventmanager(const eventmanager::ptr &m);

      virtual vis::vis_topology::vismaptype
      vis_get_nodes();

      virtual vis::vis_topology::ptr
      vis_get_topology();

      topology::maptype&
      get_switches()
      {
        return switches_;
      }

    };

  }
}

#endif
