#ifndef sstmac_hardware_network_NETWORK_MESSAGE_H
#define sstmac_hardware_network_NETWORK_MESSAGE_H

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/hardware/network/topology/routing/routing_info.h>

namespace sstmac {
    namespace hw {

  class network_message :
    virtual public sst_message,
    public serializable_type<network_message>
    {
        ImplementSerializable(network_message)

        public:
            typedef boost::intrusive_ptr<network_message> ptr;

        protected:
            bool needs_ack_;

            routing_info::routing_algorithm_t route_algo_;

            network_message(const nodeaddress::ptr& toaddr, const nodeaddress::ptr& fromaddr);

            network_message();

        public:

            static ptr
            construct();

            static ptr
            construct(const nodeaddress::ptr& toaddr, const nodeaddress::ptr& fromaddr);

            routing_info::routing_algorithm_t
            route_algo() const {
                return route_algo_;
            }

            void
            set_route_algo(routing_info::routing_algorithm_t algo) {
                route_algo_ = algo;
            }

            nodeaddress::ptr toaddr() const {
                return toaddr_;
            }

            nodeaddress::ptr fromaddr() const {
                return fromaddr_;
            }

            void
            set_toaddr(const nodeaddress::ptr &addr) {
                toaddr_ = addr;
            }

            void
            set_fromaddr(const nodeaddress::ptr &addr){
                fromaddr_ = addr;
            }

            nodeaddress::ptr toaddr_;

            nodeaddress::ptr fromaddr_;

            void
            set_needs_ack(bool n){
                needs_ack_ = n;
            }

            virtual bool
            get_needs_ack() const {
                return needs_ack_;
            }

            bool is_network_message() const {
                return true;
            }

            virtual void
            serialize_order(sst_serializer *ser);


  };

    }
}
#endif // NETWORK_MESSAGE_H
