/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_CIRCUIT_CIRCUITSWITCH_H_
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_CIRCUIT_CIRCUITSWITCH_H_

#include <sstmac/hardware/network/switches/cycleaccurate/cycleaccurateswitch.h>
#include <sstmac/common/sstmac-config.h>
#include <sstmac/hardware/network/switches/circuit/circuit_router.h>
#include <sstmac/common/stats/stats_common.h>
namespace sstmac
{
  namespace hw
  {

    class circuitswitch : public cycleaccurateswitch
    {
    public:
      typedef boost::intrusive_ptr<circuitswitch> ptr;
      typedef boost::intrusive_ptr<const circuitswitch> const_ptr;

      typedef boost::unordered_map<int, int> maptype;

      static ptr
      construct(const sim_parameters::ptr& params)
      {
        return new circuitswitch;
      }

      virtual
      ~circuitswitch()
      {
      }

      virtual networkswitch::ptr
      clone(router::ptr route, nodeaddress::ptr addr) const;

      virtual std::string
      to_string() const
      {
        return "circuitswitch(" + (my_addr_ ? my_addr_->to_string()
            : "don't know where i am") + ")";
      }

      virtual void
      finish();

    protected:
      circuitswitch();

      virtual void
      init_factory_params(const sim_parameters::ptr& params)
      {
        cycleaccurateswitch::init_factory_params(params);
        blocked_protocol_ = params->get_bool_param(
            "circuitswitch_blocked_protocol");
      }

      virtual int
      check_inport(int inport, bool &clock);

      void
      do_route(cycleaccuratepayload::ptr msg, int inport, int &outport,
          bool &kill);

      bool
      check_circuit(cycleaccuratepayload::ptr msg, int in, int out, bool &kill);

      void
      modify_circuits(cycleaccuratepayload::ptr msg, int in, int out);

      virtual injector::ptr
      get_new_injector(sim_parameters::ptr params);

      maptype circuitsin_; //points from in to out
      maptype circuitsout_; //point from out to in

      bool blocked_protocol_;

      circuit_router::ptr circrouter_;

      static stat_sum::ptr timeout_stat_;

      class circuitpacketinjector : public packetinjector
      {

      protected:
        circuitpacketinjector(sim_parameters::ptr params,
            circuitswitch::ptr par) :
          packetinjector(params, par)
        {
          circpar_ = par;
        }

        circuitswitch::ptr circpar_;
      public:
        typedef boost::intrusive_ptr<circuitpacketinjector> ptr;

        virtual std::string
        to_string() const
        {
          return "circuitpacketinjector";
        }

        virtual
        ~circuitpacketinjector()
        {
        }

        static ptr
        construct(sim_parameters::ptr params, circuitswitch::ptr par)
        {
          return ptr(new circuitpacketinjector(params, par));
        }

        void send(long nbytes, long byte_offset, const sst_message::ptr &msg);

      };
    };

  }
}

#endif /* CIRCUITSWITCH_H_ */
