/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_CYCLEACCURATEMESSAGE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_CYCLEACCURATEMESSAGE_H_INCLUDED

#include <sstmac/hardware/network/interconnect/interconnect_message.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac
{
  namespace hw
  {


    class cycleaccurateack :
        public sst_message,
        public serializable_type<cycleaccurateack>
    {
        ImplementSerializable(cycleaccurateack)

      protected:
        cycleaccurateack(long size, int vc)
            : sst_message(ACK),
            size_(size),
            vc_(vc)
        {
        }

      public:
        typedef boost::intrusive_ptr<cycleaccurateack> ptr;

      int vc_;
      int inport_;
      long size_;

        static ptr
        construct(long size, int vc){
          return ptr(new cycleaccurateack(size, vc));
        }

        virtual std::string
        to_string() const
        {
          return "cycleaccurateack";
        }

        virtual long get_byte_length() const{
          return size_;
        }

    };

    class cycleaccuratepayload :
        public interconnect_message
    {
    protected:
      cycleaccuratepayload(sst_message::ptr orig, long size, bool is_tail, int vc) :
        interconnect_message(orig, size, DATA),
        vc_(vc), size_(size), tail_(is_tail)
      {
      }

    public:
      typedef boost::intrusive_ptr<cycleaccuratepayload> ptr;

      static ptr
      construct(sst_message::ptr orig, long size, bool is_tail, int vc)
      {
        return new cycleaccuratepayload(orig, size, is_tail, vc);
      }

      virtual std::string
      to_string() const
      {
        return "cycleaccuratepayload(" + orig_->to_string() + ")";
      }

      virtual
      ~cycleaccuratepayload()
      {
      }

      virtual nodeaddress::ptr
      dest()
      {
        return orig_->toaddr();
      }

      virtual bool is_tail() const {
        return tail_;
      }

      long long msg_id_;
      int vc_;
      int inport_;
      long size_;
      bool tail_;
      timestamp bufferarrive_;

      discrete_network_switch::ptr prevswitch_;
      int prevport_;
    };

  }
}

#endif
