/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/network/switches/dist_dummyswitch.h>
#include <sstmac/backends/native/ipc_event.h>
namespace sstmac
{
  namespace hw
  {
    debug dist_dummyswitch::dbg_;

    using namespace native;

    void
    dist_dummyswitch::handle(const sst_message::ptr &msg)
    {
      SSTMAC_DEBUG << "dist_dummswitch::handle - look at my message! : "
            << msg->to_string() << " at time " << now() << " with lat " << lat_
            << "\n";
      //network_message::ptr net = boost::dynamic_pointer_cast<network_message>(
      //     msg);
      // if (!net)
      // {
      //   sst_throw(ssterror, "dist_dummyswitch:handle - message didn't cast to network_message (%s)", msg->to_string().c_str());
      // }

      ipc_event::ptr ev = ipc_event::construct(msg, eventhandler::ignore,
          my_addr_);
      ev->set_time(now() + lat_);

      if (ev->time() < last_thru_)
      {
        sst_throw(ssterror, "dist_dummyswitch::handle - it appears messages don't come in order through here");
      }
      event::ptr temp = ev;

      evman_->ipc_schedule(my_addr_, temp);

      last_thru_ = ev->time();
    }

    void
    dist_dummyswitch::set_eventmanager(const eventmanager::ptr &m)
    {
      networkswitch::set_eventmanager(m);
      // dist_ = boost::dynamic_pointer_cast<dist_eventmap>(m);
      // if (!dist_)
      // {
      //   sst_throw(ssterror, "dist_dummyswitch::set_eventmanager - eventmanager didn't cast to a dist_eventmap");
      // }
      evman_ = m;
    }
  }

}
