/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_DIST_DUMMYSWITCH_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_DIST_DUMMYSWITCH_H_INCLUDED

#include <sstmac/hardware/network/switches/networkswitch.h>
//#include <sstmac/backends/native/ip_queue.h>

//#include <sstmac/backends/native/par_eventmap.h>

namespace sstmac
{
  namespace hw
  {

    class dist_dummyswitch : public networkswitch
    {
    protected:

      //sstmac::native::ip_queue::ptr ipq_;
      nodeaddress::ptr going_;

     // native::par_eventmap::ptr dist_;

      eventmanager::ptr evman_;
      timestamp lat_;

      static debug dbg_;

      timestamp last_thru_;

      dist_dummyswitch(nodeaddress::ptr going, timestamp lat) :
        going_(going), lat_(lat)
      {
        last_thru_ = timestamp(0);
        if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> parallel | network | dist_dummyswitch");
          logger::register_logger(&dbg_);
        }
      }

    public:
      typedef boost::intrusive_ptr<dist_dummyswitch> ptr;

      static ptr
      construct(nodeaddress::ptr going, timestamp lat)
      {
        return ptr(new dist_dummyswitch(going, lat));
      }

      std::string
      to_string() const
      {
        return "dist_dummyswitch";
      }

      virtual
      ~dist_dummyswitch()
      {
      }

      virtual networkswitch::ptr
      clone(router::ptr route, nodeaddress::ptr addr) const
      {
        ptr ret = construct(addr, lat_);
        ret->init_addr(addr, route);

        return ret;
      }

      virtual void
      handle(const sst_message::ptr &msg);


      virtual void
      connect(connection_type_t ty, const connectable::ptr& mod)
      {
        connect(any_port, ty, mod);
      }

      virtual void
      connect(long port, connection_type_t ty, const connectable::ptr& mod)
      {
        connect(port, ty, mod, 1.0, 1);
      }

      virtual void
      connect(const nodeaddress::ptr& addr, int port, connection_type_t ty,
          const connectable::ptr& mod)
      {
        connect(addr->unique_id(), port, ty, mod, 1.0, 1);
      }

      virtual void
      connect(const nodeaddress::ptr& addr, connection_type_t ty,
          const connectable::ptr& mod)
      {
        connect(addr->unique_id(), ty, mod, 1.0, 1);
      }

      virtual void
      connect(int dim, int port, connection_type_t ty,
          const connectable::ptr& mod, double weight, int red)
      {
        //sst_throw(unimplementederror, "connectable does not implement weighted/redundant connect");
      }

      virtual void
      connect(long port, connection_type_t ty, const connectable::ptr& mod,
          double weight, int red)
      {
        //sst_throw(unimplementederror, "connectable does not implement weighted/redundant connect");
      }

      virtual void
      set_eventmanager(const eventmanager::ptr &m);

      virtual std::string
      get_interprocess_link_name() const {
        sst_throw(ssterror, "dist_dummyswitch::get_interprocess_link_name - don't call on this");
      }

    };
  }
}

#endif
