/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_PACKETMESSAGE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_PACKETMESSAGE_H_INCLUDED

#include <sstmac/hardware/network/interconnect/interconnect_message.h>
#include <sstmac/hardware/network/switches/packet/packet_routing_info.h>

namespace sstmac
{
  namespace hw
  {
    class packetmessage :
  public interconnect_message, public serializable_type<packetmessage>
    {
    ImplementSerializable(packetmessage)

    protected:
      packetmessage(sst_message::ptr orig, long size, bool is_tail) :
        interconnect_message(orig, size, NETWORK),
        tail_(is_tail)
      {
      }

      bool tail_;

    public:
      typedef boost::intrusive_ptr<packetmessage> ptr;

      static ptr
      construct(sst_message::ptr orig, long size, bool is_tail)
      {
        return ptr(new packetmessage(orig, size, is_tail));
      }

      virtual bool
      is_tail() const
      {
        return tail_;
      }

      virtual std::string
      to_string() const
      {
        return "packetmessage";
      }

      virtual
      ~packetmessage()
      {
      }



      /**
       * Serialize this message during parallel simulation.
       * @param ser The serializer to use
       */
      virtual void
         serialize_order(sst_serializer* ser);



      /**********************************
       *
       *
       **********************************/
    };
  }
}

#endif
