/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_FAKE_TOPOLOGY_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_FAKE_TOPOLOGY_H_INCLUDED

#include <sstmac/hardware/network/topology/regular_topology.h>

namespace sstmac
{
  namespace hw
  {

    class fake_topology :
        public regular_topology
    {
    protected:
      void
      compute_switch_coords(long swid, coordinates &coords) const {
        //do nothing
      }

    public:
      fake_topology() :
        regular_topology(1)
      {
      }

      typedef boost::intrusive_ptr<fake_topology> ptr;
      virtual int64_t
      diameter() const
      {
        throw "switchinterconnect::fake_topology::diameter() - don't call this";
      }

      static ptr
      construct()
      {
        return new fake_topology;
      }

      long
      get_switch_id(const coordinates& coords) const
      {
        return 0;
      }

      int
      get_productive_dir(
        int& dim,
        const coordinates &current,
        const coordinates &dst) const
      {
        return 0;
        //do nothing
      }

      void connect_switches(maptype &switches){
        //do nothing
      }

      void
      minimal_route_to_coords(const coordinates &current_coords,
                              const coordinates &dest_coords,
                              int &dim, int &dir) const
      {
        //do nothing
      }

      int
      minimal_distance(
        const coordinates &current_coords,
        const coordinates &dest_coords) const
      {
        return 0;
      }

      int radix() const {
        return 0;
      }

      int ndimensions() const {
        return 0;
      }

      long get_switch_id(const std::vector<int>& coords) const {
        return 0;
      }

      long get_node_id(const std::vector<int>& coords) const {
        return 0;
      }

      long num_switches() const {
        return 0;
      }

      long num_leaf_switches() const {
        return 0;
      }

      long num_nodes() const {
        return 0;
      }

      int
      convert_to_outport(int dim, int dir) {
        return 0;
      }

      void
      convert_to_dimdir(int outport, int &dim, int&dir) {
        dim = 0;
        dir = 0;
      }

      topology_descr::ptr
      get_descr(const nodeaddress::ptr &addr) const {
        return topology_descr::ptr();
      }

    };
  }
}

#endif
