/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

// flattenedbutterfly.h: Interface for torus networks.
//
// Author: Ali Pinar <apinar@sandia.gov>

#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_flattenedbutterfly_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_flattenedbutterfly_H_INCLUDED

#include <sstmac/hardware/network/topology/regular_topology.h>
#include <sstmac/hardware/network/topology/butterfly.h>

namespace sstmac
{
  namespace hw
  {

    /**
        See High Performance Datacenter Networks by
        Dennis Abts
        John Kim
        For figures/detailed explanation
    */
    class flattenedbutterfly :
        public abstract_butterfly
    {

    protected:
      flattenedbutterfly(int kary, int nfly, int nndpersw);

      virtual void
      compute_switch_coords(long uid, coordinates& coords) const;

    public:
      static const int up_dimension;
      static const int down_dimension;

      typedef boost::intrusive_ptr<flattenedbutterfly> ptr;

      static flattenedbutterfly::ptr
      construct(sim_parameters::ptr params);

      virtual int
      radix() const
      {
        return kary_ * nfly_;
      }

      virtual
      ~flattenedbutterfly()
      {
      }

      long
      num_switches() const {
        return nswitches_per_col_;
      }

      virtual void
      display_nodes(vismaptype &switches, const vis::vis_engine::ptr& eng,
          std::list<vis::vis_obj::ptr> &objs);

      void
      minimal_route_to_coords(
        const coordinates &current_coords,
        const coordinates &dest_coords,
        int &dim, int &dir) const;

      int minimal_distance(const coordinates &current_coords, const coordinates &dest_coords) const;

      void
      connect_switches(topology::maptype &switches);

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      int
      get_productive_dir(
        int &dim,
        const coordinates &src,
        const coordinates &dst) const;

      long
      get_switch_id(const coordinates &coords) const;

      virtual std::string
      to_string() const
      {
        return "flattenedbutterflytopology";
      }

      int ndimensions() const {
        return nfly_ - 1;
      }

    };

  }
} //end of namespace sstmac

#endif
