#include <sstmac/hardware/network/topology/routing/basic_routing.h>

namespace sstmac {
    namespace hw {

    SSTRegister("minimal", routing_algorithm, minimal_routing,
                "a routing algorithm for minimal routing on regular topologies");

    SSTRegister("basic", routing_algorithm, basic_routing,
                "a routing algorithm for basic routing on all topologies");

    void
    regular_routing::set_switch(const networkswitch::ptr &sw)
    {
        sw_ = sw;
        router_ = safe_cast(sw->get_router(), regular_router);
        regtop_ = safe_cast(sw_->get_topology(), regular_topology);
    }

    regular_routing::regular_routing()
    {
    }

    regular_routing::~regular_routing()
    {
    }

    int
    minimal_routing::route(const interconnect_message::ptr& msg)
    {
        int dim, dir;
        router_->minimal_route_to_node(msg->toaddr(), dim, dir);
        int port = router_->convert_to_outport(dim, dir);
        SSTMAC_DEBUG << sstprintf("Routing %p from %ld to %ld on port=%d queue=%d\n",
                                msg.get(),
                                sw_->get_addr()->unique_id(),
                                msg->toaddr()->unique_id(),
                                port, port >= 0 ? sw_->get_queue_length(port) : 0);
        return port;
    }

    void
    minimal_routing::init_routing(const interconnect_message::ptr &msg) const
    {
        msg->init_routing(minimal_routing_info::construct());
    }

    int
    basic_routing::route(const interconnect_message::ptr &msg)
    {
        return router_->basic_route(msg);
    }

    void
    basic_routing::set_switch(const networkswitch::ptr &sw)
    {
        router_ = sw->get_router();
    }

    void
    basic_routing::init_routing(const interconnect_message::ptr &msg) const
    {
        msg->init_routing(minimal_routing_info::construct());
    }

  }
}
