#ifndef sstmac_hardware_network_topology_routing_BASIC_ROUTING_H
#define sstmac_hardware_network_topology_routing_BASIC_ROUTING_H

#include <sstmac/hardware/network/topology/routing/regular_router.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac {
    namespace hw {

    class regular_routing :
        public routing_algorithm
    {
        protected:
            networkswitch::ptr sw_;
            regular_router::ptr router_;
            regular_topology::ptr regtop_;

        public:
            regular_routing();

            virtual ~regular_routing();

            virtual void
            set_switch(const networkswitch::ptr &sw);
    };

    class minimal_routing :
        public regular_routing
    {
        public:
            virtual int
            route(const interconnect_message::ptr &msg);

            virtual void
            init_routing(const interconnect_message::ptr &msg) const;

            virtual ~minimal_routing(){}

            static ptr
            construct(const sim_parameters::ptr& params){
                return new minimal_routing;
            }

            std::string
            to_string() const {
                return "minimal routing";
            }
    };

    class basic_routing :
        public routing_algorithm
    {
        protected:
            router::ptr router_;

        public:
            typedef boost::intrusive_ptr<basic_routing> ptr;

            static ptr
            construct(const sim_parameters::ptr& params){
                return new basic_routing;
            }

            virtual ~basic_routing(){}

            virtual void
            set_switch(const networkswitchPtr &sw);

            virtual int
            route(const interconnect_message::ptr &msg);

            virtual void
            init_routing(const interconnect_message::ptr &msg) const;

            std::string
            to_string() const {
                return "minimal routing";
            }

    };

    }
}


#endif // BASIC_ROUTING_H
