/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_FATTREEROUTER_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_FATTREEROUTER_H_INCLUDED

#include <sstmac/hardware/network/topology/routing/regular_router.h>
#include <sstmac/hardware/network/switches/circuit/circuit_router.h>
#include <sstmac/hardware/network/topology/fattree.h>
#include <sstmac/common/messages/sst_message.h>
#include <math.h>

namespace sstmac
{
  namespace hw
  {

    class fattreerouter :
        public virtual regular_router,
        public virtual circuit_router
    {
    private:
        bool is_top_level() const {
            return myL_ == l_ - 1;
        }

        void
        minimal_route_to_switch(
            const nodeaddress::ptr& sw_addr,
            int& dim, int& dir);

    public:
      typedef boost::intrusive_ptr<fattreerouter> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        return new fattreerouter;
      }

      virtual
      ~fattreerouter()
      {
      }

      virtual void
      init(const nodeaddress::ptr& me, const topology::ptr& top);

      virtual int
      choose_up_path();

      virtual int
      choose_down_path();

      virtual int
      get_number_paths(const sst_message::ptr& msg) const;

      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir);

      virtual void
      path_is_good(nodeaddress::ptr goingto, int fromport, int toport)
      {
      }

      void
      get_productive_ports_to_switch(const nodeaddress::ptr &dst, outports &ports);

      int
      convert_to_outport(int dim, int dir);

      void
      convert_to_dimdir(int outport, int &dim, int &dir);

      virtual void
      path_teardown(int fromport, int toport)
      {
      }

      virtual std::string
      to_string() const
      {
        return "fattreerouter";
      }

      virtual void
      set_inport(nodeaddress::ptr from, int port);

      void clear_routing_info(){
        numpicked_ = 0;
        numpicktop_ = 0;
      }

    protected:
      int l_;
      int k_;

      int myL_;
      int logicalid_;

      std::map<long, int> inports_;
      static RNG::Combo::ptr rng_;

      long num_leaf_switches_reachable_;
      long num_leaf_switches_per_path_;
      long level_relative_id_;
      long min_reachable_leaf_id_;
      long max_reachable_leaf_id_;


      int numpicked_;
      int pickstart_;

      int numpicktop_;
      int pickstarttop_;

      fattreerouter();

    };



  }
}
#endif
