#ifndef REGULAR_ROUTER_H
#define REGULAR_ROUTER_H

#include <sstmac/hardware/network/topology/regular_topology.h>
#include <sstmac/hardware/network/topology/routing/router.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac {
    namespace hw {

    class regular_router
        : public router
    {
        protected:
          int nps_;

          regular_topology::ptr regtop_;


          virtual void
          get_productive_outports(outports& prod_ports,
                                  const coordinates& current,
                                  const coordinates& dst);

          /**
            @param dim [inout] This can be modified if moving in a given dimension
                        requires first moving in another dimension
            @param src The coordinates of the source switch
            @param dst The coordinates of the dest switch
            @return The direction in dimension dim that is productive
          */
          int
          get_productive_dir(int& dim,
                             const coordinates& current,
                             const coordinates& dst) const;

        public:
          typedef boost::intrusive_ptr<regular_router> ptr;

          void
          init(const nodeaddress::ptr& me, const topology::ptr &top);

          int num_nodes() const {
            return nps_;
          }

          virtual void set_switch(const networkswitchPtr &sw);

          virtual int
          convert_to_outport(int dim, int dir){
            return router::convert_to_outport(dim, dir);
          }

          static ptr
          construct(const sim_parameters::ptr& params){
            return new regular_router;
          }

          int
          basic_route(const interconnect_message::ptr &msg);

          virtual void
          convert_to_dimdir(int outport, int &dim, int&dir){
            router::convert_to_dimdir(outport, dim, dir);
          }

          virtual void
          minimal_route_to_node(
            const nodeaddress::ptr& node_addr,
            int& dim, int& dir);

          virtual void
          minimal_route_to_switch(
            const nodeaddress::ptr& sw_addr,
            int& dim, int& dir);

          /**
            @return The inport
          */
          virtual int
          get_inport(const nodeaddress::ptr& output_buf_sw,
                     const nodeaddress::ptr& input_but_bw,
                     int dim, int dir){
            return router::convert_to_outport(dim, dir);
          }

          virtual void
          get_productive_ports_to_switch(
            const nodeaddress::ptr& dst,
            outports& ports);

    };

    class dimension_order_router
        : public regular_router
    {
        public:
            typedef boost::intrusive_ptr<dimension_order_router> ptr;

            static ptr
            construct(const sim_parameters::ptr &params);

            virtual void
            get_productive_outports(outports& prod_ports,
                                  const coordinates& current,
                                  const coordinates& dst);
    };


    }
}

#endif // REGULAR_ROUTER_H
