/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_TFF_ROUTER_H_
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_TFF_ROUTER_H_

#include <sstmac/hardware/network/topology/routing/router.h>
#include <sstmac/hardware/network/switches/networkswitch.h>
#include <sstmac/hardware/network/topology/top_from_file.h>

namespace sstmac
{
  namespace hw
  {

    class tff_router : public sstmac::hw::router
    {
    protected:
      tff_router()
      {
      }

      boost::unordered_map<long, long> nodemap_;
      boost::unordered_map<long, std::vector<connection> > rconnections_; //router to router connections
      boost::unordered_map<long, std::vector<long> > nconnections_; //router to node connections

    public:
      typedef boost::intrusive_ptr<tff_router> ptr;

      virtual
      ~tff_router()
      {
      }

      virtual std::string
      to_string() const
      {
        return "tff_router";
      }

      virtual void
      init(const nodeaddress::ptr& me, const topology::ptr& top)
      {
        router::init(me, top);
      }

      int num_nodes() const {
        return 1;
      }

      virtual void
      init_maps(const boost::unordered_map<long, long> &nodemap,
          const boost::unordered_map<long, std::vector<connection> > &rconn,
          const boost::unordered_map<long, std::vector<long> > &nconn)
      {
        nodemap_ = nodemap;
        rconnections_ = rconn;
        nconnections_ = nconn;
      }

          virtual int
          convert_to_outport(int dim, int dir){
            return router::convert_to_outport(dim, dir);
          }

          virtual void
          convert_to_dimdir(int outport, int &dim, int&dir){
            router::convert_to_dimdir(outport, dim, dir);
          }

          virtual int
          get_inport(
            const nodeaddress::ptr &output_buf_sw,
            const nodeaddress::ptr &input_buf_sw,
            int dim, int dir){
            return router::get_inport(output_buf_sw, input_buf_sw,
                                      dim, dir);
          }

    };
  }
}

#endif
