/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_TOP_FROM_FILE_DJIKSTRA_H_
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_TOP_FROM_FILE_DJIKSTRA_H_

#include <sstmac/hardware/network/topology/routing/tff_router.h>

namespace sstmac
{
  namespace hw
  {


    class tff_router_djikstra :
        public tff_router
    {
    protected:
      //this is the routing table, it maps any destination router to port number
      boost::unordered_map<long, int> rtable_;

      //maps final NODE to connected router (which is not necessarily nodemap_)
      boost::unordered_map<long, long> reverse_table_;
      bool built_rtable_;


      virtual int
      basic_route(const interconnect_message::ptr &msg);

      void
      route(const nodeaddress::ptr& dst, int &dim, int &dir);

    public:
      typedef boost::intrusive_ptr<tff_router_djikstra> ptr;

      static ptr
      construct(sstmac::sim_parameters::ptr params)
      {
        return ptr(new tff_router_djikstra());
      }

      virtual
      ~tff_router_djikstra()
      {
      }

      void
      get_productive_ports_to_switch(const nodeaddress::ptr &dst, outports& ports);

    protected:
      tff_router_djikstra();

      void
      build_rtable();

      virtual void
      init_factory_params(const sim_parameters::ptr &params);

    };

  }
}
#endif
