#ifndef sstmac_hardware_network_topology_routing_VALIANT_ROUTER_H
#define sstmac_hardware_network_topology_routing_VALIANT_ROUTER_H

#include <sstmac/hardware/network/topology/routing/basic_routing.h>

namespace sstmac
{
  namespace hw
  {

    class valiant_routing_info : public routing_info, public serializable_type<
        valiant_routing_info>
    {
    ImplementSerializable(valiant_routing_info)

    public:
      typedef enum
      {
        initial, intermediate, final
      } stage_t;

    protected:
      nodeaddress::ptr dest_switch_;

      stage_t stage_;

      valiant_routing_info() :
        stage_(initial)
      {
      }

    public:
      typedef boost::intrusive_ptr<valiant_routing_info> ptr;

      static ptr
      construct()
      {
        return new valiant_routing_info();
      }

      virtual routing_info::ptr
              clone(){
        ptr ret = construct();
        ret->stage_ = stage_;
        ret->dest_switch_ = dest_switch_;
        return ret;
      }

      virtual
      ~valiant_routing_info()
      {
      }

      virtual routing_algorithm_t
      route_algo() const
      {
        return valiant;
      }

      virtual std::string
      to_string() const
      {
        return "valiant routing";
      }

      stage_t
      stage() const
      {
        return stage_;
      }

      void
      set_stage(stage_t stage)
      {
        stage_ = stage;
      }

      void
      set_dest_switch(const nodeaddress::ptr& sw)
      {
        dest_switch_ = sw;
      }

      nodeaddress::ptr
      get_dest_switch() const
      {
        return dest_switch_;
      }

      nodeaddress::ptr
      dest_switch()
      {
        return dest_switch_;
      }

      /**
       * Serialize this message during parallel simulation.
       * @param ser The serializer to use
       */
      virtual void
      serialize_order(sst_serializer* ser);

    };

    class valiant_routing : public regular_routing
    {
    protected:
      valiant_routing();

      int
      final_step(const valiant_routing_info::ptr& rinfo,
          const interconnect_message::ptr& msg);

      int
      intermediate_step(const valiant_routing_info::ptr& rinfo,
          const interconnect_message::ptr& msg);

      /**
       Different for pure valiant and UGAL.
       */
      virtual int
      initial_step(const valiant_routing_info::ptr& rinfo,
          const interconnect_message::ptr& msg);

    public:
      typedef boost::intrusive_ptr<valiant_routing> ptr;

      virtual
      ~valiant_routing()
      {
      }

      static ptr
      construct(const sim_parameters::ptr& params)
      {
        return new valiant_routing;
      }

      virtual void
      init_routing(const interconnect_message::ptr &msg) const;

      int
      route(const interconnect_message::ptr &msg);

      virtual int
      get_number_paths(const interconnect_message::ptr& msg) const;


      virtual std::string
      to_string() const
      {
        return "valiant routing";
      }

    };

  }
}

#endif // VALIANT_ROUTER_H
