/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NIC_SIMPLENIC_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NIC_SIMPLENIC_H_INCLUDED

#include <sstmac/software/services/service.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/network/interconnect/interconnect.h>

namespace sstmac {
    namespace hw {

  /**
   * A private nested type to handle sending data.
   */



  class simplenic: public networkinterface
  {

  public:

    typedef boost::intrusive_ptr<simplenic> ptr;

  protected:
    /// Hi.
    simplenic() : networkinterface() { }

    virtual void
    recv(const sst_message::ptr& netmsg);

  private:

    void send_to_os(const sst_message::ptr& msg);

  public:
    /// Hello.
    static simplenic::ptr
    construct(
        const sim_parameters::ptr& params);

    /// Goodbye.
    virtual ~simplenic() throw ();

    /// Schedule a send operation to happen at the given time.
    /// \param recver is the global index of the receiver node.
    /// \param payload is the data sent over the network
    virtual void send(const sst_message::ptr &payload);

    virtual networkinterface::ptr
    clone(const node::ptr &parent,
        const sstmac::hw::interconnect::ptr &interconn,
        const sim_parameters::ptr& params) const;



  };


    }
  } // end of namespace sstmac.

#endif
