/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_DEGRADEDNODE_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_DEGRADEDNODE_H_INCLUDED

#include <sstmac/hardware/node/simplenode.h>

#include <sstmac/hardware/processor/simpleprocessor.h>
#include <sstmac/hardware/processor/instructionprocessor.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/node/node.h>
#include <sstmac/common/nocopy.h>

#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/units.h>


namespace sstmac
{
  namespace hw
  {

    /**
     * A simple node that is degraded, cutting its performance by a factor
     */
    class degradednode :
        public simplenode,
        virtual public nocopy
    {

    protected:
        degradednode(const sim_parameters::ptr &params)
            : simplenode(params)
        {
            factor_ = params->get_optional_double_param("node_degraded_factor", 1.0);
        }

      degradednode(double fac, const sim_parameters::ptr &params,
          nodeaddress::ptr my_id) :
        simplenode(params, my_id), factor_(fac)
      {

      }

      double factor_;

    public:
      typedef boost::intrusive_ptr<degradednode> ptr;

      static ptr
      construct(sim_parameters::ptr params, double fac, const nodeaddress::ptr& my_id)
      {
        return new degradednode(fac, params, my_id);
      }

      static ptr
      construct(sim_parameters::ptr params)
      {
        return new degradednode(params);
      }

      virtual node::ptr
      clone(nodeaddress::ptr my_id) const;

      virtual void
      initialize(eventmanager::ptr man, const topology::ptr& top);

      virtual
      ~degradednode()
      {

      }

    };

  }
} // end of namespace sstmac

#endif
