#ifndef NULLNODE_H
#define NULLNODE_H

#include <sstmac/hardware/processor/simpleprocessor.h>
#include <sstmac/hardware/processor/instructionprocessor.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/hardware/memory/memorymodel.h>
#include <sstmac/hardware/disk/vesper/disk_controller.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/node/node.h>
#include <sstmac/common/nocopy.h>

#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/units.h>

namespace sstmac
{
  namespace hw
  {

    /**
     * A null node.  All requests just complete immediately.
     */
    class nullnode :
        public node,
        virtual public nocopy
    {

    protected:
      sstmac::hw::computescheduler::ptr compute_sched_;

    protected:
      nullnode(const sim_parameters::ptr &params);

      nullnode(const sim_parameters::ptr &params,
          const nodeaddress::ptr& my_id);

      virtual bool
      has_failed();

      virtual void
      reset_failed();


    public:
      typedef boost::intrusive_ptr<nullnode> ptr;

      static ptr
      construct(const sim_parameters::ptr& params)
      {
        return new nullnode(params);
      }

      void
      init_factory_params(const sim_parameters::ptr &params);

      virtual void
      execute_kernel(AMI::AMI_COMP_FUNC func,
          const boost::intrusive_ptr<sst_message>& data);

      virtual void
      execute_kernel(AMI::AMI_COMM_FUNC func,
          const boost::intrusive_ptr<sst_message>& data);

      virtual ptr_type::ptr
      execute_kernel(AMI::AMI_QUERY_FUNC func);

      virtual ptr_type::ptr
      execute_kernel(AMI::AMI_HW_FUNC func);

      // ------- AMI functions --------- //
      virtual bool
      kernel_supported(AMI::AMI_COMP_FUNC) const;
      virtual bool
      kernel_supported(AMI::AMI_COMM_FUNC) const;
      virtual bool
      kernel_supported(AMI::AMI_QUERY_FUNC) const;
      virtual bool
      kernel_supported(AMI::AMI_HW_FUNC) const;

      virtual timestamp
      now() const;

      // --------------------- //

      virtual node::ptr
      clone(nodeaddress::ptr my_id) const;

      virtual void
      initialize(eventmanager::ptr man, const topology::ptr& top);

      virtual std::list<failable::ptr>
      get_failure_components();

      virtual void
      finish();

    };

  }
} // end of namespace sstmac

#endif // NULLNODE_H
