/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_SIMPLENODE_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_SIMPLENODE_H_INCLUDED

#include <sstmac/hardware/processor/simpleprocessor.h>
#include <sstmac/hardware/processor/instructionprocessor.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/hardware/memory/memorymodel.h>
#include <sstmac/hardware/disk/vesper/disk_controller.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/node/node.h>
#include <sstmac/common/nocopy.h>

#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/units.h>

namespace sstmac
{
  namespace hw
  {

    /**
     * A stand-in node.  This node type passes all requests directly to
     * the simulator and has no resource contention (infinite processors),
     */
    class simplenode : public node, virtual public nocopy
    {

    protected:
      void
      init(const sim_parameters::ptr& params);

      simplenode(const sim_parameters::ptr &params);

      simplenode(const sim_parameters::ptr &params,
          const nodeaddress::ptr& my_id);

      double freq_;
      timestamp preempt_;

      sstmac::hw::simpleprocessor::ptr proc_model_;
      sstmac::hw::instructionprocessor::ptr proc_inst_;
      sstmac::hw::computescheduler::ptr compute_sched_;
      sstmac::hw::disk_controller::ptr disk_;

      nodeaddress::ptr nppfs_server_;

      virtual bool
      has_failed();
      virtual void
      reset_failed();

      virtual bool
      try_comp_kernels(AMI::AMI_COMP_FUNC func, const sst_message::ptr& data);

    public:
      typedef boost::intrusive_ptr<simplenode> ptr;

      static ptr
      construct(const sim_parameters::ptr& params)
      {
        return new simplenode(params);
      }

      void
      init_factory_params(const sim_parameters::ptr &params);

      virtual void
      execute_kernel(AMI::AMI_COMP_FUNC func,
          const boost::intrusive_ptr<sst_message>& data);

      virtual void
      execute_kernel(AMI::AMI_COMM_FUNC func,
          const boost::intrusive_ptr<sst_message>& data);

      virtual ptr_type::ptr
      execute_kernel(AMI::AMI_QUERY_FUNC func);

      virtual ptr_type::ptr
      execute_kernel(AMI::AMI_HW_FUNC func);

      // ------- AMI functions --------- //
      virtual bool
      kernel_supported(AMI::AMI_COMP_FUNC) const;
      virtual bool
      kernel_supported(AMI::AMI_COMM_FUNC) const;
      virtual bool
      kernel_supported(AMI::AMI_QUERY_FUNC) const;
      virtual bool
      kernel_supported(AMI::AMI_HW_FUNC) const;

      virtual timestamp
      now() const;

      // --------------------- //

      virtual node::ptr
      clone(nodeaddress::ptr my_id) const;

      virtual void
      initialize(eventmanager::ptr man, const topology::ptr& top);

      virtual std::list<failable::ptr>
      get_failure_components();

      virtual void
      finish();

      //virtual void
      //init_failure_model(double l_proc, double l_mem, double l_disk, double l_nic, failable::ptr ps, double p_ps_node);
    };

  }
} // end of namespace sstmac

#endif
