#include <sstmac/hardware/packetflow/packetflow.h>

namespace sstmac {
    namespace hw {

long packetflow_payload::min_num_bytes_per_packet_ = 0;
long packetflow_payload::max_num_packets_ = 0;
long packetflow_payload::id_count_ = 0;
long packetflow_payload::negligible_size_ = 0;
const double packetflow_payload::uninitialized_bw = -1;

packetflow_payload::packetflow_payload(
    const sst_message::ptr &parent,
    int msg_id,
    long num_bytes,
    long offset) :
    interconnect_message(parent, num_bytes, NETWORK),
    byte_offset_(offset),
    msg_id_(msg_id),
    bytes_credited_(0),
    bytes_sent_(0),
    bytes_arrived_(0),
    bw_(uninitialized_bw),
    send_done_(0)
{
}

void
packetflow_payload::clear_intermediates()
{
    bytes_credited_ = 0;
    bytes_sent_ = 0;
    bytes_arrived_ = 0;
}

bool
packetflow_payload::get_needs_ack() const {
    bool needs_ack = is_tail()
        && orig()->is_network_message()
        && orig()->get_needs_ack();
    return needs_ack;
}

void
packetflow_payload::init_statics(int min_bytes, int max_packets, int negligible_size)
{
  min_num_bytes_per_packet_ = min_bytes;
  max_num_packets_ = max_packets;
  negligible_size_ = negligible_size;
  negligible_size_ = std::min(negligible_size_, min_num_bytes_per_packet_);
}

std::string
packetflow_payload::to_string() const
{
  return sstprintf("flow %d, %lu:%lu bw=%8.4e credits=%ld out=%ld in=%ld",
      msg_id(),
      byte_offset_,
      byte_offset_ + num_bytes_,
      bw_,
      bytes_credited_,
      bytes_sent_,
      bytes_arrived_);
}

std::string
packetflow_credit::to_string() const
{
    return sstprintf("credit: now=%ld incoming=%ld delta_t=%8.4e",
                      num_bytes_now_, num_bytes_flow_, delta_t_);
}




    }
}
