#ifndef BUFFER_H
#define BUFFER_H

#include <sstmac/hardware/packetflow/packetflow_handler.h>
#include <sstmac/hardware/packetflow/packetflow_multiplexer.h>
#include <sstmac/hardware/packetflow/packetflow_crossbar.h>
#include <sstmac/hardware/packetflow/packetflow_sender.h>

namespace sstmac {
    namespace hw {



class packetflow_simple_buffer :
    public packetflow_handler
{
    protected:
        int credit_port_;
        packetflow_handler::ptr input_;
        packetflow_handler::ptr output_;

        double out_bw_;
        timestamp out_lat_;
        long max_num_bytes_;

        packetflow_sender::ptr sender_;

        packetflow_simple_buffer(
            double out_bw,
            const timestamp& out_lat,
            long max_num_bytes);

    public:
        typedef boost::intrusive_ptr<packetflow_simple_buffer> ptr;

        static ptr
        construct(
            double out_bw,
            const timestamp& out_lat,
            long max_num_bytes)
        {
            return new packetflow_simple_buffer(out_bw, out_lat, max_num_bytes);
        }

        void
        set_output(int inport, const packetflow_handler::ptr& dest){
            output_ = dest;
            sender_->init(dest->initial_num_credits());
        }

        void
        set_input(int outport, const packetflow_handler::ptr& inp){
            credit_port_ = outport;
            input_ = inp;
        }

        virtual void
        handle_credit(const packetflow_credit::ptr &msg);

        virtual void
        handle_payload(const packetflow_payload::ptr &msg);

        long
        initial_num_credits() const {
            return max_num_bytes_;
        }

        std::string
        to_string() const;

        void
        send_along(const packetflow_payload::ptr &msg);

        void
        set_eventmanager(const eventmanager::ptr &m);
};

class packetflow_crossbar_buffer :
    public packetflow_handler
{
    protected:
        int credit_port_;

        packetflow_handler::ptr input_;

        int my_xbar_port_;

        packetflow_crossbar::ptr xbar_;

        double out_bw_;

        timestamp out_lat_;

        long max_num_bytes_;

        packetflow_crossbar_buffer(
            double out_bw,
            const timestamp& out_lat,
            long max_num_bytes);

    public:
        typedef boost::intrusive_ptr<packetflow_crossbar_buffer> ptr;

        static ptr
        construct(
            double out_bw,
            const timestamp& out_lat,
            long max_num_bytes)
        {
            return new packetflow_crossbar_buffer(out_bw, out_lat, max_num_bytes);
        }

        void
        set_input(int outport, const packetflow_handler::ptr& inp);

        void
        set_output(int inport, const packetflow_crossbar::ptr& xbar);

        virtual void
        handle_credit(const packetflow_credit::ptr &msg);

        virtual void
        handle_payload(const packetflow_payload::ptr &msg);

        void
        ack(const packetflow_payload::ptr& msg);

        long
        initial_num_credits() const {
            return max_num_bytes_;
        }

        std::string
        to_string() const;

        void
        set_eventmanager(const eventmanager::ptr &m);
};


class packetflow_multiplexer_buffer :
    public packetflow_handler
{
    protected:
        int my_mpex_port_;

        packetflow_handler::ptr input_;

        packetflow_multiplexer::ptr output_;

        packetflow_multiplexer_buffer();

    public:
        typedef boost::intrusive_ptr<packetflow_multiplexer_buffer> ptr;

        static ptr
        construct(){
            return new packetflow_multiplexer_buffer;
        }

        void
        set_output(const packetflow_multiplexer::ptr& out){
            output_ = out;
            my_mpex_port_ = out->add_input(ptr(this));
        }

        void
        set_input(const packetflow_handler::ptr& inp){
            input_ = inp;
        }

        virtual void
        handle_credit(const packetflow_credit::ptr &msg);

        virtual void
        handle_payload(const packetflow_payload::ptr &msg);

        virtual void
        send_along(const packetflow_payload::ptr &msg);

        long
        initial_num_credits() const {
            return 0;
        }

        std::string
        to_string() const {
            return "packetflow multiplexer buffer";
        }

};

    }
}


#endif // BUFFER_H
