#include <sstmac/hardware/packetflow/packetflow_crossbar.h>

namespace sstmac {
    namespace hw {

packetflow_crossbar::packetflow_crossbar(timestamp lat, double bw) :
    out_lat_(lat),
    out_bw_(bw)
{

}

void
packetflow_crossbar::handle_credit(const packetflow_credit::ptr &msg)
{
    SSTMAC_DEBUG << sstprintf("On crossbar %s, receiving {%s} for port %d\n",
                              descr_->to_string().c_str(),
                              msg->to_string().c_str(),
                              msg->port());
    senders_[msg->port()]->add_credit(msg);
}

void
packetflow_crossbar::handle_payload(const packetflow_payload::ptr &msg)
{

}

std::string
packetflow_crossbar::to_string() const {
    return std::string("crossbar ") + descr_->to_string();
}


void
packetflow_crossbar::arbitrate(
    int inport,
    const packetflow_payload::ptr& msg)
{
    int outport = router_->get_next_port(msg);
    SSTMAC_DEBUG << sstprintf("On crossbar %s, on port %d arbitrating {%s}\n",
                              descr_->to_string().c_str(),
                              outport,
                              msg->to_string().c_str());

    msg->set_inport(inport);
    msg->set_outport(outport);
    if (outport < 0){
        router_->eject(msg);
        inputs_[inport]->ack(msg);
    }
    else {
        senders_[outport]->send(msg);
    }
}

void
packetflow_crossbar::set_output(int outport, const packetflow_handler::ptr &handler)
{
    packetflow_sender::ptr sender = packetflow_sender::construct(out_lat_, out_bw_, ptr(this));
    sender->init(handler->initial_num_credits());
    sender->set_port(outport);
    senders_[outport] = sender;
    outputs_[outport] = handler;
}

void
packetflow_crossbar::send_along(const packetflow_payload::ptr& msg)
{
    SSTMAC_DEBUG << sstprintf("On crossbar %s, sending along to port %d {%s}\n",
                              descr_->to_string().c_str(),
                              msg->outport(),
                              msg->to_string().c_str());
    timestamp arrival = timestamp(msg->ser_delay()) + now() + out_lat_;
    schedule(arrival, outputs_[msg->outport()], msg);
    inputs_[msg->inport()]->ack(msg);
}

int
packetflow_crossbar::allocate_inport()
{
    int port = inputs_.size();
    inputs_[port] = 0;
    return port;
}

void
packetflow_crossbar::set_input(int inport, const packetflow_handler::ptr &input)
{
    inputs_[inport] = input;
}

void
packetflow_crossbar::set_eventmanager(const eventmanager::ptr &m)
{
    packetflow_handler::set_eventmanager(m);
    { sender_map::iterator it, end = senders_.end();
    for (it=senders_.begin(); it != end; ++it){
        it->second->set_eventmanager(m);
    } }
}

    }
}

