#ifndef PACKETFLOW_CROSSBAR_H
#define PACKETFLOW_CROSSBAR_H

#include <sstmac/hardware/packetflow/packetflow_handler.h>
#include <sstmac/hardware/packetflow/packetflow_sender.h>
#include <sstmac/hardware/packetflow/packetflow_router.h>

namespace sstmac {
    namespace hw {

class packetflow_crossbar :
    public packetflow_handler
{
    protected:
        packetflow_router::ptr router_;

        typedef boost::unordered_map<int, packetflow_handler::ptr> output_map;
        typedef boost::unordered_map<int, packetflow_handler::ptr> input_map;
        typedef boost::unordered_map<int, packetflow_sender::ptr> sender_map;

        input_map inputs_;
        output_map outputs_;
        sender_map senders_;

        timestamp out_lat_;
        double out_bw_;

        struct request {
            int port;
            long num_bytes;
        };

        typedef std::list<request> buffer_request_list;

        typedef boost::unordered_map<int, buffer_request_list> xbar_request_map;

        packetflow_crossbar(timestamp lat, double bw);

    public:
        typedef boost::intrusive_ptr<packetflow_crossbar> ptr;

        virtual ~packetflow_crossbar(){}

        static ptr
        construct(timestamp lat, double bw){
            return new packetflow_crossbar(lat, bw);
        }

        int allocate_inport();

        void
        arbitrate(int inport, const packetflow_payload::ptr &msg);

        void
        set_input(int inport, const packetflow_handler::ptr& input);

        void
        set_output(int outport, const packetflow_handler::ptr& output);

        void request_access(
            int inport,
            int outport,
            const packetflow_payload::ptr& msg);

        void send_along(const packetflow_payload::ptr &msg);

        virtual void
        handle_credit(const packetflow_credit::ptr &msg);

        virtual void
        handle_payload(const packetflow_payload::ptr &msg);

        virtual void
        set_eventmanager(const eventmanager::ptr &m);

        long
        initial_num_credits() const {
            return 0;
        }

        void
        set_router(const packetflow_router::ptr& router){
            router_ = router;
        }

        std::string
        to_string() const;

};

    }
}

#endif // PACKETFLOW_CROSSBAR_H
