#ifndef PACKETFLOW_CREDITOR_H
#define PACKETFLOW_CREDITOR_H

#include <sstmac/hardware/network/interconnect/interconnect_message.h>
#include <sstmac/common/eventmodule.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/hardware/packetflow/packetflow.h>
#include <sstmac/hardware/packetflow/packetflow_handler.h>

namespace sstmac {
    namespace hw {

struct packetflow_epoch
{
    long num_bytes;
    double start;
    double stop;
    packetflow_epoch* next;
    packetflow_epoch* prev;

    double length() const {
        return stop - start;
    }

    void split(double time);

    long add_credits(long total_bytes, double total_time);

};

class packetflow_sender :
    public eventmodule
{
    protected:
        static debug dbg_;

        long total_bytes_available_;

        long bytes_available_now_;

        packetflow_epoch* head_;

        timestamp send_lat_;

        double max_output_bw_;

        packetflow_handler::ptr parent_;

        std::list<packetflow_payload::ptr> queue_;

        int port_;

    protected:
        packetflow_sender(timestamp lat, double max_bw, const packetflow_handler::ptr& parent);

        void allocate(const packetflow_payload::ptr& msg);

        void do_send();

        bool try_send();

        void update_epochs();

    public:
        typedef boost::intrusive_ptr<packetflow_sender> ptr;

        static ptr
        construct(timestamp lat, double max_bw, const packetflow_handler::ptr& parent){
            return new packetflow_sender(lat, max_bw, parent);
        }

        void add_credit(const packetflow_credit::ptr& credit);

        /**
            @return Whether the message had enough credits to send
        */
        void send(const packetflow_payload::ptr& msg);

        void
        init(long num_credits);

        void
        set_eventmanager(const eventmanager::ptr &m);

        void
        set_port(int port){
            port_ = port;
        }

        int
        port() const {
            return port_;
        }

};

    }
}

#endif // PACKETFLOW_CREDITOR_H
