#include <sstmac/hardware/packetflow/packetflow_startpoint.h>

namespace sstmac {
    namespace hw {

debug packetflow_startpoint::dbg_;

packetflow_startpoint::packetflow_startpoint()
{
    if (!dbg_.is_real())
    {
        dbg_ = debug("<debug> packetflow");
    }
}

void
packetflow_startpoint::send(const sst_message::ptr &msg)
{
    long msg_bytes = msg->get_byte_length();
    int num_packets = msg_bytes / packetflow_payload::min_num_bytes_per_packet();
    int remainder = msg_bytes % packetflow_payload::min_num_bytes_per_packet();
    long offset = 0;
    long msg_id = packetflow_payload::allocate_msg_id();
    for (int i=0; i < num_packets; ++i){
        packetflow_payload::ptr payload = packetflow_payload::construct(
            msg, msg_id,
            packetflow_payload::min_num_bytes_per_packet(),
            offset);
        offset += packetflow_payload::min_num_bytes_per_packet();
        pending_payloads_.push_back(payload);
    }
    if (remainder){
        packetflow_payload::ptr payload = packetflow_payload::construct(
            msg, msg_id,
            remainder,
            offset);
        offset += packetflow_payload::min_num_bytes_per_packet();
        pending_payloads_.push_back(payload);
    }
    send_what_you_can();
}

void
packetflow_startpoint::send_what_you_can()
{
    std::list<packetflow_payload::ptr>::iterator
        it, end = pending_payloads_.end();
    timestamp ack_delay(1e-9);
    for (it=pending_payloads_.begin(); it != end; ++it){
        packetflow_payload::ptr msg = *it;
        msg->set_bw(1.0e9);
        SSTMAC_DEBUG << sstprintf("On startpoint, injecting to %s message {%s}\n",
                                entry_->to_string().c_str(),
                                msg->to_string().c_str());
        entry_->handle_payload(msg);
        if (msg->get_needs_ack() && acker_){
            schedule(now() + ack_delay, acker_, msg->orig()->clone(sst_message::SEND_ACK));
        }
    }
    pending_payloads_.clear();
}


    }
}
