/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/processor/instructionprocessor-config.h>
#include <sstmac/common/errors.h>

#include <stdlib.h>
#include <iostream>
#include <fstream>


namespace sstmac { namespace hw  {

  //
  // Protected constructor.
  //
  instructionprocessor_config::
  instructionprocessor_config() : processor_config(processor_config::PROC_INSTRUCTION, 1),
    flop_("PAPI_FP_INS"), tflop_(10e-9), //ns
    memop_("PAPI_LST_INS"), tmemop_(250e-9) //ns
  {}

  //
  // Protected constructor using std::strings.
  //
  instructionprocessor_config::
  instructionprocessor_config(int cores, const std::string &flop_label,
                         double tflop,
                         const std::string &memop_label,
                         double tmemop,
                         const std::string &memop_local_label,
                         double tmemop_local,
                         const std::string &memop_rand_label,
                         double tmemop_rand,
                         double pwr_st, double p_flop, double p_memop,
                         double p_memop_local, double p_memop_rand
                         ) : processor_config(processor_config::PROC_INSTRUCTION, cores),
    flop_(flop_label), tflop_(tflop),
    memop_(memop_label), tmemop_(tmemop),
    memop_local_(memop_local_label), tmemop_local_(tmemop_local),
    memop_rand_(memop_rand_label), tmemop_rand_(tmemop_rand),
    pwr_static(pwr_st), pwr_flop(p_flop), pwr_memop(p_memop), pwr_memop_local(p_memop_local),
    pwr_memop_rand(p_memop_rand)
  {}



  //
  // Public default constructor.
  //
  instructionprocessor_config::ptr
  instructionprocessor_config::construct()
  {
    instructionprocessor_config::ptr
      retval(new instructionprocessor_config());
    return retval;
  }

  //
  // Public constructor.
  //
  instructionprocessor_config::ptr
  instructionprocessor_config::construct(int cores, const std::string& flop_label,
                                    double tflop,
                                    const std::string& memop_label,
                                    double tmemop,
                                    const std::string& memop_local_label,
                                    double tmemop_local,
                                    const std::string& memop_rand_label,
                                    double tmemop_rand,
                                    double pwr_st, double p_flop, double p_memop,
                                    double p_memop_local, double p_memop_rand)
  {
    instructionprocessor_config::ptr
      retval(new instructionprocessor_config(cores, flop_label, tflop,
                                        memop_label, tmemop,
                                        memop_local_label, tmemop_local, 
                                        memop_rand_label, tmemop_rand,
                                         pwr_st,  p_flop,  p_memop,
                                         p_memop_local,  p_memop_rand));
    return retval;
  }




} } // end of namespace sstmac
