/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/processor/pinnedcomputescheduler.h>

namespace sstmac
{
  namespace hw
  {

    pinnedcomputescheduler::pinnedcomputescheduler(long nodeid, int ncores,
        timestamp preempt, int n_sock, const sw::operatingsystem::ptr& os) :
      migratingcomputescheduler(nodeid, n_sock * ncores, n_sock, preempt, os),
          core_cnt_(0), sock_cnt_(0), n_sock_(n_sock)
    {

    }

    int
    pinnedcomputescheduler::find_open_core(long threadid)
    {
      if (threadid < 0)
        return -1;

      if (threads_.find(threadid) == threads_.end())
        {
          threads_[threadid] = sock_cnt_ * (ncores_ / n_sock_) + core_cnt_;

          SSTMAC_DEBUG << "pinnedcomputescheduler: assigned thread "
                << threadid << " to core " << threads_[threadid] << "\n";
          sock_cnt_ = (sock_cnt_ + 1) % n_sock_;

          if (sock_cnt_ == 0)
            core_cnt_ = (core_cnt_ + 1) % (ncores_ / n_sock_);
        }

      if (compute_slots_[threads_[threadid]])
        {
          return -1;
        }
      else
        {
          return threads_[threadid];
        }
    }

  }
} //end of namespace sstmac
