/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_PROCESSOR_pinnedcomputescheduler_H_INCLUDED
#define SSTMAC_HARDWARE_PROCESSOR_pinnedcomputescheduler_H_INCLUDED

#include <sstmac/hardware/processor/migratingcomputescheduler.h>

namespace sstmac
{
  namespace hw
  {

    class pinnedcomputescheduler : public migratingcomputescheduler
    {
    protected:
      pinnedcomputescheduler(long nodeid, int ncores, timestamp preempt, int n_sock, const sw::operatingsystem::ptr& os);

      std::map<long, int> threads_;
      int core_cnt_;
      int sock_cnt_;
      int n_sock_;

      virtual int
      find_open_core(long threadid);

    public:
      virtual std::string
      to_string() const
      {
        return "pinnedcomputescheduler";
      }
      typedef boost::intrusive_ptr<pinnedcomputescheduler> ptr;

      virtual
      ~pinnedcomputescheduler()
      {
      }

      static pinnedcomputescheduler::ptr
      construct(long nodeid, int cores, timestamp pre, int n_sock, const sw::operatingsystem::ptr& os)
      {
        return ptr(new pinnedcomputescheduler(nodeid, cores, pre, n_sock, os));
      }

      virtual int
      get_socket(long thread_id)
      {
        if(thread_id < 0)
          return 0;

        if (threads_.find(thread_id) == threads_.end())
          {
            threads_[thread_id] = sock_cnt_ * (ncores_ / n_sock_) + core_cnt_;

            SSTMAC_DEBUG << "pinnedcomputescheduler: assigned thread "
                  << thread_id << " to core " << threads_[thread_id] << "\n";
            sock_cnt_ = (sock_cnt_ + 1) % n_sock_;

            if(sock_cnt_ == 0)
              core_cnt_ = (core_cnt_ + 1) % (ncores_ / n_sock_);


          }
        return threads_[thread_id] / (ncores_ / n_sock_);
      }

    private:

    };

  }
} //end of namespace sstmac
#endif
