/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/main/sim_manager.h>

#include <sstmac/main/multisim.h>

#include <sstmac/backends/native/driver.h>

namespace sstmac
{

  sim_manager::sim_manager()
  {
    // TODO Auto-generated constructor stub

  }

  sim_parameters::ptr
  sim_manager::check_multisim(const opts &oo, sim_parameters::ptr params)
  {
    sim_parameters::ptr ret = params;
    if (params->is_multi_sim())
    {

      std::vector<multisim::parameter_run> runs;
      multisim::multisim_init(params, runs, true);
      if (oo.run_num >= 0)
      {
        if (oo.run_num >= runs.size())
        {
          sst_throw(sstmac::valueerror, "sstmac: run number given (%d) is larger than number of runs (%d)", oo.run_num, runs.size());
        }
        ret = runs[oo.run_num].params_;
      }
      else
      {

        ret = multisim::do_multisim_fork(runs);
      }
    }

    return ret;
  }

  void
  sim_manager::run(const opts &oo, sim_parameters::ptr params)
  {

    native::driver_run(params, oo.stoptime, (oo.mode != "notime"));

  }
}
