/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */


#ifndef SSTMAC_SOFTWARE_AMI_AMI_H_INCLUDED
#define SSTMAC_SOFTWARE_AMI_AMI_H_INCLUDED

#include <sstmac/common/sstmac-config.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/common/errors.h>

namespace sstmac {

   class sst_message;
   class timestamp;
   namespace sw {

  class AMI
  {
  public:
    enum AMI_CAT
    {
      AMI_COMPUTE = 0, AMI_COMM = 1, AMI_DISP = 2, AMI_QUERY = 3, AMI_HW = 4
    };

    enum AMI_COMM_FUNC
    {
      AMI_COMM_SEND = 10101, //pretty much have to implement this one
      AMI_COMM_BCAST,
      AMI_COMM_BARRIER,
      AMI_COMM_NIC_SEND_ACK   //this is sent back to the processor after the message has left the source
    };

    enum AMI_COMP_FUNC
    {

      AMI_COMP_TIME = 67, //the basic compute-for-some-time
      AMI_COMP_SCHEDULE,
      AMI_COMP_RELEASE,
      AMI_COMP_INSTR,
			AMI_COMP_EIGER,
      //only compute the time needed to compute the
      //instructions and return it via the event data message
      AMI_COMP_INSTR_TIME,
      AMI_COMP_MEMREAD,
      AMI_COMP_MEMWRITE,
      AMI_COMP_DISKACCESS,
      AMI_COMP_STOCH,
      AMI_COMP_SLEEP,
      AMI_COMP_REGISTER_THREAD,
      AMI_GPU_MEMCPY,
      AMI_GPU_EX_ASYNC
    };

    //for changing graphics display colors, if possible
    enum AMI_DISP_FUNC
    {
      AMI_DISP_BORDER = 231,
      AMI_DISP_FILL
    };
    
    enum AMI_QUERY_FUNC
    {
      AMI_QUERY_ADDRESS = 1000
    };

    enum AMI_HW_FUNC {
      AMI_CATASTROPHIC_INTERRUPT = 999999
    };

    virtual ~AMI() {}

    virtual void
    execute_kernel(AMI::AMI_COMP_FUNC func, const boost::intrusive_ptr<sst_message>& data);

    virtual void
    execute_kernel(AMI::AMI_COMM_FUNC func, const boost::intrusive_ptr<sst_message>& data);

    virtual void
    execute_kernel(AMI::AMI_QUERY_FUNC func, const boost::intrusive_ptr<sst_message>& data);

    virtual void
    execute_kernel(AMI::AMI_DISP_FUNC func, const boost::intrusive_ptr<sst_message>& data);

    virtual void
    execute_kernel(AMI::AMI_HW_FUNC func, const boost::intrusive_ptr<sst_message>& data);

    virtual ptr_type::ptr
    execute_kernel(AMI::AMI_COMP_FUNC func);

    virtual ptr_type::ptr
    execute_kernel(AMI::AMI_COMM_FUNC func);

    virtual ptr_type::ptr
    execute_kernel(AMI::AMI_QUERY_FUNC func);

    virtual ptr_type::ptr
    execute_kernel(AMI::AMI_DISP_FUNC func);

    virtual ptr_type::ptr
    execute_kernel(AMI::AMI_HW_FUNC func);

    virtual bool kernel_supported(AMI::AMI_COMP_FUNC) const;
    virtual bool kernel_supported(AMI::AMI_COMM_FUNC) const;
    virtual bool kernel_supported(AMI::AMI_DISP_FUNC) const;
    virtual bool kernel_supported(AMI::AMI_QUERY_FUNC) const ;
    virtual bool kernel_supported(AMI::AMI_HW_FUNC) const;

    virtual timestamp
    now() const = 0; // for convenience, why not

    static const char*
    tostr(AMI_COMP_FUNC func);

    static const char*
    tostr(AMI_COMM_FUNC func);

    static const char*
    tostr(AMI_QUERY_FUNC func);

    static const char*
    tostr(AMI_DISP_FUNC func);

    static const char*
    tostr(AMI_HW_FUNC func);

    static const char*
    tostr(AMI_CAT cat);

  };

} } //end of namespace sstmac

#endif
