/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */


#include <sstmac/software/ami/environment.h>

#include <sstmac/common/errors.h>

#include <boost/foreach.hpp>

namespace sstmac { namespace sw
{

  
  

  environment::environment()
  {


  }

  environment::~environment()
  {

  }

  //
  // Allocate an object of this type.
  //
  environment::ptr
  environment::construct()
  {
    environment::ptr retval(new environment());
    //retval->set_self(retval);
    return retval;
  }

  std::string
  environment::getenv(const std::string &name) const
  {
      std::map<std::string,std::string>::const_iterator it = shell_env_.find(name);
      if (it == shell_env_.end()){
          return "";
      }
      return it->second;
  }

  void
  environment::serialize(sst_serializer* ser)
  {
  
    throw unimplementederror("environment::deserialize unimplemented");

    /*
    size_t s = mapping_.size();
    ser->pack(s);

    BOOST_FOREACH (map_type::value_type it, mapping_)
      {
        ser->pack(it.first);
        it.second->serialize(ser);
      }
    */
  }

  std::string
  environment::get_hostname(const taskid &tid) const
  {
      return hostnames_[tid.id];
  }

  void
  environment::add_hostname(long tid, const std::string &name, const nodeaddress::ptr &addr)
  {
      if ((int)hostnames_.size() <= tid){
          hostnames_.resize(tid + 1);
      }
      hostnames_[tid] = name;
      inet_addr_map_[name] = addr;
  }

  nodeaddress::ptr
  environment::get_node_address(long id) const
  {
      map_type::const_iterator it = mapping_.find(id);
      if (it == mapping_.end()){
          sst_throw(valueerror, "unable to find nodeaddress for task id %ld", id);
      }
      return it->second;
  }

  nodeaddress::ptr
  environment::get_node_address(const std::string& name) const
  {
    std::map<std::string, nodeaddress::ptr>::const_iterator it = inet_addr_map_.find(name);
    if (it == inet_addr_map_.end()){
        sst_throw(valueerror, "unable to find nodaddress for hostname %s", name.c_str());
    }
    return it->second;
  }

  environment::ptr
  environment::deserialize(sst_serializer* ser)
  {

    throw unimplementederror("environment::deserialize unimplemented");


    /*
    environment::ptr ret = environment::construct();
    size_t s;
    ser->unpack(s);

    for (size_t i = 0; i < s; i++)
      {
        int t;
        ser->unpack(t);
        taskid task(t);


        nodeaddress::ptr n = ser->get_deser_addr();
        n->deserialize(ser);

        ret->add_item(task, n);
      }

    return ret;
    */
  }

} } //end of namespace sstmac
