/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <iostream>
#include <sstmac/software/libraries/compute/sstmac_compute.h>
#include <sstmac/software/api/eiger/eigermodel.h>

using namespace sstmac;

static std::map<std::string, eigermodel::ptr> model_map_;

/// Find and (if necessary) memoize, then evaluate compute
void SSTMAC_compute_eiger(std::map<std::string, double> values, 
		std::string model_name){
	eigermodel::ptr model;
	std::map<std::string, eigermodel::ptr>::iterator it;
	if((it = model_map_.find(model_name)) == model_map_.end()){
		model = eigermodel::construct(model_name);
		model_map_[model_name] = model;
	}
	else{
		model = it->second;
	}
	double result = model->poll(values);
	SSTMAC_compute(result);
}

extern "C" void
SSTMAC_compute_eiger1(std::string model_name, std::string prefix,
		std::string name1, double val1){

	std::map<std::string, double> vals;
	vals[prefix + "_" + name1] = val1;
	SSTMAC_compute_eiger(vals, model_name);
}

extern "C" void
SSTMAC_compute_eiger2(std::string model_name, std::string prefix,
		std::string name1, double val1,
		std::string name2, double val2){
	std::map<std::string, double> vals;
	vals[prefix + "_" + name1] = val1;
	vals[prefix + "_" + name2] = val2;
	SSTMAC_compute_eiger(vals, model_name);
}

extern "C" void
SSTMAC_compute_eiger3(std::string model_name, std::string prefix,
		std::string name1, double val1,
		std::string name2, double val2,
		std::string name3, double val3){
	std::map<std::string, double> vals;
	vals[prefix + "_" + name1] = val1;
	vals[prefix + "_" + name2] = val2;
	vals[prefix + "_" + name3] = val3;
	SSTMAC_compute_eiger(vals, model_name);
}

extern "C" void
SSTMAC_compute_eiger4(std::string model_name, std::string prefix,
		std::string name1, double val1,
		std::string name2, double val2,
		std::string name3, double val3,
		std::string name4, double val4){
	std::map<std::string, double> vals;
	vals[prefix + "_" + name1] = val1;
	vals[prefix + "_" + name2] = val2;
	vals[prefix + "_" + name3] = val3;
	vals[prefix + "_" + name4] = val4;
	SSTMAC_compute_eiger(vals, model_name);
}

extern "C" void
SSTMAC_compute_eiger5(std::string model_name, std::string prefix,
		std::string name1, double val1,
		std::string name2, double val2,
		std::string name3, double val3,
		std::string name4, double val4,
		std::string name5, double val5){
	std::map<std::string, double> vals;
	vals[prefix + "_" + name1] = val1;
	vals[prefix + "_" + name2] = val2;
	vals[prefix + "_" + name3] = val3;
	vals[prefix + "_" + name4] = val4;
	vals[prefix + "_" + name5] = val5;
	SSTMAC_compute_eiger(vals, model_name);
}

extern "C" void
SSTMAC_compute_eiger6(std::string model_name, std::string prefix,
		std::string name1, double val1,
		std::string name2, double val2,
		std::string name3, double val3,
		std::string name4, double val4,
		std::string name5, double val5,
		std::string name6, double val6){
	std::map<std::string, double> vals;
	vals[prefix + "_" + name1] = val1;
	vals[prefix + "_" + name2] = val2;
	vals[prefix + "_" + name3] = val3;
	vals[prefix + "_" + name4] = val4;
	vals[prefix + "_" + name5] = val5;
	vals[prefix + "_" + name6] = val6;
	SSTMAC_compute_eiger(vals, model_name);
}

extern "C" void
SSTMAC_compute_eiger7(std::string model_name, std::string prefix,
		std::string name1, double val1,
		std::string name2, double val2,
		std::string name3, double val3,
		std::string name4, double val4,
		std::string name5, double val5,
		std::string name6, double val6,
		std::string name7, double val7){
	std::map<std::string, double> vals;
	vals[prefix + "_" + name1] = val1;
	vals[prefix + "_" + name2] = val2;
	vals[prefix + "_" + name3] = val3;
	vals[prefix + "_" + name4] = val4;
	vals[prefix + "_" + name5] = val5;
	vals[prefix + "_" + name6] = val6;
	vals[prefix + "_" + name7] = val7;
	SSTMAC_compute_eiger(vals, model_name);
}

