#include <sstmac/software/api/gni/gniapi.h>

#define nic_printf "%lu"
#define mem_printf "%lu"

ImplementAPI(sstmac::sw, gniapi,"gni");

namespace sstmac {
    namespace sw {


void
gniapi::gni_cdm_create(
    uint32_t inst_id,
    uint8_t ptag,
    uint32_t cookie,
    uint32_t modes,
    gni_cdm_handle_t *cdm_hndl
)
{
    cdm_handles_.create(cdm_hndl);
}


void
gniapi::gni_cdm_destroy(gni_cdm_handle_t cdm_hndl)
{
    cdm_handles_.free(cdm_hndl);
}


void
gniapi::gni_cdm_get_nic_address(
    uint32_t device_id,
    uint32_t *address,
    uint32_t *cpu_id
)
{
    *address = get_os()->get_my_addr()->unique_id();
    *cpu_id = 0;
}


void
gniapi::gni_cdm_attach(
    gni_cdm_handle_t cdm_hndl,
    uint32_t device_id,
    uint32_t *local_addr,
    gni_nic_handle_t *nic_hndl
)
{
    communication_domain& cdm = cdm_handles_.get_item(cdm_hndl);
    cdm.attached = true;
    *local_addr = get_os()->get_my_addr()->unique_id();
    nic_handles_.create(nic_hndl);
}


void
gniapi::gni_ep_create(
    gni_nic_handle_t nic_hndl,
    gni_cq_handle_t src_cq_hndl,
    gni_ep_handle_t *ep_hndl
)
{
    nic_handles_.validate(nic_hndl);
    cq_handles_.validate(src_cq_hndl);
    ep_handles_.create(ep_hndl);
}


void
gniapi::gni_ep_bind(
    gni_ep_handle_t ep_hndl,
    uint32_t remote_addr,
    uint32_t remote_id
)
{
    endpoint& ep = ep_handles_.get_item(ep_hndl);
    ep.addr = nodeaddress::convert_from_uniqueid(remote_addr);
    ep.bound = true;
}


void
gniapi::gni_ep_unbind(gni_ep_handle_t ep_hndl)
{
    endpoint& ep = ep_handles_.get_item(ep_hndl);
    ep.bound = false;
}


void
gniapi::gni_ep_destroy(gni_ep_handle_t ep_hndl)
{
    ep_handles_.free(ep_hndl);
}


void
gniapi::gni_mem_register(
    gni_nic_handle_t nic_hndl,
    uint64_t address,
    uint64_t length,
    gni_cq_handle_t dst_cq_hndl,
    uint32_t flags,
    uint32_t vmdh_index,
    gni_mem_handle_t *mem_hndl
)
{
    mem_registration& reg = mem_handles_.create(mem_hndl);
    reg.length = length;
    reg.address = address;
    reg.nic = nic_hndl;
    reg.cq = dst_cq_hndl;
}


void
gniapi::gni_mem_deregister(
    gni_nic_handle_t nic_hndl,
    gni_mem_handle_t *mem_hndl
)
{
    mem_registration& reg = mem_handles_.free(*mem_hndl);
    if (reg.nic != nic_hndl){
        sst_throw(gni_invalid_param,
                 "invalid nic handle " nic_printf " in mem_deregister",
                  nic_hndl);
    }
    *mem_hndl = -1;
}


void
gniapi::gni_cq_create(
    gni_nic_handle_t nic_hndl,
    uint32_t entry_count,
    uint32_t delay_count,
    gni_cq_mode_t mode,
    void (*handler)(gni_cq_entry_t*,void*),
    void *context,
    gni_cq_handle_t *cq_hndl
)
{
    nic_handles_.validate(nic_hndl);
    completion_queue& cq = cq_handles_.create(cq_hndl);
    if (handler || context){
        sst_throw(unimplementederror, "SST/macro GNI does not yet support CQ callback handlers");
    }
    if (mode == GNI_CQ_BLOCKING || delay_count != 0){
        sst_throw(unimplementederror, "SST/macro GNI does not yet support blocking CQ");
    }
    cq.entry_count = entry_count;
}


void
gniapi::gni_cq_destroy(gni_cq_handle_t cq_hndl)
{
    cq_handles_.free(cq_hndl);
}


void
gniapi::gni_post_rdma(
    gni_ep_handle_t ep_hndl,
    gni_post_descriptor_t *post_descr
)
{
    throw unimplementederror("gni_post_rdma");
}


void
gniapi::gni_post_rdma_get(
    gni_ep_handle_t ep_hndl,
    gni_mem_handle_t remote_mem_hndl
)
{
    throw unimplementederror("gni_post_rdma_get");
}


void
gniapi::gni_post_rdma_get(
    gni_ep_handle_t ep_hndl,
    void* buffer,
    uint64_t length
)
{
    throw unimplementederror("gni_post_rdma_get");
}


void
gniapi::gni_post_rdma_put(
    gni_ep_handle_t ep_hndl,
    gni_mem_handle_t remote_mem_hndl
)
{
    throw unimplementederror("gni_post_rdma_put");
}


void
gniapi::gni_post_rdma_put(
    gni_ep_handle_t ep_hndl,
    void* buffer,
    uint64_t length
)
{
    throw unimplementederror("gni_post_rdma_put");
}


void
gniapi::gni_post_fma(
    gni_ep_handle_t ep_hndl,
    gni_post_descriptor_t *post_descr
)
{
    throw unimplementederror("gni_post_fma");
}


void
gniapi::gni_post_fma_get(
    gni_ep_handle_t ep_hndl,
    gni_mem_handle_t remote_mem_hndl
)
{
    throw unimplementederror("gni_post_fma_get");
}


void
gniapi::gni_post_fma_get(
    gni_ep_handle_t ep_hndl,
    void* buffer,
    uint64_t length
)
{
    throw unimplementederror("gni_post_fma_get");
}


void
gniapi::gni_post_fma_put(
    gni_ep_handle_t ep_hndl,
    gni_mem_handle_t remote_mem_hndl
)
{
    throw unimplementederror("gni_post_fma_put");
}


void
gniapi::gni_post_fma_put(
    gni_ep_handle_t ep_hndl,
    void* buffer,
    uint64_t length
)
{
    throw unimplementederror("gni_post_fma_put");
}


void
gniapi::gni_get_completed(
    gni_cq_handle_t cq_hndl,
    gni_cq_entry_t event_data,
    gni_post_descriptor_t **post_descr
)
{
    throw unimplementederror("gni_get_completed");
}


void
gniapi::gni_cq_get_event(
    gni_cq_handle_t cq_hndl,
    gni_cq_entry_t *event_data
)
{
    throw unimplementederror("gni_cq_get_event");
}


void
gniapi::gni_cq_wait_event(
    gni_cq_handle_t cq_hndl,
    uint64_t timeout,
    gni_cq_entry_t *event_data
)
{
    throw unimplementederror("gni_cq_wait_event");
}


void
gniapi::gni_cq_test_event(gni_cq_handle_t cq_hndl)
{
    throw unimplementederror("gni_cq_test_event");
}


void
gniapi::gni_smsg_init(
    gni_ep_handle_t ep_hndl,
    gni_smsg_attr_t *local_smsg_attr,
    gni_smsg_attr_t *remote_smsg_attr
)
{
    endpoint& ep = ep_handles_.get_item(ep_hndl);
    if (ep.local_smsg_attr){
        sst_throw(gni_invalid_param, "gni_smsg_init: ep handle already has smsg mbox");
    }
    ep.local_smsg_attr = new gni_smsg_attr_t;
    ep.remote_smsg_attr = new gni_smsg_attr_t;
    ::memcpy(ep.local_smsg_attr, local_smsg_attr, sizeof(gni_smsg_attr_t));
    ::memcpy(ep.remote_smsg_attr, remote_smsg_attr, sizeof(gni_smsg_attr_t));
}


void
gniapi::gni_smsg_send(
    gni_ep_handle_t ep_hndl,
    void *header,
    uint32_t header_length,
    void *data,
    uint32_t data_length,
    uint32_t msg_id
)
{
    throw unimplementederror("gni_smsg_send");
}


void
gniapi::gni_smsg_send(
    gni_ep_handle_t ep_hndl,
    void *header,
    uint32_t header_length
)
{
    throw unimplementederror("gni_smsg_send");
}


void
gniapi::gni_smsg_send_wtag(
    gni_ep_handle_t ep_hndl,
    void *header,
    uint32_t header_length,
    void *data,
    uint32_t data_length,
    uint32_t msg_id,
    uint8_t tag
)
{
    throw unimplementederror("gni_smsg_send_wtag");
}


void
gniapi::gni_smsg_get_next(
    gni_ep_handle_t ep_hndl,
    void **header
)
{
    throw unimplementederror("gni_smsg_get_next");
}


void
gniapi::gni_smsg_get_next_wtag(
    gni_ep_handle_t ep_hndl,
    void **header,
    uint8_t *tag
)
{
    throw unimplementederror("gni_smsg_get_next_wtag");
}


void
gniapi::gni_smsg_release(gni_ep_handle_t ep_hndl)
{
    throw unimplementederror("gni_smsg_release");
}


void
gniapi::gni_smsg_set_max_retrans(
    gni_nic_handle_t gni_handle,
    uint16_t max_retrans
)
{
    throw unimplementederror("gni_smsg_set_max_retrans");
}

  }
} //end namespace
