/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/hpx/agas/agas_dist.h>

namespace hpx
{

  void
  agas_dist::handle(const sstmac::sst_message::ptr &msg)
  {
    //once component migration is enabled, need to be able to  handle it here?
  }

  bool
  agas_dist::new_gid(lva::ENTITY_TYPE t, globalid &gid,
      unsigned long long &waitid)
  {
    globalid ret(gid_count_++);
    lva l(ret, locality_, t);
    lvamap_[ret.gid_] = l;
    gid = ret;
    return true;
  }

  bool
  agas_dist::resolve_locality(const globalid &gid,
      sstmac::nodeaddress::ptr &loc, unsigned long long &waitid)
  {
    if (lvamap_.find(gid.gid_) != lvamap_.end())
      {
        loc = lvamap_[gid.gid_].addr();
        return true;
      }
    else
      {
        //it's not here, so send it to it's home base
        sstmac::nodeaddress::ptr addr =
            sstmac::nodeaddress::convert_from_uniqueid(gid.get_node_uid());

        loc = addr;
        return true;
      }
  }

  void
  agas_dist::gid_move_complete(globalid gid)
  {
    lva l = lvamap_[gid.gid_];
    lva newl(gid.gid_, locality_, l.etype());
    lvamap_[gid.gid_] = newl;
  }

  void
  agas_dist::gid_moving(globalid gid, sstmac::nodeaddress::ptr to)
  {
    lva l = lvamap_[gid.gid_];
    lva newl(gid.gid_, to, l.etype());
    lvamap_[gid.gid_] = newl;
  }

}
