/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_AGAS_AGAS_IDEAL_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_AGAS_AGAS_IDEAL_H_INCLUDED

#include <sstmac/software/api/hpx/agas/agas_server.h>
#include <sstmac/software/api/hpx/agas/globalid.h>
#include <sstmac/software/api/hpx/agas/lva.h>

namespace hpx
{

  class agas_ideal : public agas_server
  {

  protected:
    static unsigned long long gid_count_;
    sstmac::nodeaddress::ptr locality_;

    static boost::unordered_map<unsigned long long, lva> lvamap_;

  public:
    typedef boost::intrusive_ptr<agas_ideal> ptr;

    static ptr
    construct()
    {
      return ptr(new agas_ideal());
    }

    virtual
    ~agas_ideal()
    {
    }

    virtual void
    handle(const sstmac::sst_message::ptr &msg);

    virtual service::ptr
    clone()
    {
      return construct();
    }

    virtual void
    gid_move_complete(globalid gid);

    virtual void
    gid_moving(globalid gid, sstmac::nodeaddress::ptr to);

    virtual void
    start()
    {
      service::start();

      locality_ = os_->get_my_addr();
      //gid_count_ = locality_->unique_id() * globalid::get_quantum();
    }

    static std::string
    lib_name()
    {
      return "hpx_agas_ideal";
    }

    // ------ these are the useful functions ------ //

    virtual bool
    new_gid(lva::ENTITY_TYPE t, globalid &gid, unsigned long long &waitid);

    virtual bool
    resolve_locality(const globalid &gid, sstmac::nodeaddress::ptr &loc,
        unsigned long long &waitid);

    // -------------------------------------- //

  protected:
    agas_ideal()
    {

      libname_ = lib_name();
    }

  };

}

#endif
