/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_AGAS_AGAS_SERVER_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_AGAS_AGAS_SERVER_H_INCLUDED

#include <sstmac/software/services/service.h>
#include <sstmac/software/api/hpx/agas/globalid.h>
#include <sstmac/software/api/hpx/agas/lva.h>

namespace hpx
{

  class hpx_thread_manager;

  class agas_server : public sstmac::sw::service
  {

  protected:
    static sstmac::debug dbg_;

    boost::unordered_map<unsigned long long, sstmac::sw::key::ptr>
        newgid_waiting_;
    boost::unordered_map<unsigned long long, globalid> newgid_value_;

    boost::intrusive_ptr<hpx_thread_manager> threadman_;

  public:
    typedef boost::intrusive_ptr<agas_server> ptr;

    virtual
    ~agas_server();

    void
    set_threadman(boost::intrusive_ptr<hpx_thread_manager> a);

    virtual void
    gid_move_complete(globalid gid)
    {

    }

    virtual void
    gid_moving(globalid gid, sstmac::nodeaddress::ptr to){

    }

    // ------ these are the useful functions ------ //

    virtual bool
    new_gid(lva::ENTITY_TYPE t, globalid &gid, unsigned long long &waitid) = 0;

    /**
     * careful when using this, it blocks.  so don't call it from main thread.
     * @param
     * @return
     */
    virtual globalid
    new_gid(lva::ENTITY_TYPE t);

    virtual bool
    resolve_locality(const globalid &gid, sstmac::nodeaddress::ptr &loc,
        unsigned long long &waitid) = 0;

    // -------------------------------------- //

  protected:
    agas_server();

  };

}

#endif
