/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/hpx/agas/globalid.h>
#include <sstmac/common/errors.h>
#include <iostream>
#include <sstream>

namespace hpx
{

  long globalid::num_localities_ = -1;
  long long globalid::quantum_ = -1;

  //
  // Test whether two node ids match.
  //
  bool
  globalid::matches(const globalid &other) const
  {
    return ((this->gid_ == other.gid_));
  }

  //
  // Increment (prefix).
  //
  globalid&
  operator++(globalid &value)
  {
    ++value.gid_;
    return value;
  }

  //
  // Increment (postfix).
  //
  globalid
  operator++(globalid &value, int)
  {
    globalid rv(value);
    ++value.gid_;
    return rv;
  }

  //
  // Test two node ids for strict (in)equality (stronger than match).
  //
  bool
  operator==(const globalid &a, const globalid &b)
  {
    return a.gid_ == b.gid_;
  }
  bool
  operator!=(const globalid &a, const globalid &b)
  {
    return !(a == b);
  }

  //
  // Rank-compare operations.
  //
  bool
  operator<(const globalid &a, const globalid &b)
  {
    return (a.gid_ < b.gid_);
  }
  bool
  operator>(const globalid &a, const globalid &b)
  {
    return (a.gid_ > b.gid_);
  }
  bool
  operator<=(const globalid &a, const globalid &b)
  {
    return (a.gid_ <= b.gid_);
  }
  bool
  operator>=(const globalid &a, const globalid &b)
  {
    return (a.gid_ >= b.gid_);
  }

  globalid
  operator+(const globalid &a, const globalid &b)
  {
    return globalid(a.gid_ + b.gid_);
  }

  std::string
  globalid::to_string() const
  {
    std::stringstream ss(std::stringstream::in | std::stringstream::out);
    ss << "globalid(id=" << gid_ << ")";
    return ss.str();
  }

  //
  // Fairly self-explanatory.
  //
  std::ostream&
  operator<<(std::ostream &os, const globalid &id)
  {
    os << id.to_string();
    return os;
  }

}

namespace boost
{
  size_t
  hash_value(const hpx::globalid &t)
  {
    // std::cout << "calling hash_value, returning " << t->unique_id()
    //     << std::endl;
    return t.gid_;
  }
}
