/*
 * coremodel.h
 *
 *  Created on: Oct 26, 2012
 *      Author: ghendry
 */

#ifndef SSTMAC_HPX_COREMODEL_H_
#define SSTMAC_HPX_COREMODEL_H_

#include <sstmac/common/ptr_type.h>
#include <sstmac/software/api/hpx/runtime/hpx_runnable.h>
#include <sstmac/common/types.h>
#include <sstmac/common/eventmanager.h>
#include <sstmac/common/stats/stats_common.h>

namespace hpx
{

  class coremodel : public sstmac::ptr_type
  {
  protected:
    coremodel(int i, int swaps) :
        id_(i), started_(0)
    {
      swaps_ = swaps;

      //swap_vec_.resize(swaps);
      // swap_time_.resize(swaps);

      // for (int i = 0; i < swaps; i++)
      //   {
      //     swap_time_[i] = sstmac::timestamp(-1);
      //  }

      if (!dbg_.is_real())
        {
          dbg_ = sstmac::debug("<debug> hpx | hpx_coremodel | coremodel",
              sstmac::logger::cyan);
          sstmac::logger::register_logger(&dbg_);
        }

      /*if (!stat_tq_)
       {
       stat_tq_ = sstmac::stat_time_window_time_avg::construct(
       "<stats> hpx_coremodel", "coremodel_tq.csv");
       sstmac::eventmanager::register_stat(stat_tq_);
       stat_tq_->record(sstmac::timestamp(0), 0);
       */
      if (!compute_time_)
        {
          compute_time_ = sstmac::stat_sum::construct("<stats> hpx_compute",
              "compute_time.csv");
          sstmac::eventmanager::register_stat(compute_time_);
        }

      numcores_++;
      busy_ = false;
    }

    sstmac::debug dbg_;

    int swaps_;
    int id_;

    std::map<hpx_runnable::ptr, int> swap_vec_;
    //type<sstmac::timestamp>::vector swap_time_;

    //static sstmac::stat_time_window_time_avg::ptr stat_tq_;
    static long numcores_;

    static sstmac::stat_sum::ptr compute_time_;
    double compute_started_;

    bool busy_;

    sstmac::timestamp started_;

  public:
    typedef boost::intrusive_ptr<coremodel> ptr;

    static ptr
    construct(int i, int swaps)
    {
      return ptr(new coremodel(i, swaps));
    }

    bool
    get_busy() const
    {
      return busy_;
    }

    void
    set_busy(bool b)
    {
      busy_ = b;

    }

    void
    thread_start(sstmac::timestamp now)
    {
      started_ = now;
    }

    void
    thread_stop(sstmac::timestamp now)
    {

      double t = now.sec() - started_.sec();
      compute_time_->record(t / (double)numcores_);
    }

    bool
    in_swaps(hpx_runnable::ptr t);

    void
    clear_from_swap(hpx_runnable::ptr t);

    void
    put_in_swaps(hpx_runnable::ptr t);

    virtual
    ~coremodel()
    {
    }

    int
    id() const
    {
      return id_;
    }

    virtual std::string
    to_string() const
    {
      return "hpx::coremodel";
    }

  };

}

#endif /* COREMODEL_H_ */
