/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_LCOS_FUTURE_WAIT_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_LCOS_FUTURE_WAIT_H_INCLUDED

#include <sstmac/software/api/hpx/lcos/lco.h>
#include <sstmac/software/api/hpx/lcos/promise.h>
#include <sstmac/software/api/hpx/components/hpx_component.h>
#include <sstmac/software/api/hpx/runtime/actions/hpx_action_params.h>

namespace hpx
{
  namespace lcos
  {

    inline void
    wait(boost::intrusive_ptr<base_lco_result> pro)
    {
      pro->get_value();
    }

    inline void
    wait(std::vector<boost::intrusive_ptr<base_lco_result> > pros)
    {
      for (size_t i = 0; i < pros.size(); i++)
      {
        pros[i]->get_value();
      }
    }

    template<typename ResultType>
    inline void
    wait(boost::intrusive_ptr<promise<ResultType> > pro)
    {
      pro->get_value();
    }

    template<typename ResultType>
    inline void
    wait(std::vector<boost::intrusive_ptr<promise<ResultType> > > pros)
    {
      for (size_t i = 0; i < pros.size(); i++)
      {
        pros[i]->get_value();
      }
    }

    template<typename ResultType, typename ObjType>
      inline void
      wait(std::vector<boost::intrusive_ptr<promise<ResultType> > > pros, bool
      (ObjType::*CB)(size_t i, ResultType const& res),
          hpx::components::hpx_component::ptr obj)
      {
        boost::intrusive_ptr<ObjType> ccast = boost::dynamic_pointer_cast<
            ObjType>(obj);
        for (size_t i = 0; i < pros.size(); i++)
        {
          ResultType result = pros[i]->get();

          (ccast.get()->*CB)(i, result);

        }
      }

    template<typename ResultType, typename ObjType, typename Arg1Type>
      inline void
      wait(std::vector<boost::intrusive_ptr<promise<ResultType> > > pros, bool
      (ObjType::*CB)(size_t i, ResultType const& res, Arg1Type arg1),
          hpx::components::hpx_component::ptr obj, Arg1Type arg1)
      {
        boost::intrusive_ptr<ObjType> ccast = boost::dynamic_pointer_cast<
            ObjType>(obj);
        for (size_t i = 0; i < pros.size(); i++)
        {
          ResultType result = pros[i]->get();

          (ccast.get()->*CB)(i, result, arg1);

        }
      }

  }
}

#endif
