/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_move_component_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_move_component_H_INCLUDED

#include <sstmac/software/api/hpx/parcels/parcel.h>
#include <sstmac/software/api/hpx/hpx_app.h>
#include <sstmac/software/api/hpx/components/hpx_component.h>

namespace hpx
{

  namespace parcels
  {

    class move_component : public parcel
    {

    public:
      typedef boost::intrusive_ptr<move_component> ptr;

      static ptr
      construct(hpx::components::hpx_component::ptr comp, sstmac::timestamp now)
      {
        if (stats_[MOVE_COMPONENT] == sstmac::stat_time_window_sum::ptr())
          {
            stats_[MOVE_COMPONENT] = sstmac::stat_time_window_sum::construct(
                "<stats> hpx_parcels | hpx", "hpx_parcels_move_component.csv");
          }
        stats_[MOVE_COMPONENT]->record(now, 1);

        return ptr(new move_component(comp));
      }

      virtual long
      get_byte_length() const
      {
        return 16 + comp_->get_byte_size();
      }

      hpx::components::hpx_component::ptr
      get_comp() {
        return comp_;
      }

    protected:
      move_component(hpx::components::hpx_component::ptr comp) :
          sst_message(HPX),
          parcel(MOVE_COMPONENT), comp_(comp)
      {

      }


      hpx::components::hpx_component::ptr comp_;
    };

  }
}
#endif
