/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_PAYLOADIZATION_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_PAYLOADIZATION_H_INCLUDED

#include <sstmac/common/messages/valuepayload.h>
#include <sstmac/common/messages/vectorpayload.h>

namespace hpx
{
  namespace parcels
  {

    /* template<typename Type>
     class payloadizer : public sstmac::ptr_type
     {
     protected:
     payloadizer()
     {
     }

     public:
     typedef boost::intrusive_ptr<payloadizer<Type> > ptr;

     static ptr
     construct()
     {
     return ptr(new payloadizer());
     }

     virtual
     ~payloadizer()
     {
     }

     virtual std::string
     to_string() const
     {
     return "payloadizer";
     }

     virtual sstmac::payload::const_ptr
     payloadize(Type t)
     {
     return sstmac::valuepayload<Type>::construct(t);
     }
     };

     template<typename Type, typename VectorType>
     class vector1_payloadizer : public payloadizer
     {
     protected:
     vector1_payloadizer() :
     payloadizer()
     {
     }

     public:
     typedef boost::intrusive_ptr<vector1_payloadizer<Type, VectorType> > ptr;

     static ptr
     construct()
     {
     return ptr(new vector1_payloadizer());
     }

     virtual
     ~vector1_payloadizer()
     {
     }

     virtual std::string
     to_string() const
     {
     return "vector1_payloadizer";
     }

     virtual sstmac::payload::const_ptr
     payloadize(VectorType t)
     {
     return sstmac::vector1payload<Type, VectorType>::construct(t,
     t.size());
     }
     };
     */

    template<typename Type>
      sstmac::payload::const_ptr
      payloadizer(Type t)
      {
        return sstmac::valuepayload<Type>::construct(t);
      }

    template<typename Type, typename VectorType>
      sstmac::payload::const_ptr
      vector1_payloadizer(VectorType t)
      {
        return sstmac::vector1payload<Type, VectorType>::construct(t, t.size());
      }

    template<typename Type>
      class depayloadizer : public sstmac::ptr_type
      {
      protected:
        depayloadizer()
        {
        }

      public:
        typedef boost::intrusive_ptr<depayloadizer<Type> > ptr;

        static ptr
        construct()
        {
          return ptr(new depayloadizer());
        }

        virtual
        ~depayloadizer()
        {
        }

        virtual std::string
        to_string() const
        {
          return "depayloadizer";
        }

        virtual Type
        depayloadize(sstmac::payload::const_ptr pay)
        {
          boost::intrusive_ptr<const sstmac::valuepayload<Type> > casted =
              boost::dynamic_pointer_cast<const sstmac::valuepayload<Type> >(
                  pay);

          if (!casted)
            {
              std::cout << "value: " << pay->to_string() << "\n";
              std::cout << "type: " << typeid(Type).name() << "\n";

              throw sstmac::ssterror(
                  "depayloadizer() -- payload didn't cast, check the result type");
            }
          return casted->data();
        }
      };

    template<typename Type, typename VectorType>
      class vector1_depayloadizer : public depayloadizer<VectorType>
      {
      protected:
        vector1_depayloadizer()
        {
        }

      public:
        typedef boost::intrusive_ptr<vector1_depayloadizer<Type, VectorType> > ptr;

        static ptr
        construct()
        {
          return ptr(new vector1_depayloadizer());
        }

        virtual
        ~vector1_depayloadizer()
        {
        }

        virtual std::string
        to_string() const
        {
          return "vector1_depayloadizer";
        }

        virtual VectorType
        depayloadize(sstmac::payload::const_ptr pay)
        {
          boost::intrusive_ptr<const sstmac::vector1payload<Type, VectorType> > casted =
              boost::dynamic_pointer_cast<
                  const sstmac::vector1payload<Type, VectorType> >(pay);

          if (!casted)
            {
              std::cout << "value: " << pay->to_string() << "\n";
              std::cout << "type: " << typeid(Type).name() << "\n";

              throw sstmac::ssterror(
                  "depayloadizer() -- payload didn't cast, check the result type");
            }

          const Type* buf = casted->data();
          VectorType ret;
          for(int i = 0; i < casted->get_dim(); i++){
              ret.push_back(buf[i]);
          }
          return ret;
        }
      };
  }
}
#endif
