/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstream>

#include <sstmac/software/api/hpx/runtime/hpx_action_thread.h>
#include <sstmac/software/api/hpx/runtime/hpx_thread_manager.h>

namespace hpx
{

  using namespace sstmac;
  using namespace std;

  long hpx_action_thread::COMPUTE_LIB_COUNT = 0;
  static sstmac::sw::key::category waitqueue_key_cat("HPX::action_thread::waitqueue");
  void
  hpx_action_thread::run()
  {

    SSTMAC_DEBUG << "hpx_action_thread -- computing creation overhead.. \n";

    std::stringstream ss;
    ss << "hpx_action_thread_computelib-" << COMPUTE_LIB_COUNT++;
    libcomp_ = sstmac::sw::lib_compute_time::construct(ss.str());
    os_->register_lib(libcomp_);

    ss.str(std::string());

    ss << "hpx_action_thread_sleeplib-" << COMPUTE_LIB_COUNT++;
    libsleep_ = sstmac::sw::lib_sleep::construct(ss.str());
    os_->register_lib(libsleep_);

    threadman_->thread_is_starting(ptr(this));

    queuekey_ = sstmac::sw::key::construct(waitqueue_key_cat);
    //queuekey_->set_event(sstmac::sw::key::sleep);

    SSTMAC_DEBUG << "hpx_action_thread: running thread \n";

    while (!killed_)
    {
      if (params_.size() == 0)
      {
        sst_throw(ssterror, "hpx_action_thread::run - no params to operate on, should never be the case");
      }
      started_ = true;
      sstmac::payload::const_ptr res;
      hpx_action_params::ptr thisp = params_.front();
      lco_type thislco = lco_.front();
      if (comp_)
        res = action_->act(comp_, thisp);
      else
        res = action_->act(thisp);

      thislco->set_value(res, os_);

      params_.pop();
      lco_.pop();

      if (params_.size() == 0)
      {
        waiting_ = true;
        threadman_->thread_is_blocking(ptr(this), queuekey_);

      }

    }

    threadman_->thread_is_finished(ptr(this));

    // os_->unregister_lib(libcomp_);
  }

  void
  hpx_action_thread::wake_up()
  {
    if (waiting_)
    {
      threadman_->thread_is_unblocking(ptr(this), queuekey_);
      waiting_ = false;
    }
  }
}

