/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_RUNTIME_HPX_ACTION_THREAD_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_RUNTIME_HPX_ACTION_THREAD_H_INCLUDED

#include <sstmac/software/api/hpx/runtime/hpx_thread.h>
#include <sstmac/software/api/hpx/runtime/actions/hpx_action.h>
#include <sstmac/software/api/hpx/lcos/base_lco_result.h>
#include <sstmac/software/api/hpx/components/hpx_component.h>

namespace hpx
{

  class hpx_action_thread : public hpx_thread
  {
  protected:
    typedef boost::intrusive_ptr<hpx::lcos::base_lco_result> lco_type;

    boost::intrusive_ptr<actions::hpx_action> action_;
    std::queue<hpx_action_params::ptr> params_;
    std::queue<lco_type> lco_;
    hpx::components::hpx_component::ptr comp_;

    bool killed_;
    bool waiting_;
    bool started_;


    sstmac::sw::key::ptr queuekey_;

  public:
    typedef boost::intrusive_ptr<hpx_action_thread> ptr;

    static ptr
    construct(boost::intrusive_ptr<hpx_thread_manager> man,
        boost::intrusive_ptr<agas_server> agas,
        boost::intrusive_ptr<actions::hpx_action> act,
        hpx_action_params::ptr params, lco_type prom,
        sstmac::sw::softwareid sid)
    {
      return ptr(new hpx_action_thread(man, agas, act, params, prom, sid));
    }

    static ptr
    construct(boost::intrusive_ptr<hpx_thread_manager> man,
        boost::intrusive_ptr<agas_server> agas,
        boost::intrusive_ptr<actions::hpx_action> act,
        hpx_action_params::ptr params, lco_type prom,
        hpx::components::hpx_component::ptr comp, sstmac::sw::softwareid sid)
    {
      return ptr(new hpx_action_thread(man, agas, act, params, prom, comp, sid));
    }

    virtual
    ~hpx_action_thread()
    {
    }

    virtual void
    wake_up();

    virtual void
    add_action(hpx_action_params::ptr params, lco_type prom)
    {
      params_.push(params);
      lco_.push(prom);
      if(started_)
        wake_up();
    }

    virtual void
    kill()
    {
      killed_ = true;
      wake_up();
      thread::kill();
    }

  protected:
    hpx_action_thread(boost::intrusive_ptr<hpx_thread_manager> man,
        boost::intrusive_ptr<agas_server> agas,
        boost::intrusive_ptr<actions::hpx_action> act,
        hpx_action_params::ptr params, lco_type prom,
        hpx::components::hpx_component::ptr comp, sstmac::sw::softwareid sid) :
      hpx_thread(man, agas, sid), action_(act), comp_(comp), killed_(false),
      waiting_(false), started_(false)
    {
      params_.push(params);
      lco_.push(prom);
    }

    hpx_action_thread(boost::intrusive_ptr<hpx_thread_manager> man,
        boost::intrusive_ptr<agas_server> agas,
        boost::intrusive_ptr<actions::hpx_action> act,
        hpx_action_params::ptr params, lco_type prom,
        sstmac::sw::softwareid sid) :
      hpx_thread(man, agas, sid), action_(act),
          comp_(hpx::components::hpx_component::ptr()), killed_(false),
          waiting_(false), started_(false)
    {
      params_.push(params);
      lco_.push(prom);
    }

    static long COMPUTE_LIB_COUNT;

    virtual void
    run();

  };
}

#endif
