/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HPX_RUNTIME_hpx_random_balancer_H_INCLUDED
#define SSTMAC_HPX_RUNTIME_hpx_random_balancer_H_INCLUDED

#include <sstmac/software/api/hpx/runtime/hpx_load_balancer.h>
#include <sstmac/common/rng.h>
#include <sstmac/common/sstmac_env.h>
namespace hpx
{

  class hpx_random_balancer : public hpx_load_balancer
  {

  protected:

    hpx_random_balancer(double t) :
      hpx_load_balancer(), thres_(t)
    {
      if (!rng_)
        {
          std::vector<RNG::rngint_t> seeds;
          seeds.push_back(time(NULL));
          rng_ = RNG::Combo::construct(seeds);
        }
    }

    static RNG::Combo::ptr rng_;
    double thres_;

  public:
    typedef boost::intrusive_ptr<hpx_random_balancer> ptr;

    static ptr
    construct()
    {
      double t = sstmac::sstmac_env::params->get_double_param(
          "hpx_lb_threshold");
      return ptr(new hpx_random_balancer(t));
    }

    ~hpx_random_balancer()
    {

    }

    virtual std::string
    to_string() const
    {
      return "hpx_random_balancer";
    }

    void
    balance(
        boost::unordered_map<long, boost::intrusive_ptr<
            hpx::components::hpx_component> > comps,
        sstmac::sw::environment::const_ptr env,
        boost::unordered_map<hpx::components::hpx_component::ptr,
            sstmac::nodeaddress::ptr> &change);
  };

}

#endif
