#ifndef HPX_RUNTIME_HPX_RUNNABLE_H_
#define HPX_RUNTIME_HPX_RUNNABLE_H_

#include <sstmac/common/ptr_type.h>
#include <sstmac/software/process/key.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/sstmac_env.h>
namespace hpx
{

  class hpx_runnable : public virtual sstmac::ptr_type
  {

  protected:
    hpx_runnable()
    {
      // blocker_ = sstmac::sw::key::construct();
      assigned_core_ = -1;

      if (THREAD_CREATION_OVERHEAD < 0)
        {

          THREAD_CREATION_OVERHEAD
              = sstmac::sstmac_env::params->get_optional_double_param(
                  "hpx_thread_creation", 1e-6);
          THREAD_SWAP_FROM_MEM_OVERHEAD
              = sstmac::sstmac_env::params->get_optional_double_param(
                  "hpx_thread_swap_from_mem", 1e-7);
          THREAD_SWAP_FAST_OVERHEAD
              = sstmac::sstmac_env::params->get_optional_double_param(
                  "hpx_thread_swap_fast", 5e-9);
        }
      start_ = true;
    }

    sstmac::sw::key::ptr blocker_;
    int assigned_core_;
    bool start_;
  public:
    typedef boost::intrusive_ptr<hpx_runnable> ptr;

    static double THREAD_CREATION_OVERHEAD;
    static double THREAD_SWAP_FAST_OVERHEAD;
    static double THREAD_SWAP_FROM_MEM_OVERHEAD;

    sstmac::sw::key::ptr
    get_blocker()
    {
      return blocker_;
    }

    void
    set_blocker(sstmac::sw::key::ptr b)
    {
      blocker_ = b;
    }

    void
    set_core(int c)
    {
      assigned_core_ = c;
    }

    int
    get_core() const
    {
      return assigned_core_;
    }

    void
    starting(){
      start_ = true;
    }

    void
    started() {
      start_ = false;
    }

    bool
    is_starting() const {
      return start_;
    }

    virtual
    ~hpx_runnable()
    {
    }
  };

}

#endif
