/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_COMM_MPIGROUP_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_COMM_MPIGROUP_H_INCLUDED

#include <sstmac/software/process/tasklist.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/software/api/mpi/mpi_types/mpiid.h>
namespace sstmac
{
  namespace sw
  {

    class mpigroup : public ptr_type
    {
    protected:
      mpigroup(tasklist::const_ptr p);

      mpigroup(size_t size) :
        size_(size), is_comm_world_(true)
      {
      } // the comm_world constructor to save space

      tasklist::const_ptr peers_;
      long groupid_;
      static long groupcounter_;

      size_t size_; //used for comm_world
      bool is_comm_world_;  //we don't save all the peers to save space

    public:
      typedef boost::intrusive_ptr<mpigroup> ptr;

      static ptr
      construct(tasklist::const_ptr p)
      {
        return ptr(new mpigroup(p));
      }

      static ptr
      construct(size_t size)
      {
        return ptr(new mpigroup(size));
      }

      virtual
      ~mpigroup()
      {
      }

      virtual std::string
      to_string() const
      {
        return "mpigroup";
      }

      taskid
      at(int rank);

      size_t
      size() const;

      tasklist::const_ptr
      list();

      mpiid
      rank_of_task(taskid t);

    };

  }
}

#endif
