/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_implementation/basicmpi.h>
#include <sstmac/software/api/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/sstmac_env.h>
namespace sstmac
{
  namespace sw
  {

   SSTRegister("basic", mpi_implementation, basic_mpi_implementation,
              "A basic MPI [sockets] that has only two protocols: eager and rendezvous");

    basic_mpi_implementation::basic_mpi_implementation(int handsize) :
      mpi_implementation(24), //envelope size
          handsize_(handsize)
    {

    }

    mpiprotocol::ptr
    basic_mpi_implementation::get_protocol(long bytes, bool intranode,
        bool ssend, bool onesided) const
    {
      if (onesided)
      {
        return mpiprotocol::eager0_socket_protocol;
      }

      if (intranode)
      {
        if (ssend || bytes > handsize_)
        {
          return mpiprotocol::rendezvous_mmap_protocol;
        }
        else
        {
          return mpiprotocol::eager0_mmap_protocol;
        }
      }
      else
      {
        if (bytes <= handsize_)
        {
          if (ssend)
          {
            return mpiprotocol::eager_ssend_protocol;
          }
          else
          {
            return mpiprotocol::eager0_socket_protocol;
          }
        }
        else
        {
          return mpiprotocol::rendezvous_socket_protocol;
        }
      }
    }

    mpi_implementation::ptr
    basic_mpi_implementation::construct(const sim_parameters::ptr& params)
    {
        /**
          sstkeyword {
            docstring=The cutoff in bytes at which we switch from an eager to rendezvous protocol;
          }
        */
        int handsize = sstmac_env::params->get_optional_int_param("mpi_handshake_size", 65536);
        return new basic_mpi_implementation(handsize);
    }

  }
}
