/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_IMPLEMENTATION_H
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_IMPLEMENTATION_H

#include <sstmac/software/api/mpi/mpimessage.h>
#include <sstmac/common/factories/factory.h>

namespace sstmac
{
  namespace sw
  {

    class mpi_implementation :
            public factory_type
    {
    protected:
      int64_t envelope_;

      mpi_implementation(int64_t envelope);

    public:
      typedef boost::intrusive_ptr<mpi_implementation> ptr;

      virtual mpiprotocolPtr
      get_protocol(long bytes, bool intranode, bool ssend, bool onesided) const = 0;

      int64_t
      get_envelope_size() const;

      virtual std::string
      to_string() const
      {
        return "mpi implentation";
      }

    };

    DeclareFactory(mpi_implementation);

  }
}

#endif // MPI_IMPLEMENTATION_H
