/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBS_MPI_CRAYMPT_H
#define SSTMAC_SOFTWARE_LIBS_MPI_CRAYMPT_H
#include <sstmac/software/api/mpi/mpi_implementation/mpi_implementation.h>

namespace sstmac
{
    namespace sw
    {
        class rdmampi
            : public mpi_implementation
        {
            protected:
                int64_t max_vshort_msg_size_;

                int64_t max_eager_msg_size_;

                int64_t smp_single_copy_size_;

                rdmampi(int smp_single_copy_size, int max_eager_msg_size, int max_vshort_msg_size);

            public:
                typedef boost::intrusive_ptr<rdmampi> ptr;

                mpiprotocolPtr
                get_protocol(long bytes, bool intranode, bool ssend, bool onesided) const;

                static ptr
                construct(const sim_parameters::ptr& params);

                virtual std::string
                to_string() const
                {
                    return "craympt mpi implentation";
                }

                int64_t max_vshort_msg_size() const;

                int64_t max_eager_msg_size() const;

                int64_t smp_single_copy_size() const;
        };
    }
}

#endif // CRAYMPT_H
