/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_implementation/simplempi.h>
#include <sstmac/software/api/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/common/sim_parameters.h>

namespace sstmac
{
  namespace sw
  {

  SSTRegister("simple", mpi_implementation, simple_mpi_implementation,
             "The simplest possible MPI that does everything as a buffered, eager send");

    simple_mpi_implementation::simple_mpi_implementation() :
      mpi_implementation(24)
    {

    }

    mpiprotocol::ptr
    simple_mpi_implementation::get_protocol(
        long bytes,
        bool intranode,
        bool ssend, bool onesided) const
    {
      if(onesided){
        return mpiprotocol::eager0_socket_protocol;
      }

      if (intranode)
        {
          return mpiprotocol::rendezvous_mmap_protocol;
        }
      else
        {
          if (ssend){
            return mpiprotocol::eager_ssend_protocol;
          }
          else {
            return mpiprotocol::eager0_socket_protocol;
          }

        }
    }

    mpi_implementation::ptr
    simple_mpi_implementation::construct(const sim_parameters::ptr& params)
    {
      return new simple_mpi_implementation();
    }

  }
}
