#include <sstmac/software/api/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/api/mpi/mpiserver.h>
#include <sstmac/software/api/mpi/rma/mpiqueue-getrequest.h>
namespace sstmac
{
  namespace sw
  {

    void
    rendezvous_rma::send_header(const mpiqueue::ptr &queue,
        const mpimessage::ptr &msg)
    {
      SSTMACBacktrace("MPI Rendezvous Protocol: RMA Send Header");
      SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
            << "]: start rendezvous rdma send of message " << msg->to_string()
            << "\n";
      //msg->set_needs_ack(false);  //we don't care about the nic ack for this

      queue->post_header(msg);
    }

    void
    rendezvous_rma::handle_header(const mpiqueue::ptr &queue,
        const mpimessage::ptr &msg)
    {
      SSTMACBacktrace("MPI Rendezvous Protocol: RMA Handle Header");
      SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
            << "]:: handle rendezvous rdma rma header on msg "
            << msg->to_string() << "\n";

      mpi_rma_message::ptr rmsg = boost::dynamic_pointer_cast<mpi_rma_message>(
          msg);

      if (queue->windows_.find(rmsg->winid()) == queue->windows_.end())
      {
        sst_throw(ssterror, "rendezvous_rma::handle_header - window %d not stored in mpiqueue", rmsg->winid());
      }

      mpi_window::ptr win = queue->windows_[rmsg->winid()];

      //  if (win->get_current_epoch() == rmsg->epoch())
      //  {

      if (rmsg->get_rmatype() == mpi_rma_message::get)
      {
        mpi_rma_message::op_info inf = rmsg->get_opinfo();
        payload::const_ptr load = win->get_data(inf);

        rmsg->mutate(sst_message::RDMA_GET_REQ_TO_RSP);
        rmsg->set_content(load);
        rmsg->set_needs_ack(false);
        queue->post_rdma(rmsg);
      }
      else if (rmsg->get_rmatype() == mpi_rma_message::put)
      {
        mpi_rma_message::op_info inf = rmsg->get_opinfo();
        win->put_data(inf, rmsg->content());

        rmsg->mutate(sst_message::RDMA_PUT_REQ_TO_RSP);
        rmsg->set_content(payload::const_ptr());
        rmsg->set_needs_ack(false);
        queue->post_rdma(rmsg);
      }
      else if (rmsg->get_rmatype() == mpi_rma_message::acc)
      {
        mpi_rma_message::op_info inf = rmsg->get_opinfo();
        win->acc_data(inf, rmsg->content(), msg->type());

        rmsg->mutate(sst_message::RDMA_PUT_REQ_TO_RSP);
        rmsg->set_content(payload::const_ptr());
        rmsg->set_needs_ack(false);
        queue->post_rdma(rmsg);
      }

      // }
      //  else
      //  {
      //    queue->headers_waiting_epoch_[rmsg->epoch()].push(msg);
      //  }

    }

    void
    rendezvous_rma::handle_payload(const mpiqueue::ptr &queue,
        const mpimessage::ptr &msg)
    {
      SSTMACBacktrace("MPI Rendezvous Protocol: RMA Handle Payload");
      SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
            << "]: handle rendezvous rdma rma payload on msg "
            << msg->to_string() << "\n";

      mpiqueue::get_requests_t::iterator it =
          queue->gets_waiting_for_data_.find(msg->id());
      if (it == queue->gets_waiting_for_data_.end())
      {
        sst_throw(illformederror,
            "mpiqueue[%s]: rendezvous_rma::handle_payload: "
            "data message without a matching ack on %s",
            queue->id_string().c_str(),
            msg->to_string().c_str());
      }
      mpiqueue::getrequest::ptr recver = it->second;
      queue->gets_waiting_for_data_.erase(it);
      recver->handle(msg);
    }

    bool
    rendezvous_rma::send_needs_completion_ack() const
    {
      return false;
    }

    bool
    rendezvous_rma::send_needs_rendezvous_ack() const
    {
      return false;
    }

    void
    rendezvous_rma::recv_header(const mpiqueue::ptr &queue,
        const mpimessage::ptr &msg, const mpiqueue::recvrequest::ptr& req)
    {
      sst_throw(ssterror, "rendezvous_rma::recv_header - calling this function doesn't make sense");
    }

    void
    rendezvous_rma::recv_payload(const mpiqueue::ptr &queue,
        const mpimessage::ptr &msg, const mpiqueue::recvrequest::ptr& req)
    {
      sst_throw(ssterror, "rendezvous_rma::recv_payload - calling this function doesn't make sense");

    }

  }
}
