/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_queue/mpiqueue-proberequest.h>

#include <sstmac/software/api/mpi/mpistatus.h>
#include <sstmac/common/errors.h>

namespace sstmac { namespace sw {

  // Pointer types.
  
  

  //
  // Hi there.
  //
  mpiqueue::proberequest::proberequest(const mpirequest_t key, const mpicomm::ptr &comm,
                                       const mpiid &source, const mpitag &tag, eventhandler::ptr completion) :
    myid_(comm->id()), source_(source), tag_(tag), completion_(completion), key_(key)
  {}

  //
  // Hello.
  //
  mpiqueue::proberequest::ptr
  mpiqueue::proberequest::construct(const mpirequest_t key, const mpicomm::ptr &comm, const mpiid &source,
                                    const mpitag &tag, eventhandler::ptr completion)
  {
    return proberequest::ptr
      (new proberequest(key, comm, source, tag, completion));
  }

  //
  // Test whether we match a given message.
  //
  bool mpiqueue::proberequest::
  matches(const mpimessage::ptr& message) const {
    //std::cerr << "myid_ == " << myid_ << "\n";
    //std::cerr << "message->commid() == " << message->commid() << "\n";
    mpicommid inid = message->commid();
    bool sameid = (myid_ == inid);
    return (sameid && 
            source_.matches(message->source()) &&
            tag_.matches(message->tag()));
  }

  //
  // Consider this request complete.
  //
  void mpiqueue::proberequest::complete(
                                        const mpimessage::ptr& message)
  {
    if(completion_ != 0) {
      key_->complete(message);
      completion_->handle(message);
    }
  }

} } // end of namespace sstmac.
