/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_queue/mpiqueue-recvrequest.h>
#include <sstmac/software/api/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/api/mpi/mpimessage.h>
#include <sstmac/common/debug.h>
#include <sstmac/common/errors.h>

namespace sstmac
{
  namespace sw
  {

    // Pointer types.


    //
    // Hello.
    //
    mpiqueue::recvrequest::recvrequest(const mpirequest_t key,
        const mpiqueue::ptr& queue, int count, const mpitype &type,
        const mpiid &source, const mpitag &tag, const mpicomm::ptr &comm,
        mpimessage::category cat, eventhandler::ptr &completion) :
      queue_(queue), source_(source), tag_(tag), comm_(comm), cat_(cat),
          seqnum_(seqnum_unassigned), count_(count), type_(type), key_(key),
          completion_(completion)
    {
    }

    //
    // Hello.
    //
    mpiqueue::recvrequest::ptr
    mpiqueue::recvrequest::construct(const mpirequest_t key,
        const mpiqueue::ptr& queue, int count, const mpitype &type,
        const mpiid &source, const mpitag &tag, const mpicomm::ptr &comm,
        mpimessage::category cat, eventhandler::ptr &completion)
    {
      recvrequest::ptr retval(
          new recvrequest(key, queue, count, type, source, tag, comm, cat,
              completion));

      return retval;
    }

    //
    // Goodbye.
    //
    mpiqueue::recvrequest::~recvrequest()
    {
    }

    bool
    mpiqueue::recvrequest::open_source() const
    {
      return source_ == mpiid::any_source;
    }

    //
    // We be done.
    //
    void
    mpiqueue::recvrequest::handle(const mpimessage::ptr& mess)
    {

      if (mess->count() < 0)
      {
        mess->get_protocol()->recv_header(queue_, mess, recvrequest::ptr(this));
      }
      else
      {
        mess->get_protocol()->recv_payload(queue_, mess, recvrequest::ptr(this));

        queue_->finalize_recv(mess);
        key_->complete(mess);
        if (key_->get_status())
        {
          key_->get_status()->set_recv_type(type_);
        }
        completion_->handle(mess);
      }

    }

    bool
    mpiqueue::recvrequest::matches(const mpimessage::ptr &msg)
    {
      return ((cat_ == msg->cat()) && (comm_->id() == msg->commid())
          && (seqnum_ == seqnum_unassigned ? true : seqnum_ == msg->seqnum())
          && source_.matches(msg->source()) && tag_.matches(msg->tag()));
    }

  }
} // end of namespace sstmac
