/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_QUEUE_MPIQUEUE_SENDREQUEST_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_QUEUE_MPIQUEUE_SENDREQUEST_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_queue/mpiqueue.h>
#include <sstmac/common/messages/messageid.h>
#include <sstmac/common/nocopy.h>
#include <sstmac/common/debug.h>
#include <sstmac/software/api/mpi/mpiserver.h>

namespace sstmac { namespace sw {

  // forward declaration of important types.
  //class payload;

  /**
   * A nested type to handle individual send operations.
   */
  class mpiqueue::sendrequest :
    virtual public nocopy,
    public ptr_type
  {
    friend class mpiqueue;

    /// The queue.
    mpiqueue::ptr queue_;

    /// The object that gets notified when we're done.
    eventhandler::ptr completion_;

    mpirequest_t key_;

    mpimessage::ptr mess_;


  protected:
    sendrequest(const mpimessage::ptr& mess,
              const mpirequest_t key, const mpiqueue::ptr& queue,
              const eventhandler::ptr& completion);




  public:

    virtual std::string to_string() const{
            return "sendrequest";
          }

    typedef boost::intrusive_ptr<sendrequest> ptr;

    /// Hi there.
    static sendrequest::ptr
    construct(const mpimessage::ptr& mess,
              const mpirequest_t key, const mpiqueue::ptr& queue,
              const eventhandler::ptr& completion);

    /// Goodbye.
    virtual ~sendrequest() throw();

    bool matches(const mpimessage::ptr& send_ack) const;

    /// Eventhandler completion.
    void complete(const mpimessage::ptr &msg);

    void wait_for_buffer();
  };

} } // end of namespace sstmac

#endif
