/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_QUEUE_USERTHREAD_MPIQUEUE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_QUEUE_USERTHREAD_MPIQUEUE_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_queue/mpiqueue.h>

namespace sstmac { namespace sw
{

  class user_thread_mpiqueue :
          public mpiqueue
  {
  protected:
    /// Hi there.
    user_thread_mpiqueue(timestamp post_rdma_delay,
                         timestamp post_header_delay,
                         timestamp poll_delay);

    bool is_polling_;

    key::ptr blocking_key_;

    mpimessage::ptr incoming_msg_;

    std::list<mpimessage::ptr> pending_msgs_;

    timestamp post_rdma_delay_;

    timestamp post_header_delay_;

    timestamp poll_delay_;

  public:
    typedef boost::intrusive_ptr<user_thread_mpiqueue> ptr;

    virtual std::string
    to_string() const{
            return "user thread mpiqueue";
    }

    virtual ~user_thread_mpiqueue() throw();

    /// \param server:   The MPI server through which we route messages.
    /// \param node:     The node to which we post copy and compute requests.
    /// \param mintrans: Smallest MPI message.
    static ptr
    construct(const sim_parameters::ptr& params);

    virtual timestamp progress_loop(const mpirequest_t& req);

    virtual void start_progress_loop(const std::vector<mpirequest_t>& req);

    virtual void start_progress_loop(const std::vector<mpirequest_t>& req, timestamp timeout);

    virtual void finish_progress_loop(const std::vector<mpirequest_t>& req);

    virtual eventhandler::ptr progress_done_handler(const OS_API::ptr& os, const mpirequest_t& req);

    virtual void incoming_message(const mpimessage::ptr& message);

    virtual void buffered_send(const mpimessage::ptr& msg);

    virtual void buffered_recv(const mpimessage::ptr& msg, const recvrequestPtr& req);

    virtual void buffer_unexpected(const mpimessage::ptr& msg);

    virtual void post_rdma(const mpimessage::ptr& msg);

    virtual void post_header(const mpimessage::ptr& msg);

  protected:
    bool at_least_one_complete(const std::vector<mpirequest_t>& req);

    virtual void do_send(const mpimessage::ptr& mess);

    virtual void do_recv(const mpiqueue::recvrequestPtr& req);

    void clear_pending();

  };


} } // end of namespace sstmac


#endif
