/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_ENGINES_MPIBCASTENGINE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_ENGINES_MPIBCASTENGINE_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/mpicollective.h>
#include <sstmac/software/api/mpi/mpi_comm/mpicomm.h>
#include <sstmac/software/api/mpi/mpi_types/mpitype.h>
#include <sstmac/software/api/mpi/mpi_types/mpitag.h>
#include <sstmac/software/api/mpi/mpirequest.h>
#include <sstmac/software/api/mpi/mpi_queue/mpiqueue.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpi_iterator/topologyiterator.h>
#include <sstmac/software/api/mpi/mpistatus.h>

namespace sstmac { namespace sw
{

  // Important types.


 // class payload;

  /**
   * A basic mpikernel to provide an MPI_Bcast.
   */
  class mpibcastengine : public mpicollective
  {
    /// The iterator to whom we entrust decisions on where to send and receive.
    topologyiterator::ptr iter_;
    /// The size of the data array.
    int count_;
    /// The type we are sending and receiving.
    mpitype type_;
    /// The payload we are sending.
    payload::const_ptr content_;
    /// My rank.
    mpiid rank_;

    /// The number of pending sends.
    int pending_sends_;
    /// The number of pending receives.
    int pending_recvs_;

    bool completing_;

    bool is_root_;

    /// Fire off the next set of send and receive operations.
    void
    sendrecv();

  protected:
    /// Hi.
    mpibcastengine(const topologyiterator::ptr& iter,
        const mpirequest::ptr& thekey, const
            mpiqueue::ptr& queue, int count, const mpitype &type,
        const mpiid &root, const mpitag &tag, const mpicomm::ptr &comm,
        const payload::const_ptr& content,
            eventhandler::ptr completion);

    /// Callback method to indicate that a send operation has completed.
    virtual void
    send_complete(int count, const mpitype &type, const mpitag &tag,
        const mpiid &dest);

    /// Callback method to indicate that a receive operation has completed.
    virtual void
    recv_complete(int count, const mpistatus_t& stat);

    /// Callback method to indicate that a copy operation has completed.
    virtual void
    copy_complete(int count, const mpitype &type, const mpitag &tag,
        const mpiid &source);

  public:
    typedef boost::intrusive_ptr<mpibcastengine> ptr;

    std::string to_string() const {
        return sstprintf("mpibcast(tag=%d)", tag_.id);
    }

    /// Hello.
    static mpibcastengine::ptr
    construct(const topologyiterator::ptr& iter,
        const mpirequest::ptr& thekey, const
            mpiqueue::ptr& queue, int count, const mpitype &type,
        const mpiid &root, const mpitag &tag, const mpicomm::ptr &comm,
        const payload::const_ptr& content,
            eventhandler::ptr completion);

    /// Goodbye.
    virtual
    ~mpibcastengine() throw ();

    /// Start this kernel.
    virtual void
    start();
  };

} } // end of namespace sstmac

#endif
