/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLECTIVE_IMMEDIATECOPY_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLECTIVE_IMMEDIATECOPY_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/mpicollective.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>
#include <deque>

namespace sstmac { namespace sw {

  /**
   * Utility type to handle concurrent copying out of
   * a shared data array.  Simulates copying data for remote sources.
   */
  class mpicollective::immediatecopy : public eventhandler {
    mpicollective::ptr parent_;
    int count_;
    mpitype type_;
    mpitag tag_;
    mpiid dest_;

  protected:
    // Hi.
    immediatecopy(const mpicollective::ptr& parent, int count,
                 const mpitype &type, const mpitag &tag, const mpiid &dest) :
      parent_(parent), count_(count), type_(type), tag_(tag), dest_(dest)
    {}

  public:
    typedef boost::intrusive_ptr<immediatecopy> ptr;

    // Hello.
    static immediatecopy::ptr
    construct(const mpicollective::ptr& parent, int count,
              const mpitype &type, const mpitag &tag, const mpiid &dest)
    {
      return immediatecopy::ptr(new immediatecopy(parent, count, type, tag, dest));
    }

    std::string to_string() const { return "mpicollective::immediatecopy"; }

    // Bye.
    virtual ~immediatecopy() throw() {}

    // Event completion.
    virtual void handle(const sst_message::ptr &msg) {
  //    if(debugger::active())
    //    debug() << "mpbcastengine::immediatecopy[" << this << "]::handle( "
     //           << now << ")\n";
      parent_->copy_complete(count_, type_, tag_, dest_);
    }
  };

} } // end of namespace sstmac

#endif
