/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLECTIVE_RECVER_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLECTIVE_RECVER_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/mpicollective.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/software/api/mpi/mpi_types/mpitype.h>
#include <sstmac/software/api/mpi/mpi_types/mpitag.h>
#include <sstmac/software/api/mpi/mpi_types/mpiid.h>
#include <sstmac/software/api/mpi/mpimessage.h>
#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>

namespace sstmac { namespace sw
{

  /**
   * A private mpirecv type to handle individual recv operations for
   * collective mpi operations.
   */
  class mpicollective::recver : public eventhandler
  {
    /// The parent collective operation.
    mpicollective::ptr parent_;
    /// Number of elements sent.
    int count_;
    /// MPI type expected.
    mpitype type_;
    /// MPI tag associated with the operation.
    mpitag tag_;
    /// The peer from whom this message was received.
    mpiid source_;

  protected:
    /// Hi.
    recver(const mpicollective::ptr& parent, int count,
        const mpitype &type, const mpitag &tag, const mpiid &source) :
      parent_(parent), count_(count), type_(type), tag_(tag), source_(source)
    {
    }

  public:
    virtual std::string to_string() const {
                    return "recver";
            }

    typedef boost::intrusive_ptr<recver> ptr;

    /// Hello.
    static recver::ptr
    construct(const mpicollective::ptr& parent, int count,
        const mpitype &type, const mpitag &tag, const mpiid &source)
    {
      return recver::ptr(new recver(parent, count, type, tag,
          source));
    }

    /// Goodbye
    virtual
    ~recver() throw ()
    {
    }

    /// Event handler callback.
    virtual void
    handle(const sst_message::ptr &msg)
    {
    //  if (dbg_.is_active())
     //   dbg_ << "mpicollective::recver[" << "]::handle(" << ")\n";

      mpimessage::ptr mmsg = boost::dynamic_pointer_cast<
          mpimessage>(msg);
      mpistatus::ptr stat = mmsg->build_status();
      stat->set_recv_type(type_);
      parent_->recv_complete(count_, stat);
    }
  };

} } // end of namespace sstmac

#endif
