/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLECTIVE_SENDER_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLECTIVE_SENDER_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/mpicollective.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/software/api/mpi/mpi_types/mpitype.h>
#include <sstmac/software/api/mpi/mpi_types/mpitag.h>
#include <sstmac/software/api/mpi/mpi_types/mpiid.h>
#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>

namespace sstmac { namespace sw
{

  /**
   * A private mpisend type to handle individual send operations for
   * collective mpi operations.
   */
  class mpicollective::sender : public eventhandler
  {
    /// The parent collective operation.
    mpicollective::ptr parent_;
    /// The MPI status object associated with the collective operation.
    mpistatus::ptr status_;
    /// Send length
    int count_;
    /// Send type.
    mpitype type_;
    /// Send tag.
    mpitag tag_;
    /// Destination
    mpiid dest_;

  protected:
    /// Hi.
    sender(const mpicollective::ptr& parent,
        const mpistatus::ptr& status, int count,
        const mpitype &type, const mpitag &tag, const mpiid &dest) :
      parent_(parent), status_(status), count_(count), type_(type), tag_(tag),
          dest_(dest)
    {
    }

  public:
    virtual std::string to_string() const {
                      return "sender";
              }

    typedef boost::intrusive_ptr<sender> ptr;

    /// Hello.
    static sender::ptr
    construct(const mpicollective::ptr& parent,
        const mpistatus::ptr& status, int count,
        const mpitype &type, const mpitag &tag, const mpiid &dest)
    {
      return sender::ptr(new sender(parent, status, count, type,
          tag, dest));
    }

    /// Goodbye
    virtual
    ~sender() throw ()
    {
    }

    /// Access the mpistatus.
    mpistatus::ptr
    get_mpistatus()
    {
      return status_;
    }

    /// Event handler callback.
    virtual void
    handle(const sst_message::ptr &msg)
    {
    //  if (dbg_.is_active())
    //    dbg_ << "mpicollective::sender[" << "]::handle(" << ")\n";
      parent_->send_complete(count_, type_, tag_, dest_);
    }
  };

} } // end of namespace sstmac

#endif
