/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_strategies/mpicoreallreduce.h>

#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/engines/mpirabenseifner.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/engines/mpilogtwovirt.h>
#include <sstmac/software/libraries/unblockhandler.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  

  //
  // Hi.
  //
  mpicoreallreduce::mpicoreallreduce()
  {
  }

  //
  // Hello.
  //
  mpicoreallreduce::ptr
  mpicoreallreduce::construct()
  {
    return mpicoreallreduce::ptr (new mpicoreallreduce);
  }

  //
  // Perform a fused allreduce-allreduce operation
  //
  void
  mpicoreallreduce::execute(const mpirequest_t& req, const
      mpiqueue::ptr& queue, int count, const mpitype &type, const mpiop &op,
      const mpitag &tag, const mpicomm::ptr &comm,
      const payload::const_ptr& content, OS_API::ptr &os) const
  {
    eventhandler::ptr completion = queue->progress_done_handler(os, req);
    mpirabenseifner::ptr engine = mpirabenseifner::construct(
        true, req, queue, count, type, op, mpiid(-1), tag, comm, content,
        completion);

    engine->start();
  }

  // Pointer types.


  //
  // Hi.
  //
  mpiwilkeallreduce::mpiwilkeallreduce()
  {
  }

  //
  // Hello.
  //
  mpiwilkeallreduce::ptr
  mpiwilkeallreduce::construct()
  {
    return new mpiwilkeallreduce;
  }

  //
  // Perform a fused allreduce-allreduce operation
  //
  void
  mpiwilkeallreduce::execute(const mpirequest_t& req, const
      mpiqueue::ptr& queue, int count, const mpitype &type, const mpiop &op,
      const mpitag &tag, const mpicomm::ptr &comm,
      const payload::const_ptr& content, OS_API::ptr &os) const
  {
      if (content){
          //actual payload
          mpicoreallreduce::execute(req, queue, count, type,
                                    op, tag, comm, content, os);
      }
      else {
          eventhandler::ptr completion = queue->progress_done_handler(os, req);
          mpilogtwovirtallreduce::ptr engine = mpilogtwovirtallreduce ::construct(
              req, queue,
              count, type, op,
              mpiid(-1), tag, comm,
              content, completion);

          engine->start();
      }
  }

} } // end of namespace sstmac
