/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_strategies/mpicorealltoallv.h>

#include <sstmac/software/libraries/unblockhandler.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/engines/mpidirectalltoall.h>

namespace sstmac { namespace sw {

  // Pointer types.


  //
  // Hi.
  //
  mpicorealltoallv::mpicorealltoallv() {
  }

  //
  // Hello.
  //
  mpicorealltoallv::ptr mpicorealltoallv::construct() {
    return mpicorealltoallv::ptr(new mpicorealltoallv);
  }

  //
  // Must be virtual.
  //
  mpicorealltoallv::~mpicorealltoallv() throw() {
  }

  //
  // Build a kernel.
  //
  void
  mpicorealltoallv::execute(const mpirequest_t& req,
      const mpiqueue::ptr& queue,
      const std::vector<int> &sendcnts,
      const mpitype &sendtype,
      const std::vector<int> &recvcnts,
      const mpitype &recvtype,
      const mpitag &tag, const mpicomm::ptr &comm,
      const std::vector<payload::const_ptr >& content,
      OS_API::ptr &os
  ) const
  {
    eventhandler::ptr completion = queue->progress_done_handler(os, req);
    mpidirectalltoall::ptr engine = mpidirectalltoall::construct(
        req, queue, sendcnts, sendtype, recvcnts, recvtype, tag, comm,
        content, completion);

    engine->start();
  }

} } // end of namespace sstmac
