/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_strategies/mpicorebcast.h>

#include <sstmac/software/api/mpi/mpi_strategies/mpi_iterator/btscatteriterator.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/engines/mpibcastengine.h>
#include <sstmac/software/libraries/unblockhandler.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  


  //
  // Hello.
  //
  mpicorebcast::mpicorebcast()
  {

  }

  //
  // Hi.
  //
  mpicorebcast::ptr
  mpicorebcast::construct()
  {
    return mpicorebcast::ptr (new mpicorebcast);
  }

  //
  // Goodbye.
  //
  mpicorebcast::~mpicorebcast() throw ()
  {
  }


  //
  // Build a kernel to perform a broadcast.
  //
  void
  mpicorebcast::execute(const mpirequest_t& req,
      const mpiqueue::ptr& queue, int count, const mpitype &type,
      const mpiid &root, const mpitag &tag, const mpicomm::ptr &comm,
      const payload::const_ptr& content,
      OS_API::ptr &os
  ) const
  {
    topologyiterator::ptr iter = btscatteriterator::construct(
        comm->rank().id, comm->size().id, root.id);
    eventhandler::ptr completion = queue->progress_done_handler(os, req);
    mpibcastengine::ptr engine = mpibcastengine::construct(iter, req, queue, count, type, root,
        tag, comm, content, completion);
    engine->start();


  }

} } // end of namespace sstmac
