/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_strategies/mpicorersend.h>

#include <sstmac/software/api/mpi/mpi_queue/mpiqueue.h>
#include <sstmac/software/libraries/unblockhandler.h>

namespace sstmac { namespace sw
{

  
  

  //
  // Hello.
  //
  mpicorersend::mpicorersend()
  {
  }

  //
  // Build.
  //
  mpicorersend::ptr
  mpicorersend::construct()
  {
    return mpicorersend::ptr (new mpicorersend);
  }

  //
  // Goodbye.
  //
  mpicorersend::~mpicorersend() throw ()
  {
  }

  //
  // Basic nonblocking send.
  //
  void
  mpicorersend::execute(const mpirequest_t& req,
                        const mpiqueue::ptr& queue,
                        int count, const mpitype &type,
                        const mpiid &dest,
                        const mpitag &tag, const mpicomm::ptr &comm,
                        const payload::const_ptr& content, OS_API::ptr &os) const
  {
    bool rsend = true, ssend = false;
    eventhandler::ptr completion = queue->progress_done_handler(os, req);
    mpiqueue::sendinfo sinfo;
    sinfo.rsend = true;
    queue->send(req, count, type, dest, tag, comm, sinfo,
        mpimessage::user, completion, content);
  }

} } // end of namespace sstmac.
