/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_types/mpiid.h>
#include <sstmac/common/errors.h>
#include <iostream>
#include <sstream>
#include <sstmac/software/api/mpi/mpitypes.h>

namespace sstmac { namespace sw {

  //
  // The static "any source" id within a communicator.
  //
  const mpiid mpiid::any_source(MPI_ANY_SOURCE);

  //
  // The "root" identifier is allowed in certain contexts.
  //
  const mpiid mpiid::root(-10);

  const mpiid mpiid::proc_null(MPI_PROC_NULL);

  //
  // Refer to a peer index within a communicator.
  //
  mpiid::mpiid(int idit) :
    id(idit)
  {}

  //
  // Test whether two node ids match.
  //
  bool mpiid::matches(const mpiid &other) const {
    return ((this->id == any_source.id) || (this->id == other.id));
  }

  //
  // Increment (prefix).
  //
  mpiid& operator++(mpiid &value) {
    ++value.id;
    return value;
  }

  //
  // Increment (postfix).
  //
  mpiid operator++(mpiid &value, int) {
    mpiid rv(value);
    ++value.id;
    return rv;
  }
  
  //
  // Test two node ids for strict (in)equality (stronger than match).
  //
  bool operator==(const mpiid &a, const mpiid &b) {
    return a.id == b.id;
  }
  bool operator!=(const mpiid &a, const mpiid &b) {
    return !(a == b);
  }

  //
  // Rank-compare operations.
  //
  bool operator<(const mpiid &a, const mpiid &b) { return (a.id < b.id); }
  bool operator>(const mpiid &a, const mpiid &b) { return (a.id > b.id); }
  bool operator<=(const mpiid &a, const mpiid &b){ return (a.id <= b.id); }
  bool operator>=(const mpiid &a, const mpiid &b){ return (a.id >= b.id); }

  std::string mpiid::to_string() const {
      std::stringstream ss(std::stringstream::in | std::stringstream::out);
      ss << "mpiid(id=" << id << ")";
      return ss.str();
  }

  //
  // Fairly self-explanatory.
  //
  std::ostream& operator<<(std::ostream &os, const mpiid &id) {
    os << id.to_string();
    return os;
  }

} } // end of namespace sstmac
