/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_TYPES_MPIID_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_TYPES_MPIID_H_INCLUDED

#include <iosfwd>
#include <sstmac/common/types.h>

namespace sstmac { namespace sw {

  /// Index within a communicator.
  struct mpiid {
    int id;
    explicit mpiid(int idit=-1);
    bool matches(const mpiid &other) const;

    std::string to_string() const;

    static const mpiid any_source;
    static const mpiid root;
    static const mpiid proc_null;
  };

  mpiid& operator++(mpiid &value);
  mpiid operator++(mpiid &value, int);
  bool operator==(const mpiid &a, const mpiid &b);
  bool operator!=(const mpiid &a, const mpiid &b);
  bool operator<(const mpiid &a, const mpiid &b);
  bool operator>(const mpiid &a, const mpiid &b);
  bool operator<=(const mpiid &a, const mpiid &b);
  bool operator>=(const mpiid &a, const mpiid &b);

  std::ostream& operator<<(std::ostream &os, const mpiid &id);

} } // end of namespace sstmac

#endif
