/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_types/mpiop.h>
#include <sstmac/common/types.h>

namespace sstmac
{
  namespace sw
  {

    static int nextid = 0;

    ///// Reduction operations.
    mpiop::mpiop(const std::string &labelit) : //, const timestamp &costit) :
      label(labelit),
      //cost_per_entry(costit),
          id(nextid++), commute_(true)
    {
    }

    mpiop::mpiop(void
    (*func)(void *, void *, int *, long *), int commute) :
      label("user"), id(nextid++), userfunc_(func), commute_(commute)
    {

    }

    mpiop::mpiop(const mpiop &other)
    {

      deep_copy(other);
    }

    mpiop&
    mpiop::operator=(const mpiop& other)
    {
      deep_copy(other);
      return *this;
    }

    void
    mpiop::deep_copy(const mpiop& other)
    {
      label = other.label;

      id = other.id;

      commute_ = other.commute_;
      userfunc_ = other.userfunc_;

    }

    //
    // Common reduction operations.
    // For simplicity, I am assuming that reductions take 1 ns/op
    const mpiop mpiop::max("max");//, 1E-9);

    const mpiop mpiop::min("min");//, 1E-9);

    const mpiop mpiop::sum("sum");//, 1E-9);

    const mpiop mpiop::prod("prod");//, 1E-9);

    const mpiop mpiop::land("land");//, 1E-9);

    const mpiop mpiop::band("band");//, 1E-9);

    const mpiop mpiop::lor("lor");//, 1E-9);

    const mpiop mpiop::bor("bor");//, 1E-9);

    const mpiop mpiop::lxor("lxor");//, 1E-9);

    const mpiop mpiop::bxor("bxor");//, 1E-9);

    const mpiop mpiop::minloc("minloc");//, 1E-9);

    const mpiop mpiop::maxloc("maxloc");//, 1E-9);

    const mpiop mpiop::replace("replace");//,1E-9);

    const mpiop mpiop::op_null("op_null");//,1E-9);

  }
} // end of namespace sstmac
