/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpipayload.h>

DeclareSerializable(sstmac::sw::mpipayload)
  ;

namespace sstmac
{
  namespace sw
  {

    /// Construction time.
    mpipayload::mpipayload(void* data, mpitype t, int count, bool usewrapper) :
      type_(t), cnt_(count), wrapped_(usewrapper), given_(false), real_(true)
    {
      bufsize_ = count * t.get_packed_size();

     // std::cout << "Creating mpipayload with count " << count << " type size " << t.get_packed_size() << " and bufsize  " << bufsize_ << "\n";
      if (usewrapper)
      {
        buf_ = data;
      }
      else if (count > 0)
      {
        buf_ = malloc(sizeof(char) * bufsize_);
        int pos = 0;
        t.pack(data, count, buf_, bufsize_, &pos);
      }
      else
      {
        buf_ = 0;
      }
    }

    //internal construction
    mpipayload::mpipayload(void* data, mpitype t, int count) :
      type_(t), cnt_(count), wrapped_(false), given_(true), real_(true)
    {
    //  std::cout << "Creating mpipayload with count " << count << " type size " << t.get_packed_size() << " and bufsize  " << bufsize_ << "\n";

      bufsize_ = count * t.get_packed_size();

      buf_ = data;

    }

    mpipayload::mpipayload() :
      real_(false)
    {
      buf_ = 0;
      bufsize_ = 0;
      cnt_ = 0;
      wrapped_ = false;
      given_ = true;
    }

    void
    mpipayload::serialize_order(sst_serializer* ser)
    {
      if (ser->get_mode() != sst_serializer::UNPACK && !real_)
      {
        sst_throw(ssterror, "mpipayload::serialize_order - payload not real, what's the big deal");
      }
      ser->do_primitive(bufsize_);
      type_.serialize_order(ser);
      ser->do_primitive(cnt_);
     // ser->do_primitive(wrapped_);
     // ser->do_primitive(given_);

      if (bufsize_ > 0)
      {
       // std::cout << "mpipayload::serialize_order - bufsize is " << bufsize_
       //     << "\n";
        if (ser->get_mode() == sst_serializer::UNPACK)
        {
          real_ = true;
          buf_ = malloc(sizeof(char) * bufsize_);
        //  std::cout << "we're unpacking, making a new buffer \n";
        }
        if (buf_)
        {
        //  std::cout << "copying data..\n";
          ser->do_array(buf_, bufsize_);
        }
      }
    }


    std::string
    mpipayload::to_string() const
    {
      std::stringstream ss;
      if (cnt_ == 1 && type_ == mpitype::mpi_int)
      {
        int* d = (int*) buf_;
        ss << "mpipayload(" << type_.to_string() << ", " << cnt_ << ", val="
            << *d << ")";
      }else if(cnt_ < 10){
        ss << "mpipayload(" << type_.to_string() << ", " << cnt_ << ", bytes=";
        char* cast = (char*)buf_;
        for(int i = 0; i < bufsize_; i++){
          ss << (int)cast[i] << ",";
        }
      }
      else
      {
        ss << "mpipayload(" << type_.to_string() << ", " << cnt_ << ")";
      }
      return ss.str();
    }

  }
}
