/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpirequest.h>

namespace sstmac
{
  namespace sw
  {

    debug mpirequest::dbg_;

    mpirequest::mpirequest(const key::ptr &k) :
        key_(k), complete_(false), cancelled_(false)
    {
      if(!dbg_.is_real()){
          dbg_ = debug("<debug> mpirequest | mpi");
      }
    }

    mpirequest::~mpirequest()
    {
    }

    //
    // Build me a request.
    //
    mpirequest::ptr
    mpirequest::construct(const key::ptr &k)
    {
      mpirequest::ptr retval(new mpirequest(k));
      //retval->set_self(retval);
      return retval;
    }

    void
    mpirequest::complete(const mpimessage::ptr &msg)
    {
      if (!cancelled_)
        {
          if (msg)
            {
              SSTMAC_DEBUG << "mpirequest::complete - building stat \n";
              stat_ = msg->build_status();

              if (!stat_)
                {
                  SSTMAC_DEBUG
                        <<
                        "mpirequest::complete - something wrong, null stat returned \n";
                }
            }
          else
            {
              SSTMAC_DEBUG << "mpirequest::complete - null message, no stat \n";
            }
        }
      complete_ = true;
    }

  }
} //end of namespace sstmac
