/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_RMA_MPIQUEUE_GETREQUEST_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_RMA_MPIQUEUE_GETREQUEST_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_queue/mpiqueue.h>
#include <sstmac/common/nocopy.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * A nested type to handle individual mpi receive requests.
     */
    class mpiqueue::getrequest : virtual public nocopy, public ptr_type
    {

    public:
      typedef boost::intrusive_ptr<getrequest> ptr;

    private:

      friend class mpiqueue;

      /// The queue to whom we belong.
      boost::intrusive_ptr<mpiqueue> queue_;

      mpirequest_t key_;

      /// The object that will be notified upon completion.
      eventhandler::ptr completion_;

    protected:
      /// Hello.
      getrequest(const mpirequest_t key, const mpiqueue::ptr& queue,
          const eventhandler::ptr &completion);

    public:
      virtual std::string
      to_string() const
      {
        return "getrequest";
      }

      /// Hello.
      static getrequest::ptr
      construct(const mpirequest_t key, const mpiqueue::ptr& queue,
          const eventhandler::ptr &completion)
      {
        getrequest::ptr retval(new getrequest(key, queue, completion));

        return retval;
      }

      /// Goodbye.
      virtual
      ~getrequest()
      {
      }

      /// We be done.
      void
      handle(const mpimessage::ptr& mess);

      bool
      is_cancelled() const
      {
        return key_->is_cancelled();
      }
    };

  }
} // end of namespace sstmac

#endif
