/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#include "shmem/memory/symmetric.h"
#include <stdlib.h>

namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {
      debug symmetric::dbg_;

      symmetric::symmetric(size_t size, long addr, bool real) :
        size_(size), addr_(addr), real_(real)
      {
        if (real_)
          raw_ = malloc(size);

        op_counter_ = 0;

        if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> symmetric | shmem", logger::yellow);
          logger::register_logger(&dbg_);
        }
      }

      symmetric::symmetric(size_t size, size_t align, long addr, bool real) :
        size_(size), addr_(addr), real_(real), align_(align)
      {
        if (real_){
          //apparently mac osx doesn't have memalign?  screw it

          //raw_ = memalign(align, size);
        }

        op_counter_ = 0;

        if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> symmetric | shmem", logger::yellow);
          logger::register_logger(&dbg_);
        }
      }

      symmetric::symmetric(size_t size, long addr, void* raw) :
        size_(size), addr_(addr), real_(true), raw_(raw)
      {

        op_counter_ = 0;

        if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> symmetric | shmem", logger::yellow);
          logger::register_logger(&dbg_);
        }
      }

      void
      symmetric::realloc(size_t sz)
      {
        if (real_)
          raw_ = std::realloc(raw_, sz);
        size_ = sz;
      }
    }
  }
}
