/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SHMEM_MEMORY_WAIT_H_
#define SSTMAC_SHMEM_MEMORY_WAIT_H_

#include <sstmac/common/ptr_type.h>
#include "shmem.h"

namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {

      class wait_base : public ptr_type
      {
      public:
        typedef boost::intrusive_ptr<wait_base> ptr;

        virtual void*
        get_raw() = 0;

        virtual bool
        evalu_wait() = 0;
      };

      template<typename T>
        class wait : public wait_base
        {

        protected:
          wait(T* r, T val, int c) :
            raw_(r), val_(val), cond_(c)
          {

          }

          T* raw_;
          T val_;
          int cond_;

        public:
          typedef boost::intrusive_ptr<wait> ptr;

          static ptr
          construct(T* r, T val, int c)
          {
            return ptr(new wait(r, val, c));
          }

          virtual
          ~wait()
          {
          }

          virtual std::string
          to_string() const
          {
            return "shmem::wait";
          }

          virtual void*
          get_raw()
          {
            return raw_;
          }

          /**
           * Returns true if condition has been satisfied
           * @return
           */
          virtual bool
          evalu_wait()
          {

            switch (cond_)
            {
            case SHMEM_CMP_EQ:
              return (*raw_ == val_);
              break;

            case SHMEM_CMP_NE:
              return (*raw_ != val_);
              break;

            case SHMEM_CMP_GT:
              return (*raw_ > val_);
              break;

            case SHMEM_CMP_LE:
              return (*raw_ <= val_);
              break;

            case SHMEM_CMP_LT:
              return (*raw_ < val_);
              break;

            default:
              sst_throw(ssterror, "shmem::wait::evalu_wait - unknown comparision %d", cond_);
            }
          }
        };
    }
  }
}

#endif
