/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_SHMEM_ATOMIC_TRANSFER_H_
#define SSTMAC_SHMEM_ATOMIC_TRANSFER_H_

#include "shmem/messages/transfer.h"
#include <sstmac/common/messages/valuepayload.h>
#include "shmem/messages/fake_data.h"
#include "shmem/messages/atomic.h"

namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {

      template<typename T>
        class atomic_transfer : public transfer
        {

        protected:
          long addr_;
          int offset_;
          long from_pe_;
          long to_pe_;

          atomic::ptr at_;

          atomic_transfer(long addr, int offset, long frompe, long tope,
              atomic::ptr a) :
            addr_(addr), offset_(offset), from_pe_(frompe), to_pe_(tope),
                at_(a)
          {

          }

        public:
          typedef boost::intrusive_ptr<atomic_transfer> ptr;

          static ptr
          construct(long addr, int offset, long frompe, long tope,
              atomic::ptr a)
          {
            return ptr(new atomic_transfer(addr, offset, frompe, tope, a));
          }

          virtual std::string
          to_string() const
          {
            return "atomic_transfer";
          }

          virtual
          ~atomic_transfer()
          {

          }

          virtual long
          size_bytes() const
          {
            return at_->size_bytes();
          }

          virtual long
          get_remote_pe() const
          {
            return to_pe_;
          }

          virtual long
          get_local_pe() const
          {
            return from_pe_;
          }

          virtual TRANSFER_TYPES
          get_trans_type() const
          {
            return ATOMIC;
          }

          virtual void
          execute(transfer_api::ptr a) const
          {
            payload::const_ptr ret;
            if (addr_ >= 0)
            {
              symmetric::ptr s = a->get_symmetric_by_addr(addr_);

              void* r = s->get_raw();

              SSTMAC_DEBUG << "swap_transfer: executing on addr " << addr_
                    << " which is raw " << (long) r << " with offset "
                    << offset_ << "\n";
              T* casted = (T*) r;

              //casted += offset_;
              ret = valuepayload<T>::construct(*casted);

              at_->execute(a, addr_, offset_);
            }
            else
            {
              ret = fake_data::construct(sizeof(T));
            }

            a->send_get_response(const_ptr(this), ret);
          }

        };

    }
  }
}
#endif
