/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_SHMEM_INC_TRANSFER_H_
#define SSTMAC_SHMEM_INC_TRANSFER_H_

#include "shmem/messages/transfer.h"
//#include "shmem/api.h"

namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {

      template<typename T>
        class inc_transfer : public transfer
        {

        protected:

          long addr_;
          int offset_;
          long from_pe_;
          long to_pe_;

          inc_transfer(long addr, int offset, long frompe, long tope) :
            addr_(addr), offset_(offset), from_pe_(frompe), to_pe_(tope)
          {

          }

        public:
          typedef boost::intrusive_ptr<inc_transfer> ptr;

          static ptr
          construct(long addr, int offset, long frompe, long tope)
          {
            return ptr(new inc_transfer(addr, offset, frompe, tope));
          }

          virtual std::string
          to_string() const
          {
            return "inc_transfer";
          }

          virtual
          ~inc_transfer()
          {

          }

          virtual long
          size_bytes() const
          {
            return 0;
          }

          virtual long
          get_remote_pe() const
          {
            return to_pe_;
          }

          virtual long
          get_local_pe() const
          {
            return from_pe_;
          }

          virtual TRANSFER_TYPES
          get_trans_type() const
          {
            return ATOMIC;
          }

          /**
           * The API passed in is the remote api to act on.
           */
          virtual void
          execute(transfer_api::ptr a) const
          {
            symmetric::ptr s = a->get_symmetric_by_addr(addr_);

            void* r = s->get_raw();

            SSTMAC_DEBUG << "inc_transfer: executing on addr " << addr_
                  << " which is raw " << (long) r << " with offset " << offset_
                  << "\n";
            T* casted = (T*) r;
            casted += offset_;

            *casted = *casted + 1;

            a->check_wait(casted);
          }
        };

    }
  }
}
#endif
