/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_SHMEM_LOCK_OP_H_
#define SSTMAC_SHMEM_LOCK_OP_H_

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/debug.h>

#include "shmem/transfer_api.h"
namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {
      //  class api;

      class lock_op : public ptr_type
      {
      public:
        enum LOCK_TYPES
        {
          LOCK, // acquire lock, block until
          CLEAR, // clear lock
          TEST, // acquire lock, but don't block
          LOCK_OK, //  lock was acquired
          LOCK_NOK
        //  lock was not acquired (for test)
        };
      protected:
        LOCK_TYPES ltype_;

        long addr_;
        long frompe_;
        long tope_;

        lock_op(LOCK_TYPES ty, long a, long f, long t) :
          ltype_(ty), addr_(a), frompe_(f), tope_(t)
        {

        }
      public:
        typedef boost::intrusive_ptr<lock_op> ptr;
        typedef boost::intrusive_ptr<const lock_op> const_ptr;

        static ptr
        construct(LOCK_TYPES ty, long a, long f, long t)
        {
          return ptr(new lock_op(ty, a, f, t));
        }

        virtual
        ~lock_op()
        {
        }

        virtual std::string
        to_string() const
        {
          return "lock_op";
        }

        virtual long
        get_addr() const
        {
          return addr_;
        }

        virtual LOCK_TYPES
        get_ltype() const
        {
          return ltype_;
        }

        virtual long
        from_pe() const
        {
          return frompe_;
        }

        virtual long
        to_pe() const
        {
          return tope_;
        }

      };

    }
  }
}
#endif
