/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_SHMEM_transfer_H_
#define SSTMAC_SHMEM_transfer_H_

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/debug.h>

#include "shmem/transfer_api.h"
namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {
      //  class api;

      class transfer : public ptr_type
      {
      protected:
        static long tcount_;
        long tid_;

        static debug dbg_;

        transfer()
        {
          tid_ = tcount_++;

          if (!dbg_.is_real())
          {
            dbg_ = debug("<debug> shmem | transfer", logger::magenta);
            logger::register_logger(&dbg_);
          }
        }
      public:
        typedef boost::intrusive_ptr<transfer> ptr;
        typedef boost::intrusive_ptr<const transfer> const_ptr;

        enum TRANSFER_TYPES
        {
          GET, PUT, ATOMIC
        };

        virtual
        ~transfer()
        {
        }

        virtual long
        get_tid() const
        {
          return tid_;
        }

        virtual long
        size_bytes() const = 0;

        virtual long
        get_remote_pe() const = 0;

        virtual long
        get_local_pe() const = 0;

        virtual void
        execute(transfer_api::ptr a) const = 0;

        virtual TRANSFER_TYPES
        get_trans_type() const = 0;

      };

    }
  }
}
#endif
