/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SHMEM_transfer_api_H_
#define SSTMAC_SHMEM_transfer_api_H_

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/messages/payload.h>

#include "shmem/memory/symmetric.h"
#include "shmem/messages/transfer.h"

namespace sstmac
{
  namespace sw
  {

    namespace shmem
    {
      class transfer;

      class transfer_api : public virtual ptr_type
      {

      public:
        typedef boost::intrusive_ptr<transfer_api> ptr;

        virtual symmetric::ptr
        get_symmetric_by_addr(long addr) = 0;

        virtual void
        put_done(long tid) = 0;

        virtual void
        get_arrived(payload::const_ptr pay) = 0;

        virtual void
        send_put_complete(boost::intrusive_ptr<const transfer> t) const = 0;

        virtual void
        send_get_response(boost::intrusive_ptr<const transfer> t,
            payload::const_ptr pay) = 0;

        virtual void
        unblock() = 0;

        virtual symmetric::ptr
        get_fake_sym(size_t s) = 0;

        virtual void
        check_wait(void* base) = 0;

      };
    }

  }
}
#endif
