#ifndef SIMPLE_MESSAGE_H
#define SIMPLE_MESSAGE_H

#include <sstmac/hardware/network/network_message.h>
#include <sstmac/software/process/operatingsystem.h>

namespace simpmsg
{

class simple_message :
    public sstmac::hw::network_message,
    public sstmac::library_message
{
    public:
        typedef enum {
            send,
            rdma_get_start,
            rdma_put_start,
            rdma_put_ack,
            rdma_get_ack,
            msg_arrived,
            null_event
        } event_t;

        typedef boost::intrusive_ptr<simple_message> ptr;

    private:
        event_t event_;

        sstmac::sw::taskid dest_;

        sstmac::sw::taskid src_;

        long num_bytes_;

        long msg_id_;

        void reverse();

    protected:
        void clone_into(const simple_message::ptr& msg) const;

        void clone_into(MESSAGE_TYPES ty, const simple_message::ptr& msg) const;

    public:
        static const int ack_size;
        static const int header_size;

        simple_message(long num_bytes);

        static const char*
        tostr(event_t ev);

        event_t event() const {
            return event_;
        }

        void set_event(event_t ev)
        {
            event_  = ev;
        }

        long msg_id() const {
            return msg_id_;
        }

        void set_msg_id(long id) {
            msg_id_ = id;
        }

        int dest() const {
            return dest_.id;
        }

        void set_dest(int dest) {
            dest_ = sstmac::sw::taskid(dest);
        }

        int src() const {
            return src_.id;
        }

        void set_src(int src) {
            src_ = sstmac::sw::taskid(src);
        }

        long get_byte_length() const {
            return num_bytes_ >= 0 ? num_bytes_: header_size;
        }

        void set_virtual_num_bytes() {
            num_bytes_ = -(abs(num_bytes_));
        }

        const sstmac::sw::taskid&
        source_task() const {
            return src_;
        }

        const sstmac::sw::taskid&
        dest_task() const {
            return dest_;
        }

        long
        source_thread(const sstmac::sw::operatingsystem::ptr& os) const;

        long
        dest_thread(const sstmac::sw::operatingsystem::ptr& os) const;

        sstmac::sst_message::ptr
        clone(MESSAGE_TYPES t) const;

        void
        mutate(mutation_t mut);
};

class simpmsg_queue :
    public sstmac::ptr_type
{
    private:
       std::list<simple_message::ptr> pending_messages_;

       std::list<sstmac::sw::key::ptr> blocked_keys_;

       sstmac::sw::operatingsystem::ptr os_;

       simpmsg_queue(const sstmac::sw::operatingsystem::ptr& os);

    public:
        typedef boost::intrusive_ptr<simpmsg_queue> ptr;

        static ptr
        construct(const sstmac::sw::operatingsystem::ptr& os);

        static ptr
        construct();

        simple_message::ptr
        poll_until_message();

        void put_message(const simple_message::ptr& message);

        std::string
        to_string() const {
            return "simple message queue";
        }
};

}

#endif // SIMPLE_MESSAGE_H
