#include <sstmac/software/api/simpmsg/simpmsg.h>
#include <sstmac/software/api/simpmsg/sstmac_simpmsg_api.h>
#include <sstmac/software/process/operatingsystem.h>

using namespace sstmac;
using namespace sstmac::sw;

namespace simpmsg {

uint64_t simpmsg_thread::num_threads_ = 0;

simpmsg_thread::simpmsg_thread()
{
    std::string libcomp_name = sstprintf("simpmsg_thread_compute_%lu", num_threads_);
    compute_ = lib_compute_time::construct(libcomp_name);
    ++num_threads_;
}

void
simpmsg_thread::start()
{
    operatingsystem::ptr fullos =
        operatingsystem::get_current_os();
    fullos->register_lib(compute_);
    operatingsystem::get_current_thread()->spawn(this);
}

void
simpmsg_thread::compute(double sec)
{
    compute_->compute(timestamp(sec));
}

simpmsg_api::ptr
get_current_simpmsg()
{
  thread::ptr t = OS_API::get_current_thread();
  return t->get_api<simpmsg_api> ();
}

simpmsg_api::ptr
simp_api()
{
    return get_current_simpmsg();
}

void
simp_init()
{
    get_current_simpmsg()->init();
}

void
simp_finalize()
{
    get_current_simpmsg()->finalize();
}

int simp_rank()
{
    return get_current_simpmsg()->rank();
}

int simp_nproc()
{
    return get_current_simpmsg()->nproc();
}

/**
    @param dst The destination to send to
*/
void
simp_send(int dst, const simple_message::ptr& msg)
{
    get_current_simpmsg()->send(dst, msg);
}

void
simp_rdma_put(int dst, const simple_message::ptr& msg)
{
    get_current_simpmsg()->rdma_put(dst, msg);
}

void
simp_rdma_get(int dst, const simple_message::ptr& msg)
{
    get_current_simpmsg()->rdma_get(dst, msg);
}

simple_message::ptr
simp_poll()
{
    return get_current_simpmsg()->poll_until_notification();
}

double
simp_time()
{
    return operatingsystem::get_current_os()->now().sec();
}

void compute(double sec)
{
    thread::ptr thr = OS_API::get_current_thread();
    app::ptr my_app = safe_cast(thr, app);
    my_app->compute(timestamp(sec));
}



}
