#ifndef SSTMAC_SIMPMSG_API_H
#define SSTMAC_SIMPMSG_API_H

#include <sstmac/software/api/api.h>
#include <sstmac/software/api/simpmsg/simple_message.h>
#include <sstmac/software/process/app.h>
#include <sstmac/software/services/service.h>
#include <sstmac/software/process/pmi.h>

namespace simpmsg
{


class simpmsg_api :
    public sstmac::sw::api,
    public sstmac::sw::process_manager
{

    private:
       int rank_;

       int nproc_;

       std::string server_libname_;

       sstmac::sw::softwareid sid_;

       sstmac::sw::environment::const_ptr env_;

       simpmsg_queue::ptr queue_;

       typedef boost::unordered_map<long, simple_message::event_t> ack_map;
       ack_map pending_acks_;

       long msg_id_ticker_;

    protected:
        simpmsg_api();

    public:
        typedef boost::intrusive_ptr<simpmsg_api> ptr;

        void init(const sstmac::sw::softwareid& sid);

        virtual void init_os(const sstmac::sw::OS_API::ptr &os);

        void init();

        void finalize();

        int rank() const {
            return rank_;
        }

        int nproc() const {
            return nproc_;
        }

        static ptr
        construct(const sstmac::sim_parameters::ptr& params)
        {
            return new simpmsg_api;
        }

        void handle(const simple_message::ptr& msg);

        void send(int dst, const simple_message::ptr& msg);

        void rdma_put(int dst, const simple_message::ptr& msg);

        void rdma_get(int src, const simple_message::ptr& msg);

        simple_message::ptr
        poll_until_notification();

};

class simpmsg_server :
    public sstmac::sw::service
{

    private:
        int appid_;

        boost::unordered_map<int, simpmsg_api::ptr> procs_;

        simpmsg_api::ptr
        get_proc(int rank) const;

    protected:
        simpmsg_server(int appid);

    public:
        typedef boost::intrusive_ptr<simpmsg_server> ptr;

        void handle(const sstmac::sst_message::ptr& msg);

        void register_proc(int rank, const simpmsg_api::ptr& proc);

        static ptr
        construct(int appid);

        sstmac::sw::service::ptr
        clone();
};

}


#endif // SSTMAC_SIMPMSG_API_H
