#include <sstmac/software/api/sockets/socketapi.h>
#include <sstmac/software/api/sockets/lib_sockets.h>
#include <sstmac/software/process/app.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/software/services/launch/launcher.h>
#include <algorithm>

using namespace sstmac;
using namespace sstmac::sw;

static std::string socket_libname = lib_compute_memmove::get_name("libsocket");

static socketapi::ptr
get_lib_socket()
{
  thread::ptr t = OS_API::get_current_thread();
  return t->get_api<socketapi>();
}

extern "C" int
_SSTMAC_socket(socket_domain_t domain, socket_type_t type, socket_protocol_t protocol)
{
    int fd = get_lib_socket()->allocate_socket(domain, type, protocol);
    return fd;
}

extern "C" int
_SSTMAC_close(int fd)
{
    socketapi::ptr socklib = get_lib_socket();
    socklib->close(fd);
    return 0;
}

extern "C" ssize_t
_SSTMAC_sendto(int fd, const void *message, size_t length, int flags, const sockaddr *dest_addr, socklen_t dest_len)
{
    socketapi::ptr socklib = get_lib_socket();
    nodeaddress::ptr addr = nodeaddress::get_addr(dest_addr->sin_addr.s_addr);
    socklib->connect(fd, dest_addr->sin_port, addr);

    bool nonblocking = flags & MSG_DONTWAIT;
    bool need_ack = !nonblocking;
    socklib->send(fd, length, need_ack);
    socklib->disconnect(fd);

    //for now just always assume that every byte gets sent
    return length;
}

extern "C" ssize_t
_SSTMAC_recvfrom(int fd, void *buffer, size_t length, int flags, sockaddr *address, socklen_t *address_len)
{
    socketapi::ptr socklib = get_lib_socket();
    bool blocking_recv = flags & MSG_WAITALL;
    bool blocking_accept = !(flags & MSG_DONTWAIT);
    bool peek_only = flags & MSG_PEEK;
    bool oob = flags & MSG_OOB;
    if (oob){
        sst_throw(valueerror, "simulated socket API in SST/macro does not support MSG_OOB flag");
    }

    unsigned long remote_id  = address->sin_addr.s_addr;
    nodeaddress::ptr remote_addr = remote_id == INADDR_ANY ? nodeaddress::any_addr : nodeaddress::get_addr(remote_id);
    nodeaddress::ptr conn_addr = socklib->accept(fd, remote_addr, blocking_accept);
    if (!conn_addr){
        if (blocking_accept){
            sst_throw(illformederror, "blocking accept on fd %d did not return valid connection");
        }
        else {
            return EWOULDBLOCK;
        }
    }

    if (peek_only){
        if (blocking_recv || blocking_accept)
            sst_throw(illformederror, "cannot specify booth MSG_PEEK and MSG_WAITALL in recvfrom on socket");

        sstmac::sw::socket::ptr sock = socklib->get_socket(fd);
        return std::min(sock->bytes_available, (long) length);
    }

    ssize_t bytes_recvd = socklib->recv(fd, length, blocking_recv);
    address->sin_addr.s_addr = conn_addr->unique_id();

    socklib->unaccept(fd);
    return bytes_recvd;
}

extern "C" unsigned long
_SSTMAC_inet_addr(const char *name)
{
    bool localhost = !(::strcmp(name, "localhost")) || !(::strcmp(name, "127.0.0.1"));
    if (localhost){
        return operatingsystem::get_current_node()->unique_id();
    }
    else {
        return operatingsystem::get_current_env()->get_node_address(name)->unique_id();
    }
}

extern "C" int
_SSTMAC_bind(int fd, const sockaddr *addr, socklen_t addr_len)
{
    socketapi::ptr socklib = get_lib_socket();
    unsigned short number = addr->sin_port;
    nodeaddress::ptr node_addr = operatingsystem::get_current_node();
    socklib->bind(fd, number, node_addr);
    return 0;
}
