/*  shared.cc                                                                */

#include "sstmac_upc/memory/shared.h"

namespace sstmac
{
    namespace sw
    {
        namespace upc
        {
            debug shared::dbg_;
            std::vector<shared::ptr>* shared::constructed_;
            
            shared::shared() :
                raw_(NULL), size_(0), nblocks_(0), starting_thread_(0), real_(false)
            {
                shared_map_add(ptr(this));
                
                if (!dbg_.is_real())
                {
                    dbg_ = debug("<debug> shared | upc", logger::yellow);
                    logger::register_logger(&dbg_);
                }
            }
            
            shared::shared(size_t nblocks, size_t nbytes) : 
                size_(nblocks * nbytes), nblocks_(nblocks), starting_thread_(0), real_(true)
            {
                raw_ = malloc(nblocks * nbytes);
                
                shared_map_add(ptr(this));
                
                if (!dbg_.is_real())
                {
                    dbg_ = debug("<debug> shared | upc", logger::yellow);
                    logger::register_logger(&dbg_);
                }
            }
            
            shared::shared(size_t nblocks, size_t nbytes, bool real) :
                size_(nblocks * nbytes), nblocks_(nblocks), starting_thread_(0), real_(real)
            {
                if (real_)
                    raw_ = malloc(nblocks * nbytes);
                
                shared_map_add(ptr(this));
                
                if (!dbg_.is_real())
                {
                    dbg_ = debug("<debug> shared | upc", logger::yellow);
                    logger::register_logger(&dbg_);
                }
            }
            
            shared::shared(size_t nblocks, size_t nbytes, size_t thread) : 
                size_(nblocks * nbytes), nblocks_(nblocks), starting_thread_(thread), real_(true)
            {
                if (real_)
                    raw_ = malloc(nblocks * nbytes);
                
                shared_map_add(ptr(this));
                
                if (!dbg_.is_real())
                {
                    dbg_ = debug("<debug> shared | upc", logger::yellow);
                    logger::register_logger(&dbg_);
                }
            }
            
            void
            shared::shared_map_add(ptr sobj)
            {
                if (!constructed_)
                {
                    constructed_ = new shared_map();
                }
                constructed_->push_back(sobj);
            }
            
            void *
            shared::get_offset_raw(size_t baseaddr, size_t threadid, size_t phase, size_t typesize, size_t blocksize)
            {
                SSTMAC_DEBUG << "upc::shared::get_offset_raw(" << baseaddr << ", " << threadid << ", " << phase << ", " << typesize << ", " << blocksize << ")\n";
                return (void *)((char *)raw_ + (threadid - starting_thread_) * typesize * blocksize + baseaddr + phase * typesize);
            }
            
            void *
            shared::get_offset_raw(size_t baseaddr, size_t threadid, size_t phase, size_t typesize)
            {
                size_t blocksize = size_ / (nblocks_ * typesize);
                return get_offset_raw(baseaddr, threadid, phase, typesize, blocksize);
            }
            
        }
    }
}
