/*  shared.h                                                             */

#ifndef SSTMAC_SHMEM_shared_H_
#define SSTMAC_SHMEM_shared_H_

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/debug.h>

namespace sstmac
{
    namespace sw
    {
        namespace upc
        {
            // Maintains a simple map of a shared block of memory.
            // Has enough info to partition the memory block into the specified number of blocks.
            class shared : public ptr_type
            {
                private:
                    void *raw_;
                    size_t size_;
                    size_t nblocks_;
                    
                    // Normally zero, except for objects allocated by upc_alloc()
                    size_t starting_thread_;
                    
                    bool real_;
                    
                    static debug dbg_;
                    
                    shared();
                    
                    shared(size_t nblocks, size_t nbytes);
                    
                    shared(size_t nblocks, size_t nbytes, bool real);
                    
                    shared(size_t nblocks, size_t nbytes, size_t thread);
                    
                    void
                    shared_map_add(boost::intrusive_ptr<shared> sobj);
                    
                public:
                    typedef boost::intrusive_ptr<shared> ptr;
                    
                    typedef std::vector<shared::ptr> shared_map;
                    static shared_map* constructed_;
                    
                    virtual
                    ~shared()
                    {
                        if (real_) free(raw_);
                    }
                    
                    virtual std::string
                    to_string() const
                    {
                        return "shared";
                    }
                    
                    static ptr
                    construct()
                    {
                        return ptr(new shared());
                    }
                    
                    static ptr
                    construct(size_t nblocks, size_t nbytes)
                    {
                        return ptr(new shared(nblocks, nbytes));
                    }
                    
                    static ptr
                    construct(size_t nblocks, size_t nbytes, bool real)
                    {
                        return ptr(new shared(nblocks, nbytes, real));
                    }
                    
                    static ptr
                    construct(size_t nblocks, size_t nbytes, size_t thread)
                    {
                        return ptr(new shared(nblocks, nbytes, thread));
                    }
                    
                    // Member functions.
                    
                    bool
                    is_real() const
                    {
                        return real_;
                    }
                    
                    size_t
                    get_nblocks() const
                    {
                        return nblocks_;
                    }
                    
                    size_t
                    get_size() const
                    {
                        return size_;
                    }
                    
                    void *
                    get_raw() const
                    {
                        return raw_;
                    }
                    
                    size_t
                    get_starting_thread() const
                    {
                        return starting_thread_;
                    }
                    
                    void *
                    get_offset_raw(size_t baseaddr, size_t threadid, size_t phase, size_t typesize, size_t blocksize);
                    
                    void *
                    get_offset_raw(size_t baseaddr, size_t threadid, size_t phase, size_t typesize);
                    
                    // Copy constructor / assignment operator.
                    
                private:
                    
                    shared(const shared& other);
                    
                    shared&
                    operator=(const shared& other);
                    
            };
        }
    }
}

#endif
