/*  shared_heap.cc                                                           */

#include "sstmac_upc/memory/shared_heap.h"

namespace sstmac
{
  namespace sw
  {
    namespace upc
    {
      debug shared_heap::dbg_;

      shared_heap::shared_heap()
      {
        shared_count_ = 0;
        lock_count_ = 0;

        if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> shared_heap | upc", logger::red);
          logger::register_logger(&dbg_);
        }
      }

      void
      shared_heap::add_shared(shared::ptr sobj)
      {
        SSTMAC_DEBUG << "upc::shared_heap::add_shared(" << sobj << ")\n";

        if (sobj)
        {
          shared_list_.push_back(sobj);
          shared_count_++;
        }
      }

      void
      shared_heap::delete_shared(shared::ptr sobj)
      {
        SSTMAC_DEBUG << "upc::shared_heap::delete_shared(" << sobj << ")\n";

        if(!sobj)
          return;

        std::vector<shared::ptr>::iterator it, end;
        end = shared_list_.end();
        it = shared_list_.begin();
        while (it != end && *it != sobj)
          it++;
        if (*it == sobj)
        {
          shared_list_.erase(it);
          shared_count_--;
        }
        else
        {
          sst_throw(ssterror, "upc::shared_heap::delete_shared - could not delete unregistered shared object.");
        }
      }

      bool
      shared_heap::contains_shared(shared::ptr sobj)
      {
        SSTMAC_DEBUG << "upc::shared_heap::contains_shared(" << sobj << ")\n";

        std::vector<shared::ptr>::iterator it, end;
        end = shared_list_.end();
        it = shared_list_.begin();
        while (it != end && *it != sobj)
          it++;
        return (*it == sobj);
      }

      void
      shared_heap::add_lock(lock *lck)
      {
        SSTMAC_DEBUG << "upc::shared_heap::add_lock(" << lck->to_string()
              << ")\n";

        lock_list_.push_back(lck);
        lock_count_++;
      }

      void
      shared_heap::delete_lock(lock *lck)
      {
        SSTMAC_DEBUG << "upc::shared_heap::delete_lock(" << lck->to_string()
              << ")\n";

        std::vector<lock*>::iterator it, end;
        end = lock_list_.end();
        it = lock_list_.begin();
        while (it != end && *it != lck)
          it++;
        if (*it == lck)
        {
          lock_list_.erase(it);
          lock_count_--;
        }
        else
        {
          sst_throw(ssterror, "upc::shared_heap::delete_lock - could not delete unregistered upc lock object.");
        }
      }
    }
  }
}
