/*  get_transfer.h                                                           */

#ifndef SSTMAC_UPC_get_transfer_H_
#define SSTMAC_UPC_get_transfer_H_

#include "sstmac_upc/messages/transfer.h"
#include <sstmac/common/messages/vectorpayload.h>
#include "sstmac_upc/messages/fake_data.h"

namespace sstmac
{
    namespace sw
    {
        namespace upc
        {

            template<typename T>
            class get_transfer : public transfer
            {
                protected:
                    
                    shared::ptr shobj_;
                    size_t size_;
                    size_t baseaddr_;
                    size_t phase_;
                    long from_thread_;
                    long to_thread_;
                    bool real_;

                    get_transfer(shared::ptr shobj, size_t size, size_t baseaddr, size_t phase, long from_thread, long to_thread, bool real) :
                        shobj_(shobj), size_(size), baseaddr_(baseaddr), phase_(phase), from_thread_(from_thread), to_thread_(to_thread), real_(real)
                    {

                    }

                public:
                    typedef boost::intrusive_ptr<get_transfer> ptr;

                    static ptr
                    construct(shared::ptr shobj, size_t baseaddr, size_t phase, long from_thread, long to_thread)
                    {
                        return ptr(new get_transfer(shobj, 1, baseaddr, phase, from_thread, to_thread, true));
                    }
                    
                    static ptr
                    construct(shared::ptr shobj, size_t baseaddr, size_t phase, long from_thread, long to_thread, bool real)
                    {
                        return ptr(new get_transfer(shobj, 1, baseaddr, phase, from_thread, to_thread, real));
                    }

                    static ptr
                    construct(shared::ptr shobj, size_t size, size_t baseaddr, size_t phase, long from_thread, long to_thread)
                    {
                        return ptr(new get_transfer(shobj, size, baseaddr, phase, from_thread, to_thread, true));
                    }
                    
                    static ptr
                    construct(shared::ptr shobj, size_t size, size_t baseaddr, size_t phase, long from_thread, long to_thread, bool real)
                    {
                        return ptr(new get_transfer(shobj, size, baseaddr, phase, from_thread, to_thread, real));
                    }

                    virtual std::string
                    to_string() const
                    {
                        return "get_transfer";
                    }

                    virtual
                    ~get_transfer()
                    {
                    }

                    virtual long
                    size_bytes() const
                    {
                        return 0;
                    }

                    virtual long
                    get_remote_thread() const
                    {
                        return to_thread_;
                    }

                    virtual long
                    get_local_thread() const
                    {
                        return from_thread_;
                    }

                    virtual TRANSFER_TYPES
                    get_trans_type() const
                    {
                        return GET;
                    }

                    virtual void
                    execute(transfer_api::ptr a) const
                    {
                        payload::const_ptr ret;
                        if (real_ && a->check_shared_heap(shobj_))
                        {
                            void* r = shobj_->get_offset_raw(baseaddr_, to_thread_, phase_, sizeof(T));

                            SSTMAC_DEBUG << "get_transfer: executing on thread " << to_thread_
                                    << " with raw " << (long) r << " with base address "
                                    << baseaddr_ << " and phase " << phase_ << "\n";
                            T* casted = (T*) r;
                            
                            ret = vector1payload<T, T*>::construct(casted, size_);
                        }
                        else
                        {
                            ret = fake_data::construct(sizeof(T) * size_);
                        }

                        a->send_get_response(const_ptr(this), ret);
                    }
                
            };
            
        }
    }
}

#endif
