/*  message.h                                                                */

#ifndef SSTMAC_UPC_message_H_
#define SSTMAC_UPC_message_H_

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/hardware/network/network_message.h>

namespace sstmac
{
    namespace sw
    {
        namespace upc
        {
            const int UPC_MSG_OVERHEAD = 16;

            class message : public library_message, public hw::network_message
            {
                public:
                    enum UPC_MSG_TYPE
                    {
                        TRAN, PUT_COMPLETE, GET_RESPONSE
                    };
                    
                protected:
                    UPC_MSG_TYPE upctype_;
                    boost::intrusive_ptr<const ptr_type> data_;

                    long bytes_;

                    taskid src_;
                    taskid dst_;

                    message(UPC_MSG_TYPE t, const std::string& libname,
                        const nodeaddress::ptr& toaddr, const nodeaddress::ptr& fromaddr,
                        taskid src, taskid dst, boost::intrusive_ptr<const ptr_type> d,
                        sst_message::MESSAGE_TYPES ty) :
                    sst_message(ty), library_message(libname),
                        network_message(toaddr, fromaddr), upctype_(t), data_(d),
                        src_(src), dst_(dst)
                    {
                        bytes_ = -1;
                        if (data_)
                        {
                            transfer::const_ptr tran = boost::dynamic_pointer_cast<
                                const transfer>(data_);
                            if (tran)
                            {
                                bytes_ = UPC_MSG_OVERHEAD + tran->size_bytes();
                            }
                            else
                            {
                                payload::const_ptr p =
                                    boost::dynamic_pointer_cast<const payload>(data_);
                                if (p)
                                {
                                    bytes_ = p->get_byte_length() + UPC_MSG_OVERHEAD;
                                }
                            }

                        }
                        if (bytes_ < 0)
                            bytes_ = UPC_MSG_OVERHEAD;
                    }

                public:
                    typedef boost::intrusive_ptr<message> ptr;

                    static ptr
                    construct(
                        UPC_MSG_TYPE t,
                        const std::string& libname,
                        const nodeaddress::ptr& toaddr,
                        const nodeaddress::ptr& fromaddr,
                        taskid src,
                        taskid dst,
                        sst_message::MESSAGE_TYPES ty,
                        boost::intrusive_ptr<const ptr_type> d = boost::intrusive_ptr<
                            const ptr_type>())
                    {
                        return ptr(new message(t, libname, toaddr, fromaddr, src, dst, d, ty));
                    }

                    virtual
                    ~message()
                    {
                    }

                /*  void
                    reverse()
                    {
                        //also flip the addresses
                        nodeaddress::ptr dst = fromaddr_;
                        nodeaddress::ptr src = toaddr_;
                        toaddr_ = dst;
                        fromaddr_ = src;

                        taskid srcid = src_;
                        taskid dstid = dst_;
                        src_ = dstid;
                        dst_ = srcid;
                    }

                    virtual void
                    mutate(mutation_t mut)
                    {
                        if (mut == sst_message::RDMA_GET_REQ_TO_RSP)
                        {

                            msgtype_ = RDMA_GET_RSP;

                            reverse();
                            bytes_ = type_.size * count_;
                            //no MPI overhead on this - just the payload
                            //recompute_bytes();
                        }
                        else
                        {
                            sst_throw(valueerror, "invalid mutation type received by shmem::message: %s", sst_message::tostr(mut));
                        }
                    }*/

                    virtual std::string
                    to_string() const
                    {
                        return "message";
                    }

                    virtual UPC_MSG_TYPE
                    get_upc_type() const
                    {
                        return upctype_;
                    }

                    virtual boost::intrusive_ptr<const ptr_type>
                    get_data()
                    {
                        return data_;
                    }

                    virtual bool
                    get_needs_ack() const
                    {
                        return false;
                    }

                    virtual const sw::taskid&
                    source_task() const
                    {
                        return src_;
                    }

                    virtual const sw::taskid&
                    dest_task() const
                    {
                        return dst_;
                    }

                    virtual long
                    get_byte_length() const
                    {
                        return bytes_;
                    }
            };
        }
    }
}

#endif
