/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/compute/lib_compute_time.h>
#include <sstmac/software/libraries/libwaiters.h>

namespace sstmac
{
  namespace sw
  {

    debug lib_compute_time::dbg_;

    lib_compute_time::lib_compute_time(softwareid id)
    {
      if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> lib_compute_time | lib_compute");
          logger::register_logger(&dbg_);
        }

      libname_ = "computelib" + id.to_string();
    }

    lib_compute_time::lib_compute_time(const std::string& id)
    {
      if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> lib_compute_time | lib_compute");
          logger::register_logger(&dbg_);
        }

      libname_ = id;
    }

    lib_compute_time::ptr
    lib_compute_time::construct(softwareid id)
    {
      lib_compute_time::ptr retval(new lib_compute_time(id));
      return retval;
    }

    lib_compute_time::ptr
    lib_compute_time::construct(const std::string& id)
    {
      lib_compute_time::ptr retval(new lib_compute_time(id));
      return retval;
    }

    lib_compute_time::~lib_compute_time()
    {

    }

    void
    lib_compute_time::compute(timestamp time)
    {
      SSTMACBacktrace("Compute Time");
      if (supported())
        {
          if (time.sec() < 0)
            {
              throw valueerror(
                  "lib_compute_time can't compute for less than zero time");
            }
          sst_message::ptr cmsg = sleep_message::construct(get_lib_name(), time);

          SSTMAC_DEBUG << "lib_compute_time: " << sstprintf("%p", this)
                       << " executing compute kernel for "
                       << time << "\n";
          os_->execute_kernel(AMI::AMI_COMP_TIME, cmsg);

          SSTMAC_DEBUG << "lib_compute_time: blocking \n";
          wait_this(cmsg);
        }
      else
        {
          sst_throw(valueerror,
              "lib_compute_time requires the presence of a processor model which can compute for just an amount of time");
        }
    }

  }
} //end of namespace sstmac
