/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_MEM_ACCESS_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_MEM_ACCESS_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/messages/eventdata.h>
#include <sstmac/common/eventcallback.h>
#include <sstmac/software/ami/ami.h>
#include <sstmac/software/process/operatingsystem.h>

namespace sstmac
{
  namespace sw
  {

    class mem_access : 
        public eventdata,
        public callback_message,
        public serializable_type<mem_access>
    {
    ImplementSerializable(mem_access)

    protected:
      mem_access(long bytes,
                 const std::string& libn,
                 AMI::AMI_COMP_FUNC access_type,
                 long srcth, long destth);

      long bytes_;

      AMI::AMI_COMP_FUNC func_;

      event_step_callback::ptr page_event_cb_;

    public:

      typedef boost::intrusive_ptr<mem_access> ptr;

      static ptr
      construct(long bytes,
                const std::string& libn,
                AMI::AMI_COMP_FUNC access_type,
                long thr);

      static ptr
      construct(long bytes,
                const std::string& libn,
                AMI::AMI_COMP_FUNC access_type,
                long srcth, long dstth);


      AMI::AMI_COMP_FUNC
      get_function() const;

      virtual long
      get_byte_length() const;

      long
      source_thread(const sw::operatingsystem::ptr& os) const;

      long
      dest_thread(const sw::operatingsystem::ptr& os) const;

      event_step_callback::ptr
      pop_page_event_callback();

      nodeaddress::ptr toaddr() const;

      nodeaddress::ptr fromaddr() const;

      void set_page_event_callback(const event_step_callback::ptr& cb);

      long uid_;
      long srcthread_;
      long dstthread_;
    };

  }
}

#endif
