/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_local_file_system_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_local_file_system_H_INCLUDED

#include <sstmac/software/libraries/library.h>
#include <sstmac/software/libraries/filesystem/openfile.h>

namespace sstmac
{
  namespace sw
  {

    class local_file_system : public library
    {
    protected:
      local_file_system(softwareid id)
      {
        libname_ = "localfilesystem" + id.to_string();
      }


    public:
      typedef boost::intrusive_ptr<local_file_system> ptr;

      static ptr
      construct(softwareid id)
      {
        return ptr(new local_file_system(id));
      }


      virtual
      ~local_file_system()
      {
      }


      openfile::ptr
      open(const std::string& name);

      void
      read(openfile::ptr file, long bytes);
      void
      write(openfile::ptr file, long bytes);

      void
      seek(openfile::ptr file, long bytes);

      void
      close(openfile::ptr file);

      virtual bool
      supported() const;

    };

  }
} /* namespace sstmac */
#endif /* local_file_system_H_ */
