/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_OPENFILE_PARALLEL_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_OPENFILE_PARALLEL_H_INCLUDED

#include <sstmac/software/libraries/filesystem/file_descriptor.h>
#include <sstmac/software/libraries/filesystem/openfile.h>
#include <math.h>

namespace sstmac
{
  namespace sw
  {

    class openfile_parallel : public openfile
    {
    protected:
      openfile_parallel(file_descriptor::ptr fd) :
          openfile(fd->name_, -1), file_(fd)
      {

      }

    public:
      typedef boost::intrusive_ptr<openfile_parallel> ptr;

      static ptr
      construct(file_descriptor::ptr fd)
      {
        return ptr(new openfile_parallel(fd));
      }

      virtual std::string
      to_string() const
      {
        return "openfile_parallel";
      }

      virtual void
      update_max_access(long bytes)
      {

        if (bytes > file_->total_size_)
          {
            long diff = bytes - file_->total_size_;
            long each = ceil(diff / file_->nodes_.size());

            for (uint i = 0; i < file_->sizes_.size(); i++)
              {
                file_->sizes_[i] += each;
              }
            file_->total_size_ = bytes;
          }
      }

      file_descriptor::ptr file_;

    };

  }
}

#endif
