#ifndef LIBWAITERS_H
#define LIBWAITERS_H

#include <sstmac/software/process/os_api.h>
#include <sstmac/common/debug.h>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/software/process/waiter.h>
#include <sstmac/software/process/key.h>
#include <sstmac/software/process/blockwaiter.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/software/libraries/unblockhandler.h>

#include <sstmac/software/libraries/library.h>
#include <sstmac/software/libraries/wait.h>


namespace sstmac {
    namespace sw {

    class callback_lib_waiter :
        public library_waiter
    {
        public:
            typedef boost::intrusive_ptr<callback_lib_waiter> ptr;

            virtual void
            wait_msg(const sst_message::ptr &msg);

            virtual void
            incoming_msg(const sst_message::ptr &msg);

            static ptr
            construct(){
                return new callback_lib_waiter;
            }

            std::string
            to_string() const {
                return "callback library waiter";
            }
    };

    class serial_blocking_lib_waiter :
        public library_waiter
    {
        protected:
            operatingsystem::ptr os_;
            key::ptr key_;
            key::category key_cat_;

            serial_blocking_lib_waiter(key::category cat);

        public:
            typedef boost::intrusive_ptr<serial_blocking_lib_waiter> ptr;

            virtual void
            init_os(const operatingsystem::ptr &os){
                os_ = os;
                key_ = key::construct(key_cat_);
            }

    };

    class serial_single_msg_lib_waiter :
        public serial_blocking_lib_waiter
    {
        protected:
            sst_message::ptr pending_msg_;

            serial_single_msg_lib_waiter(key::category cat);

        public:
            typedef boost::intrusive_ptr<serial_single_msg_lib_waiter> ptr;

            static ptr
            construct(key::category cat){
                return new serial_single_msg_lib_waiter(cat);
            }

            virtual void
            wait_msg(const sst_message::ptr &msg);

            virtual void
            incoming_msg(const sst_message::ptr& msg);

            std::string
            to_string() const {
                return "serial single message library waiter";
            }

    };

    class serial_generic_lib_waiter :
        public serial_blocking_lib_waiter
    {
        private:
            wait::ptr wait_req_;

        protected:
            serial_generic_lib_waiter(key::category cat);

        public:
            typedef boost::intrusive_ptr<serial_generic_lib_waiter> ptr;

            static ptr
            construct(key::category cat){
                return new serial_generic_lib_waiter(cat);
            }

            virtual void
            wait_msg(const sst_message::ptr& msg);

            virtual sst_message::ptr
            wait_msg_type(sst_message::MESSAGE_TYPES ty, int count);

            virtual void
            incoming_msg(const sst_message::ptr& msg);

            std::string
            to_string() const {
                return "serial generic message library waiter";
            }
    };

    class multithreaded_blocking_lib_waiter :
        public library_waiter
    {
        protected:
            key::category cat_;
            std::map<sst_message::ptr, key::ptr> keys_;
            operatingsystem::ptr os_;

            multithreaded_blocking_lib_waiter(key::category cat);

        public:
            typedef boost::intrusive_ptr<multithreaded_blocking_lib_waiter> ptr;

            static ptr
            construct(key::category cat){
                return new multithreaded_blocking_lib_waiter(cat);
            }

            virtual void
            wait_msg(const sst_message::ptr &msg);

            virtual void
            incoming_msg(const sst_message::ptr &msg);

            std::string
            to_string() const {
                return "multithreaded blocking library waiter";
            }

            virtual void
            init_os(const operatingsystem::ptr &os){
                os_ = os;
            }
    };

  }
}

#endif // LIBWAITERS_H
