/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPIIO_MPIIOAPI_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPIIO_MPIIOAPI_H_INCLUDED

#include <sstmac/software/api/mpi/mpiapi.h>
#include <sstmac/software/libraries/mpiio/mpiio_file.h>

namespace sstmac
{
  namespace sw
  {

    enum mpi_mode {
      mpi_mode_rdonly = 1,
      mpi_mode_rdwr = 2,
      mpi_mode_wronly = 4,
      mpi_mode_create = 8,
      mpi_mode_excl = 16,
      mpi_mode_delete_on_close = 32,
      mpi_mode_unique_opn = 64,
      mpi_mode_sequential = 128,
      mpi_mode_append = 256
    };

    class mpiioapi : public library
    {
    protected:
      mpiioapi(softwareid id, mpiapi::ptr m) : mpi_(m)
      {

        libname_ = "mpiioapi" + id.to_string();
      }

      mpiapi::ptr mpi_;

    public:
      typedef boost::intrusive_ptr<mpiioapi> ptr;

      static ptr
      construct(softwareid id, mpiapi::ptr m)
      {
        return ptr(new mpiioapi(id, m));
      }
      virtual
      ~mpiioapi();


      // --------- the functions -------- //
      void mpi_file_open(const mpicomm::ptr &comm, std::string filename, int mode, int info, mpiio_file::ptr);
      void mpi_file_close();


      void mpi_file_seek();
      void mpi_file_set_view();
      void mpi_file_set_atomicity();

      void mpi_file_read();
      void mpi_file_read_at();
      void mpi_file_read_all();

      void mpi_file_write();
      void mpi_file_write_at();
      void mpi_file_write_all();

    };

  }
}
#endif /* MPIIOAPI_H_ */
