#include "libtopomap.hpp"
#include "MersenneTwister.h"

int TPM_Map_annealing(int r, TPM_Graph *ptg_ref, TPM_Graph_ewgts *ptgc_ref, TPM_Routes *routes_ref, TPM_Graph_vwgts *nprocs_ref, TPM_Graph *ltg_ref, TPM_Graph_ewgts *weights_ref, TPM_Mapping *mapping_ref, double *cong, int *dil, int *iters) {
  TPM_Graph& ptg = *ptg_ref;
  TPM_Graph_ewgts& ptgc = *ptgc_ref;
  TPM_Routes& routes = *routes_ref; // routes in h
  TPM_Graph& ltg = *ltg_ref;
  TPM_Graph_ewgts& ltgw = *weights_ref; // ltg edge weights!
  //TPM_Graph_vwgts& nprocs = *nprocs_ref;
  TPM_Mapping& mapping = *mapping_ref;
  MTRand mtrand;

  /* this uses threshold accepting (TA) which might be superior to
   * simulated annealing -- see Dueck and Scheuer: "Threshold accepting:
   * a general purpose optimization algorithm appearing superior to
   * simulated annealing" (Journal of Computational Physics,
   * 90(1):161-175, 1990) */
  // choose threshold value T
  int T=100;
  // threshold factor (0<TF<1) -- normalized to 100
  int TF=10;
  // max tests
  int Zmax=30;
  // start config == mapping
  TPM_Mapping C(mapping.size()); std::copy(mapping.begin(), mapping.end(), C.begin());
  // reference solution
  TPM_Mapping Cref(mapping.size()); std::copy(mapping.begin(), mapping.end(), Cref.begin());
  // cost of reference
  double Cref_cost;
  int Cref_dil;
  TPM_Get_cong(r, &ptg, &ptgc, &routes, &ltg, &ltgw, &Cref, &Cref_cost, &Cref_dil);

  // max. test time in seconds
  int stop=10;
  int ctr=0;

  double timeout = MPI_Wtime()+stop;
  while(MPI_Wtime() < timeout || ctr < 2) {
    for(int z=0; z<Zmax; z++) {
      // choose new config -- exchange two positions randomly
      TPM_Mapping Ctmp(mapping.size()); std::copy(mapping.begin(), mapping.end(), Ctmp.begin());
      int src = mtrand.randInt(mapping.size()-1);
      int tgt = mtrand.randInt(mapping.size()-1);
      int x=Ctmp[src];
      Ctmp[src] = Ctmp[tgt];
      Ctmp[tgt] = x;
      // get costs of new config
      double cost;
      int dil;
      TPM_Get_cong(r, &ptg, &ptgc, &routes, &ltg, &ltgw, &Ctmp, &cost, &dil);
      if(cost < Cref_cost) {
        std::copy(Ctmp.begin(), Ctmp.end(), Cref.begin());
        Cref_cost = cost;
        Cref_dil = dil;
      }
      if(cost - std::min(cost, Cref_cost) > -T) std::copy(Ctmp.begin(), Ctmp.end(), C.begin());
    }
    T = T*TF/100;
    ctr++;
  }
  //INFO(if(!r) printf("[annealing: %i iterations took %f s]\n", ctr, MPI_Wtime()-timeout+stop));
  *iters = ctr;
  std::copy(Cref.begin(), Cref.end(), mapping.begin());
  *cong = Cref_cost;
  *dil = Cref_dil;
  return 0;
}
