#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <unistd.h>
#include <sys/stat.h>
#include <vector>
#include <string>
#include <queue>
#include <numeric>
#include <algorithm>
#include <map>
#include <limits>
#include <list>
#include "virt_graph_scal.hpp"

#ifndef LIBTOPOMAP_HPP
#define LIBTOPOMAP_HPP

//#define BGP

#define DBG5(stmt) //stmt // debug TPM_Benchmark
#define DBG4(stmt) //stmt // debug greedy mapping
#define DBG3(stmt) //stmt // debug get_cong
#define DBG2(stmt) stmt // debug startup and initial graph distribution
#define DBG1(stmt)  //stmt // debug SSSP
#define DBG(stmt)  stmt // debug parser
#define MEM(stmt) stmt // print memory consumption information for the graphs
#define INFO(stmt) stmt // print additional information about optimization

#define FAKE_MPI22

// rcm-specifics
#define RCM_PREFIX TPM_
#include "rcm.h"

// enable all sorts of (unnecessary) checks (for safety)
#define CHECKS 1

typedef std::vector<std::string> TPM_Nodenames;
typedef std::vector<std::vector<int> > TPM_Graph; // adjacency list graph representation
typedef std::vector<std::vector<int> > TPM_Graph_ewgts; // edge weights
typedef std::vector<std::vector<int64_t> > TPM_Graph_lewgts; // long edge weights
typedef std::vector<std::vector<double> > TPM_Graph_dewgts; // double edge weights
typedef std::vector<int> TPM_Graph_vwgts; // vertex weights
typedef std::vector<double> TPM_Graph_dists; // SSSP dists
typedef std::vector<int> TPM_Graph_preds; // SSSP predecessors
typedef std::vector<int> TPM_Mapping; // mapping (permutation)

// this is exported and NULL by default but can be set to a filename to
// read the fake mapping (rank - fake-name) from :-)
extern char *TPM_Fake_names_file; 
#include "routing.hpp"

typedef struct {
  double weight;
  int src; // in ltg
  int dst; // in ltg
  int srcmapped; // which vertex is src mapped to in ptg
} prioQedge_t;

class prioQedge_compare_func {
  public:
  bool operator()(prioQedge_t x, prioQedge_t y) {
    if(x.weight < y.weight) return true;
    return false;
  }
};

class dijk_compare_func {
  public:
  bool operator()(std::pair<double,int> x, std::pair<double,int> y) {
    if(x.first > y.first) return true;
    return false;
  }
};

class max_compare_func {
  public:
  bool operator()(std::pair<double,int> x, std::pair<double,int> y) {
    if(x.first < y.first) return true;
    return false;
  }
};

// in topoparser.cpp
int TPM_Read_topo(const char *topofile, TPM_Nodenames *names, TPM_Graph *g, TPM_Graph_ewgts *w);
int TPM_Fake_hostname(const char *fname, int rank, char *name, int namesize);

// in libtopomap.cpp
int TPM_Topomap(MPI_Comm distgr, const char *topofile, int max_mem, int *newrank);
int TPM_Map_comm(MPI_Comm topocomm, const char *topofile, MPI_Comm *outcomm);
int TPM_Translate_rank(MPI_Comm sourcecomm, MPI_Comm targetcomm, int rank);

// in bgbtopo.cpp
int TPM_Get_bgp_topo(int r, TPM_Graph *g, int *me);

// tools in libtopomap.cpp (should go somewhere else)
int TPM_Write_graph_comm(MPI_Comm grphcomm, const char *filename);
int TPM_Write_phystopo(MPI_Comm grphcomm, const char *filename, const char *topofile);
void TPM_Benchmark_graphtopo(MPI_Comm distgr, int newrank, int dsize, double *before, double *after);

// on-node mapping
int TPM_Node_mapping(TPM_Graph* ltg_ref, TPM_Graph_ewgts *ltgw_ref, TPM_Mapping* idx2rank_ref, MPI_Comm comm, int rank);

void TPM_Get_cong(int r, TPM_Graph *h_ref, TPM_Graph_ewgts *ptgc_ref, TPM_Routes *routes_ref, TPM_Graph *g_ref, TPM_Graph_ewgts *weights_ref, TPM_Mapping *map_ref, double *max_cong, int *max_dil, bool print=false, double *avg_cong=NULL, double *avg_dil=NULL, const char *outfile=NULL);

// heuristics
int TPM_Map_greedy(int r, TPM_Graph *ptg_ref, TPM_Graph_ewgts *ptgc_ref, TPM_Graph_vwgts *nprocs_ref, TPM_Graph *ltg_ref, TPM_Graph_ewgts *weights_ref, TPM_Mapping *mapping_ref, int start);
int TPM_Map_greedy_routing(int r, TPM_Graph *ptg_ref, TPM_Graph_ewgts *ptgc_ref, TPM_Routes *routes_ref, TPM_Graph_vwgts *nprocs_ref, TPM_Graph *ltg_ref, TPM_Graph_ewgts *weights_ref, TPM_Mapping *mapping_ref, int start);
int TPM_Map_annealing(int r, TPM_Graph *ptg_ref, TPM_Graph_ewgts *ptgc_ref, TPM_Routes *routes_ref, TPM_Graph_vwgts *nprocs_ref, TPM_Graph *ltg_ref, TPM_Graph_ewgts *weights_ref, TPM_Mapping *mapping_ref, double *cong, int *dil, int *iters);
int TPM_Map_rcm(int r, TPM_Graph *ptg_ref, TPM_Graph_vwgts *nprocs_ref, TPM_Graph *ltg_ref, TPM_Mapping *mapping_ref, int start);
int TPM_Map_recursive(int r, TPM_Graph *ptg_ref, TPM_Graph_ewgts *ptgc_ref, TPM_Graph_vwgts *nprocs_ref, TPM_Graph *ltg_ref, TPM_Graph_ewgts *weights_ref, TPM_Mapping *mapping_ref, TPM_Mapping ltg_map /* indices in logical topology */, TPM_Mapping *ptg_map_ref /* map from passed ptg (subgraph) into global ptg */);
int TPM_Map_recursive_routing(int r, TPM_Graph *ptg_ref, TPM_Graph_ewgts *ptgc_ref, TPM_Graph_vwgts *nprocs_ref, TPM_Graph *ltg_ref, TPM_Graph_ewgts *weights_ref, TPM_Mapping *mapping_ref, TPM_Mapping ltg_map /* indices in logical topology */, TPM_Mapping *ptg_map_ref /* map from passed ptg (subgraph) into global ptg */, TPM_Routes *routes_ref);
int TPM_Map_scotch(int r, TPM_Graph *ptg_ref, TPM_Graph_ewgts *ptgc_ref, TPM_Graph_vwgts *nprocs_ref, TPM_Graph *ltg_ref, TPM_Graph_ewgts *weights_ref, TPM_Mapping *mapping_ref, int me, MPI_Comm topocomm);

#endif
