/* this file contains the reader for the routing information!
 * A very simple parser ...
 */
#include "libtopomap.hpp"
#include "parser.hpp"
#include "routing.hpp"

/**
 * read routes from a file -- first detect what the route type is and
 * then read them into the TPM_Routes structure.
 */
int TPM_RoutesRead(TPM_Routes *r, char *file) {
  // open topology file
  FILE *fd = fopen(file, "r");
  if (!fd) return 1; // fopen failed

  struct stat statbuf;
  stat(file, &statbuf);

  // read complete file in memory ...
  unsigned char *buf=(unsigned char*)malloc(statbuf.st_size);;
  int ret = fread(buf, 1, statbuf.st_size, fd);
  fclose(fd);

  // start the ugly hand-made parser (no, not using boost this time :-)
  int pos=0;
  //printf("reading routing file %s\n", file);

  // move to first whitespace
  while(buf[pos] != ' ') pos++;
  while(buf[pos] == ' ') pos++;

  /* compressed Torus XYZ routing ... this is the vanilla torus routing.
   * It is mostly implemented but does not work for XE-6 (see specialized routing)
   * because XE-6 is not a real torus (two nodes hang off a single Gemini ...
   * This can, however, be used for XT-5 networks ... just not tested yet */
  if(strstr((char*)&buf[pos], "COMP3DTORUSXYZ") == (char*)&buf[pos]) {
    //printf("found routing mode compressed XYZ\n");
    pos+=13;

    r->type = COMPXYZ;

    // scan to first number -- find number of hosts' XYZ coords
    while(! (buf[pos] >= '0' && buf[pos] <= '9') ) pos++;
    int n = read_int(&buf[pos], &pos);

    //printf("found n=%i\n", n);
    r->rank_to_xyz.resize(n); // init vector

    // scan to first number -- find maximum coordinates (Torus dimensions)
    while(! (buf[pos] >= '0' && buf[pos] <= '9') ) pos++;
    xyzcoords_t coords;
    coords.x = read_int(&buf[pos], &pos);
    while(buf[pos] == ' ') pos++; // fast-forward over whites
    coords.y = read_int(&buf[pos], &pos);
    while(buf[pos] == ' ') pos++; // fast-forward over whites
    coords.z = read_int(&buf[pos], &pos);
    r->xyzmaxdims = coords;
    //printf("maxdims: x=%i, y=%i, z=%i\n", coords.x, coords.y, coords.z);

    // read XYZ coordinates for each process
    for(int i=0; i<=n; i++) {
      int v = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      coords.x = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      coords.y = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      coords.z = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      while(buf[pos] != '\n') pos++; pos++; // next line

      r->rank_to_xyz[v] = coords;
      r->xyz_to_rank[coords] = v;
      //printf("%i x=%i, y=%i, z=%i\n", v, coords.x, coords.y, coords.z);
    }

    // scan to first number
    while(! (buf[pos] >= '0' && buf[pos] <= '9') ) pos++;
    int m = read_int(&buf[pos], &pos);
    while(buf[pos] != '\n') pos++; pos++; // next line
    //printf("found m=%i\n", m);

    // read irregular routes
    for(int i=0; i<m; i++) {
      int src = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      int tgt = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites
      //printf("%i->%i", src, tgt);

      std::pair<int,int> srcdst = std::make_pair(src, tgt);
      std::vector<int> *route = new(std::vector<int>);

      while(buf[pos] != '\n') { // next line
          int hop = read_int(&buf[pos], &pos);
          while(buf[pos] == ' ') pos++; // fast-forward over whites

          route->push_back(hop);
          //printf(" %i", hop);
      }
      pos++; // skip to next line
      //printf("\n");
      r->routes[srcdst] = route;
    }
  } else if(strstr((char*)&buf[pos], "COMPCRAYXE6") == (char*)&buf[pos]) {
    //printf("found routing mode Cray XE6 XYZ\n");
    pos+=11;

    r->type = COMPCRAYXE6;

    // scan to first number -- find number of hosts' XYZ coords
    while(! (buf[pos] >= '0' && buf[pos] <= '9') ) pos++;
    int n = read_int(&buf[pos], &pos);

    //printf("found n=%i\n", n);
    r->rank_to_xyz.resize(n); // init vector
    r->xe6gemini.resize(n); // init vector

    // scan to first number -- find maximum coordinates (Torus dimensions)
    while(! (buf[pos] >= '0' && buf[pos] <= '9') ) pos++;
    xyzcoords_t coords;
    coords.x = read_int(&buf[pos], &pos);
    while(buf[pos] == ' ') pos++; // fast-forward over whites
    coords.y = read_int(&buf[pos], &pos);
    while(buf[pos] == ' ') pos++; // fast-forward over whites
    coords.z = read_int(&buf[pos], &pos);
    r->xyzmaxdims = coords;
    //printf("maxdims: x=%i, y=%i, z=%i\n", coords.x, coords.y, coords.z);

    // read XYZ coordinates for each process
    for(int i=0; i<=n; i++) {
      int v = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      int gemini = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      coords.x = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      coords.y = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      coords.z = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      while(buf[pos] != '\n') pos++; pos++; // next line

      //printf("%i x=%i, y=%i, z=%i gemini: %i (size: %i)\n", v, coords.x, coords.y, coords.z, gemini, r->xe6gemini.size());
      r->xe6gemini[v] = gemini;
      r->rank_to_xyz[v] = coords;
      r->xyz_to_rank[coords] = gemini;
    }

    // scan to first number
    while(! (buf[pos] >= '0' && buf[pos] <= '9') ) pos++;
    int m = read_int(&buf[pos], &pos);
    while(buf[pos] != '\n') pos++; pos++; // next line
    //printf("found m=%i\n", m);

    // read irregular routes
    for(int i=0; i<m; i++) {
      int src = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites

      int tgt = read_int(&buf[pos], &pos);
      while(buf[pos] == ' ') pos++; // fast-forward over whites
      //printf("%i->%i", src, tgt);

      std::pair<int,int> srcdst = std::make_pair(src, tgt);
      std::vector<int> *route = new(std::vector<int>);

      while(buf[pos] != '\n') { // next line
        int hop = read_int(&buf[pos], &pos);
        while(buf[pos] == ' ') pos++; // fast-forward over whites

        route->push_back(hop);
        //printf(" %i", hop);
      }
      pos++; // skip to next line
      //printf("\n");
      r->routes[srcdst] = route;
    }
    //printf("done reading routes\n");
  } else if(strstr((char*)&buf[pos], "ROUTES") == (char*)&buf[pos]) {
      //printf("found routing mode uncompressed\n");
      pos+=5;

      r->type = UNCOMPRESSED;

      // scan to first number -- find number of hosts' XYZ coords
      while(! (buf[pos] >= '0' && buf[pos] <= '9') ) pos++;
      int n = read_int(&buf[pos], &pos);
      while(buf[pos] != '\n') pos++; pos++; // next line

      //printf("reading %i routes\n", n);

      // read irregular routes
      for(int i=0; i<n; i++) {
        int src = read_int(&buf[pos], &pos);
        while(buf[pos] == ' ') pos++; // fast-forward over whites

        int tgt = read_int(&buf[pos], &pos);
        while(buf[pos] == ' ') pos++; // fast-forward over whites
        //printf("%i->%i", src, tgt);

        std::pair<int,int> srcdst = std::make_pair(src, tgt);
        std::vector<int> *route = new(std::vector<int>);

        while(buf[pos] != '\n') { // next line
          int hop = read_int(&buf[pos], &pos);
          while(buf[pos] == ' ') pos++; // fast-forward over whites

          route->push_back(hop);
          //printf(" %i", hop);
        }
        pos++; // skip to next line
        //printf("\n");
        r->routes[srcdst] = route;
      }

  } else {
    printf("routing file not recognized!\n");
  }
  free(buf);
  return 0;
}
