#ifndef SSTMAC_SOFTWARE_PROCESS_GRAPHVIZ_H
#define SSTMAC_SOFTWARE_PROCESS_GRAPHVIZ_H

#define GraphVizAppendBacktrace(x) GraphVizIncrementStack __graphviz_tmp_variable__(x)
#define GraphVizDoNothing(x) int __graphviz_tmp_variable__

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/stats/stat_logger.h>
#include <sstmac/software/process/thread.h>
#include <sstmac/common/config.h>

namespace sstmac {
    namespace sw {


    class GraphVizIncrementStack {
        public:
            GraphVizIncrementStack(const char* fxn);

            ~GraphVizIncrementStack();
    };

    class GraphViz :
        public stat_logger
    {
        public:
            typedef boost::intrusive_ptr<GraphViz> ptr;

            std::string to_string() const
            {
                return "grahpviz";
            }

            static ptr
            construct();

            void simulation_finished(timestamp end);

            void** allocate_trace();

            void count_trace(long count, const sw::thread::ptr& thr);

        private:
            GraphViz();

            typedef std::pair<long, long long> graphviz_call;
            class Trace :
                public ptr_type
            {

                private:

                    std::map<void*, graphviz_call> calls_;

                    long long self_;

                    void* fxn_;

                    GraphViz::ptr parent_;

                    Trace(const GraphViz::ptr& parent, void* fxn);

                public:
                    typedef boost::intrusive_ptr<Trace> ptr;

                    static ptr construct(const GraphViz::ptr& parent, void* fxn);

                    std::string to_string() const
                    {
                        return "graphviz trace";
                    }

                    std::string
                    summary() const;

                    void* fxn() const;

                    void add_call(void* fxn, int ncalls, long count);

                    void add_self(long count);

            };

            void add_call(int ncalls, long count, void* fxn, void* callfxn);

            std::map<void*, Trace::ptr> traces_;

            std::map<void*, std::string> ptr_to_fxn_;

            std::map<std::string, void*> fxn_to_ptr_;

            friend class Trace;

            Trace::ptr get_trace(void* fxn);


    };

    }
}


#endif // GRAPHVIZ_H
