#ifndef sstmac_software_process_PMI_H
#define sstmac_software_process_PMI_H

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/nodeaddress.h>
#include <boost/unordered_map.hpp>
#include <sstmac/software/process/softwareid.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/network/interconnect/interconnect.h>

namespace sstmac {
    namespace sw {

class process_manager :
    virtual public ptr_type
{
    private:
        typedef boost::unordered_map<int, nodeaddress::ptr> proc_to_node_map;

        typedef boost::unordered_map<int, proc_to_node_map> app_to_proc_to_node_map;

        typedef boost::unordered_map<nodeaddress::ptr, int> node_to_proc_map;

        typedef boost::unordered_map<int, node_to_proc_map> app_to_node_to_proc_map;

        static app_to_proc_to_node_map node_map_;

        static app_to_node_to_proc_map proc_map_;

        nodeaddress::ptr my_addr_;

        hw::topology::ptr top_;

        hw::interconnect::ptr interconn_;

        softwareid sid_;

    public:
        virtual void init(const softwareid& sid);

        virtual void init_os(const OS_API::ptr& os);

        hw::topology::ptr get_topology() const {
            return top_;
        }

        hw::interconnect::ptr get_interconnect() const {
            return interconn_;
        }

        int get_partner(const nodeaddress::ptr& addr) const;

        nodeaddress::ptr my_addr() const {
            return my_addr_;
        }

};

    }
}



#endif // PMI_H
