/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/process/user_app.h>

namespace sstmac
{
  namespace sw
  {

    class user_app_cxx :
        public user_app
    {
    public:
      typedef boost::intrusive_ptr<user_app_cxx> ptr;

      typedef boost::intrusive_ptr<const user_app_cxx> const_ptr;

    protected:
      /** Constructor - pass along arguments to parent mpi class */
      user_app_cxx(sstmac::sw::softwareid id)
          : user_app(id)
      {
      }

      user_app_cxx(const sim_parameters::ptr& params) : user_app(params) {}

      static user_app_cxx::ptr
      construct(sstmac::sw::softwareid id) {
          return new user_app_cxx(id);
      }

    public:
      /** Destructor - do nothing */
      ~user_app_cxx() throw () {}

      static user_app_cxx::ptr
      construct(const sim_parameters::ptr& params)
      {
          return new user_app_cxx(params);
      }

      sstmac::sw::app::ptr
      clone(sstmac::sw::softwareid newid) {
          return construct(newid);
      }

      std::string to_string() const {
        return "user_mpiapp_cxx";
      }

      void skeleton_main();

      void
      consume_params(const sstmac::sim_parameters::ptr &params);

    };

  } //end namespace sw
} //end namespace sstmac



