/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SERVICES_LAUNCH_ALLOCATION_CARTALLOCATION_H_INCLUDED
#define SSTMAC_SOFTWARE_SERVICES_LAUNCH_ALLOCATION_CARTALLOCATION_H_INCLUDED

#include <vector>

#include <sstmac/hardware/network/topology/hdtorus.h>
#include <sstmac/software/services/launch/allocation/allocationstrategy.h>

#include <sstmac/common/errors.h>

namespace sstmac
{
  namespace sw
  {

    class cartallocation : public allocationstrategy
    {
      sstmac::hw::hdtorus::ptr network_;
      sstmac::hw::regular_topology::ptr regtop_;
      std::vector<int> sizes_;
      std::vector<int> offsets_;

      cartallocation();

      void
      allocate_dim(int dim, std::vector<int>& vec, node_set& allocation);

    public:
      typedef boost::intrusive_ptr<cartallocation> ptr;

      virtual void init(const hw::topology::ptr &top);

      void init_factory_params(const sim_parameters::ptr &params);

      virtual
      ~cartallocation() throw () {}

      static ptr
      construct(sim_parameters::ptr params);
      
      void
      allocate(int nnode, node_set &allocation);

      virtual void
      set_topology(const sstmac::hw::topology::ptr& top);

    };

  }
}

#endif
