/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_ROUNDROBININDEXING_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_ROUNDROBININDEXING_H_INCLUDED

#include <sstmac/software/services/launch/indexing/indexstrategy.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/common/sim_parameters.h>
namespace sstmac
{
  namespace sw
  {

    /**
     * An index strategy that allocates indices using a round robin.
     */
    class roundrobinindexing : public indexstrategy
    {
    protected:
      roundrobinindexing();

    public:
      typedef boost::intrusive_ptr<roundrobinindexing> ptr;

      static roundrobinindexing::ptr
      construct(sim_parameters::ptr params);

      virtual
      ~roundrobinindexing() throw ();

      /// Perform round robin allocation.
      /// This method does not test the uniqueness of input nodes.
      /// \param nodes is the list of unique nodes to be used for the allocation
      /// \param ppn is the nominal number of processes allocated on each node.
      /// \param result is the resulting vector of length nodes (size nproc)
      /// \param nproc the total number of processes to allocate
      /// \throw valueerror if nodes.empty()
      /// \throw valueerror if ppn <= 0
      virtual void
      allocate(const appid& aid, const node_set &nodes, int ppn, std::vector<nodeaddress::ptr> &result, int nproc);

    };

  }
} // end of namespace sstmac.

#endif
