/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_instantlaunch_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_instantlaunch_H_INCLUDED

#include <sstmac/software/process/taskid.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/software/services/launch/launcher.h>
#include <sstmac/common/nocopy.h>
#include <sstmac/software/services/launch/launchinfo.h>

#include <sstmac/software/process/app.h>

#include <sstmac/software/ami/environment.h>
#include <sstmac/common/debug.h>
#include <sstmac/software/services/launch/messages/launch_message.h>

#include <sstmac/software/services/launch/allocation/allocationstrategy.h>
#include <sstmac/software/services/launch/indexing/indexstrategy.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * A launcher that does not model the time it takes to
     * send the application code or request to the nodes
     * across the network, the app just instantly comes up
     * at a given time
     */
    class instantlaunch :
            public launcher,
            virtual public nocopy
    {
    public:
      typedef boost::intrusive_ptr<instantlaunch> ptr;

    protected:

      instantlaunch(const sim_parameters::ptr& params);

      launch_message::ptr current_;
      key::ptr block_;
        
        enum APP_STATE {WAITING, LAUNCHED, FINISHED, NOT_RUNNING_HERE };
        
        
        std::map<appid, APP_STATE> states_;


    public:

      virtual
      ~instantlaunch() throw () { }

      static instantlaunch::ptr
      construct(const sim_parameters::ptr& params);

      virtual std::string
      to_string() const
      {
        return "instantlaunch";
      }

      virtual void
      handle(const sst_message::ptr &msg);

      virtual void
      start();

      virtual service::ptr
      clone();


        virtual void
        apps_failed();
        
        virtual void 
        restart_app(launchinfo::ptr linfo);
        

      static void clear_statics();


    };

  }
} // end of namespace sstmac

#endif
