/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_MESSAGES_LAUNCH_MESSAGE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_MESSAGES_LAUNCH_MESSAGE_H_INCLUDED

#include <sstmac/hardware/network/network_message.h>
#include <sstmac/software/services/launch/launchinfo.h>

namespace sstmac
{
  namespace sw
  {

    class launch_message :
        public hw::network_message,
        public library_message,
        public timed_message
    {

    public:

      typedef boost::intrusive_ptr<launch_message> ptr;

      enum LAUNCHTYPE
      {
        ARRIVE, START, COMPLETE, KILL, RESTART
      };

      static ptr
      construct(launchinfo::ptr &i, LAUNCHTYPE t)
      {
        return ptr(new launch_message(i, t));
      }

      /**
       * Stringifier
       * @return String description
       */
      virtual std::string
      to_string() const
      {
        return "launch_message";
      }

      launchinfo::ptr
      get_info()
      {
        return info_;
      }

      virtual long
      get_byte_length() const
      {
        return 16;
      }

      LAUNCHTYPE
      get_launch_type() const
      {
        return launchtype_;
      }

      virtual const sw::taskid&
      source_task() const
      {
        return srcdst_;
      }

      virtual const sw::taskid&
      dest_task() const
      {
        return srcdst_;
      }

    protected:
      launch_message(launchinfo::ptr &i, LAUNCHTYPE t) :
        sst_message(sst_message::LAUNCH), 
        library_message("launcher"),
        timed_message(i->start_),
        info_(i), launchtype_(t)
      {
        needs_ack_ = false;
        srcdst_ = taskid(taskid::unbound_task);
      }

      launchinfo::ptr info_;
      LAUNCHTYPE launchtype_;

      taskid srcdst_;
    };

  }
}

#endif
