/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_MESSAGES_RESTART_MESSAGE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_MESSAGES_RESTART_MESSAGE_H_INCLUDED

#include <sstmac/software/services/launch/messages/launch_message.h>

#include <sstmac/software/services/launch/launchinfo.h>

namespace sstmac
{
  namespace sw
  {

    class restart_message : public launch_message
    {

    public:
      typedef boost::intrusive_ptr<restart_message> ptr;


      static ptr
      construct(launchinfo::ptr &i)
      {
        return ptr(new restart_message(i));
      }

      /**
       * Stringifier
       * @return String description
       */
      virtual std::string
      to_string() const
      {
        return "restart_message";
      }

      bool is_restart_wait_;
      bool is_restart_node_;
      std::list<app::ptr> newapps_;

    protected:
      restart_message(launchinfo::ptr &i) :
        sst_message(LAUNCH),
        launch_message(i, launch_message::RESTART), 
        is_restart_wait_(false), is_restart_node_(false)
      {

      }


    };

  }
}

#endif
