/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SERVICES_SERVICE_H_INCLUDED
#define SSTMAC_SOFTWARE_SERVICES_SERVICE_H_INCLUDED

#include <sstmac/software/process/waiter.h>
#include <sstmac/software/libraries/library.h>
#include <sstmac/common/factories/factory.h>

namespace sstmac
{
    namespace hw {
        class node;
        typedef boost::intrusive_ptr<node> nodePtr;
    }

  namespace sw
  {

    class operatingsystem;
    class service : public library, public eventhandler
    {

    protected:
      service();

      virtual
      ~service();

    public:
      typedef boost::intrusive_ptr<service> ptr;

      virtual std::string
      to_string() const
      {
        return "service";
      }

      virtual void
      start()
      {

      }

      virtual void
      finish()
      {
      }

      virtual void 
      unregister_all_libs();

      virtual service::ptr
      clone() = 0;

      virtual void
      init_full_os(const boost::intrusive_ptr<operatingsystem>& o);

      virtual void
      incoming_message(const sst_message::ptr &msg)
      {
        handle(msg);
      }

    protected:

      boost::intrusive_ptr<operatingsystem> fullos_;

      bool
      is_app_init(appid aid);

    };

    class service_factory :
        public SSTFactory<service>
    {

        public:
            static void
            start_services(const hw::nodePtr& node, const sim_parameters::ptr& params);

    };

  }
} //end of namespace sstmac

#endif
