#include <sstmac/backends/native/manager.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac {
    namespace sw {

    sstmac_register_app(mpi_printnodes);

    class node_name_logger
        : public stat_logger
    {
        private:
            long max_tid_;

            boost::unordered_map<long, std::string> node_names_;

        public:
            typedef boost::intrusive_ptr<node_name_logger> ptr;

            node_name_logger()
                : stat_logger("<stats> nodename", "nodes.dat"),
                  max_tid_(0)
            {
            }

            virtual ~node_name_logger(){}

            static ptr construct(const sim_parameters::ptr& params){
                return new node_name_logger;
            }

            void add_node(long tid, const std::string& name)
            {
                node_names_[tid] = name;
                max_tid_ = std::max(max_tid_, tid);
            }

            void simulation_finished(timestamp end)
            {
                for (long i=0; i <= max_tid_; ++i){
                    std::cout << sstprintf("Rank %5ld: ", i) << node_names_[i] << std::endl;
                }
            }

            virtual std::string to_string() const { return "node name logger"; }
    };

    static node_name_logger::ptr nodelog = 0;

    int mpi_printnodes_main(int argc, char** argv)
    {
        if (!nodelog){
            nodelog = node_name_logger::construct(sstmac_env::params);
            operatingsystem::get_current_os()->get_eventmanager()->register_stat(nodelog);
            logger::register_logger(nodelog.get());
        }

        hw::interconnect::ptr interconn = native::manager::global->get_interconnect();
        nodeaddress::ptr addr = operatingsystem::get_current_node();

        nodelog->add_node(operatingsystem::get_current_tid().id, interconn->label(addr->unique_id()));

        return 0;
    }

    }
}


