/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/test/mpicongestion.h>
#include <sstmac/software/api/mpi/mpiapi.h>
#include <sstmac/sstmacro.h>

#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>
#include <cstring>
#include <stdio.h>

namespace sstmac
{
  namespace sw
  {

    SSTRegister("MPI_congestion | mpi_congestion", app, mpicongestion);
// Pointer types.

//
// Hello.
//
    mpicongestion::ptr
    mpicongestion::construct(softwareid id)
    {
      mpicongestion::ptr obj(new mpicongestion(id));
      //  obj->set_self(obj);
      return obj;
    }

//
// Goodbye.
//
    mpicongestion::~mpicongestion() throw ()
    {
    }

//
// Get a copy.
//
    app::ptr
    mpicongestion::clone(softwareid id)
    {
      return mpicongestion::construct(id);
    }

    void
    mpicongestion::consume_params(const sim_parameters::ptr &params)
    {

      iterations_ = params->get_int_param("mpicongestion_iterations");
      window_ = params_->get_int_param("mpicongestion_window");
      count_ = params->get_int_param("mpicongestion_count");

    }

//
// Go.
//
    void
    mpicongestion::skeleton_main()
    {
      if (mpi() == 0)
        {
          throw nullerror("mpicongestion::run:  mpiapi pointer is null.");
        }
      SSTMAC_DEBUG << "mpicongestion[" << id_.to_string()
            << "]: mpi initializing... \n";

      MPI_Init(NULL, NULL);

      int rank, size;
      MPI_Comm_rank(MPI_COMM_WORLD, &rank);
      MPI_Comm_size(MPI_COMM_WORLD, &size);

      SSTMAC_DEBUG << "mpicongestion[" << rank
            << "]: mpi initialized, executing " << iterations_
            << " iterations \n";


      int tag = 0;
      if (rank == 0){ //recver
          int nreqs = window_ * (size - 1);
          MPI_Request* req = new MPI_Request[nreqs];
          for (int i=0; i < iterations_; ++i){
            MPI_Request* reqptr = req;
            for (int w=0; w < window_; ++w){
                for (int src=1; src < size; ++src, ++reqptr){
                    MPI_Irecv(MPI_PAYLOAD_IGNORE, count_, MPI_INT, src, tag, MPI_COMM_WORLD, reqptr);
                }
            }
            MPI_Waitall(nreqs, req, MPI_STATUSES_IGNORE);
        }
      }
      else {
          int nreqs = window_;
          MPI_Request* req = new MPI_Request[nreqs];
          int dst = 0;
          for (int i=0; i < iterations_; ++i){
              MPI_Request* reqptr = req;
              for (int w=0; w < window_; ++w, ++reqptr){
                  MPI_Isend(MPI_PAYLOAD_IGNORE, count_, MPI_INT, dst, tag, MPI_COMM_WORLD, reqptr);
              }
              MPI_Waitall(nreqs, req, MPI_STATUSES_IGNORE);
          }
      }
      MPI_Finalize();
      SSTMAC_DEBUG << "mpiprotocol[" << rank
            << "]: finalized\n";
    }

  }
} // end of namespace sstmac.
