/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/test/test_compute.h>
#include <sstmac/common/sstmac-config.h>

extern "C" {
  void test_compute_c(double time, int blocks);
}

namespace sstmac { namespace sw
{
  SSTRegister("test_compute", app, test_compute);


  test_compute::test_compute(softwareid id) :
    app(id)
  {


  }

  // Build a new minimd object.
  //
  test_compute::ptr
  test_compute::construct(softwareid id)
  {
    test_compute::ptr retval(new test_compute(id));
    // retval->set_self(retval);
    return retval;
  }

  void
  test_compute::consume_params(const sim_parameters::ptr &params)
  {

  }

  //
  // Get a copy of this object.
  //
  app::ptr
  test_compute::clone(softwareid id)
  {
    return construct(id);
  }

  test_compute::~test_compute()
  {

  }

  void
  test_compute::skeleton_main()
  {
    SSTMAC_DEBUG << "t=" << now() << ": Hello World, i'm computer number "
                 << id_.task_.id << ", about to begin computing.\n";

    double ctime = 0.1234;
    int cblocks = 1024;

    // test the C++ app compute mechanism
    timestamp tstart( now() );
    compute( timestamp(ctime) );
    compute_block_read(cblocks);
    timestamp tstop( now() );
    std::cout << "C++ compute test consumed " << tstop - tstart << "\n";
    
    // test the C compute interface
    tstart = timestamp( now() );
    test_compute_c(ctime,cblocks);
    tstop = timestamp( now() );
    std::cout << "C   compute test consumed " << tstop - tstart << "\n";
  }

} } //end of namespace sstmac
