#include <sstmac/software/process/app.h>

namespace sstmac
{
  namespace sw
  {

    class test_thread : public thread
    {
        private:
            int threadnum_;

        protected:
            test_thread(int threadnum)
                : thread(softwareid()), threadnum_(threadnum)
            {
            }

        public:
            typedef boost::intrusive_ptr<test_thread> ptr;

            static ptr
            construct(int threadnum)
            {
                return new test_thread(threadnum);
            }

            void run()
            {
                std::cout << "Running thread " << threadnum_ << "\n";
            }
    };

    class test_os : public app
    {
    protected:
        int nthread_;

    public:
      typedef boost::intrusive_ptr<test_os> ptr;

      test_os(softwareid id)
          : app(id)
      {
      }

      test_os(const sim_parameters::ptr& params) : app(params) {}

      virtual ~test_os()
      {
      }

      virtual void
      skeleton_main()
      {
          for (int i=0; i < nthread_; ++i){
              thread::ptr thr = test_thread::construct(i);
              this->get_os()->start_thread(thr);
          }
      }


      static ptr
      construct(const sim_parameters::ptr& params)
      {
          return new test_os(params);
      }

      virtual app::ptr
      clone(softwareid id)
      {
          return construct(id);
      }

      virtual void
      consume_params(const sim_parameters::ptr &params)
      {
          nthread_ = params->get_int_param("test_os_nthread");
      }

      virtual std::string
      to_string() const
      {
        return "test_os";
      }

    protected:
      static test_os::ptr
      construct(softwareid id)
      {
          return new test_os(id);
      }

    };

    SSTRegister("sstmac_test_os", app, test_os,
                "test case for certain os features such as spawing threads");

  }
} //end of namespace sstmac
