#include <sstmac/software/process/app.h>
#include <sstmac/hardware/packettrain/trainstation.h>
#include <sstmac/hardware/packettrain/trainlink.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/hardware/network/network_message.h>

namespace sstmac
{
  namespace sw
  {

#if USE_DEBUG_TRAJECTORY
  using hw::trainlink;
  using hw::trainstation;
  using hw::traincongestionpoint;
  using hw::packettrain;
  using hw::switch_direction;
  using hw::up;
  using hw::down;
  using hw::left;
  using hw::right;

    class test_packet_train :
        public app
    {

    public:
      typedef boost::intrusive_ptr<test_packet_train> ptr;

      test_packet_train(softwareid id)
          : app(id)
      {
      }

      virtual ~test_packet_train()
      {
      }

      virtual void
      skeleton_main();

      static ptr
      construct(softwareid id)
      {
          return new test_packet_train(id);
      }

      virtual app::ptr
      clone(softwareid id)
      {
          return construct(id);
      }

      virtual void
      consume_params(const sim_parameters::ptr &params)
      {
      }

      virtual std::string
      to_string() const
      {
        return "test_os";
      }

    };

class trainswitch :
    public eventmodule
{

    private:
        trainlink::ptr links_[4];

    protected:
        trainswitch()
        {
        }

    public:
        typedef boost::intrusive_ptr<trainswitch> ptr;

        static ptr construct(){
            return new trainswitch();
        }

        void set_up_link(const trainlink::ptr& link){
            links_[up] = link;
        }

        void set_down_link(const trainlink::ptr& link){
            links_[down] = link;
        }

        void set_left_link(const trainlink::ptr& link){
            links_[left] = link;
        }

        void set_right_link(const trainlink::ptr& link){
            links_[right] = link;
        }

        void handle(const sst_message::ptr &msg);

};

class train_message :
    public hw::network_message
{
    private:
        long bytes_;
    public:
        train_message(long bytes)
            : bytes_(bytes)
        {

        }

        long get_byte_length() const {
            return bytes_;
        }
};

#define enumcase(x) case x: return #x;

static const char*
tostr(switch_direction dir){
    switch(dir){
    enumcase(up);
    enumcase(down);
    enumcase(right);
    enumcase(left);
    default:
        return "error";
    }
}

void
trainswitch::handle(const sst_message::ptr &msg)
{
    packettrain::ptr train = boost::dynamic_pointer_cast<packettrain>(msg);
    if (!train){
        sst_throw(illformederror, "trainswitch could not cast msg");
    }
    switch_direction dir = train->route(this);
    std::cout << sstprintf("Routing %s %s\n", tostr(dir), train->to_string().c_str());
    links_[dir]->send(train);
}

trainswitch::ptr A, B, C, E, F, G;
trainstation::ptr D,H;

static void set_trajectory_ABCD(const packettrain::ptr& train)
{
    train->add_step(A, right);
    train->add_step(B, right);
    train->add_step(C, right);
}

void
test_packet_train::skeleton_main()
{
    eventmanager::ptr man = operatingsystem::get_current_os()->get_eventmanager();
    donothinghandler::ptr handler = donothinghandler::construct();
    timestamp hop_latency(0); //cycles

    /**
                      H
                      | 6
      A ->(0) B ->(1) C ->(2) D

                      | 5

      E ->(3) F ->(4) G
    */

    A = trainswitch::construct(); A->set_eventmanager(man);
    B = trainswitch::construct(); B->set_eventmanager(man);
    C = trainswitch::construct(); C->set_eventmanager(man);
    E = trainswitch::construct(); E->set_eventmanager(man);
    F = trainswitch::construct(); F->set_eventmanager(man);
    G = trainswitch::construct(); G->set_eventmanager(man);

    D = trainstation::construct(handler); D->set_eventmanager(man);
    H = trainstation::construct(handler); H->set_eventmanager(man);

    int single_packet = 1;
    int double_packet = 2;

    int single_freq = 1000;
    int double_freq = 1000;

    std::string arbtype = "packet";
    std::string linktype = "network";

    trainlink::ptr AB = trainlink::construct(B, hop_latency, double_packet,
                                             single_freq, 1.0, false, false,
                                             arbtype, linktype); AB->set_eventmanager(man);
    trainlink::ptr BC = trainlink::construct(C, hop_latency, single_packet,
                                             single_freq, 1.0, false, false,
                                             arbtype, linktype); BC->set_eventmanager(man);
    trainlink::ptr CD = trainlink::construct(D, hop_latency, double_packet,
                                             double_freq, 1.0, false, false,
                                             arbtype, linktype); CD->set_eventmanager(man);
    trainlink::ptr EF = trainlink::construct(F, hop_latency, single_packet,
                                             double_freq, 1.0, false, false,
                                             arbtype, linktype); EF->set_eventmanager(man);
    trainlink::ptr FG = trainlink::construct(G, hop_latency, single_packet,
                                             single_freq, 1.0, false, false,
                                             arbtype, linktype); FG->set_eventmanager(man);

    trainlink::ptr GC = trainlink::construct(C, hop_latency, single_packet, single_freq, 1.0,
                                             false, false, arbtype, linktype); GC->set_eventmanager(man);

    trainlink::ptr CH = trainlink::construct(H, hop_latency, single_packet, single_freq, 1.0,
                                             false, false, arbtype, linktype); CH->set_eventmanager(man);

    A->set_right_link(AB);
    B->set_right_link(BC);
    C->set_right_link(CD);

    E->set_right_link(EF);
    F->set_right_link(FG);

    G->set_up_link(GC);
    C->set_up_link(CH);

#if 1
    // train from B to C
    packettrain::ptr tBCH = packettrain::construct(new train_message(1000));
    tBCH->add_step(B, right);  tBCH->add_step(C, up);

    packettrain::ptr shorty = packettrain::construct(new train_message(4));
    shorty->add_step(A, right); shorty->add_step(B, right); shorty->add_step(C, right);

    packettrain::ptr tABCD = packettrain::construct(new train_message(5000));
    tABCD->add_step(A, right); tABCD->add_step(B, right); tABCD->add_step(C, right);

    packettrain::ptr tCD = packettrain::construct(new train_message(2500));
    tCD->add_step(C, right);

    packettrain::ptr tEFGCD = packettrain::construct(new train_message(5000));
    tEFGCD->add_step(E, right); tEFGCD->add_step(F, right);
    tEFGCD->add_step(G, up); tEFGCD->add_step(C, right);

    packettrain::ptr tBCD = packettrain::construct(new train_message(1000));
    tBCD->add_step(B, right); tBCD->add_step(C, right); tBCD->add_step(D, right);

    A->handle(tABCD);
    //E->handle(tEFGCD);

    //sleep for 0.5
    sleep(timestamp(0.5));
    //A->handle(shorty);
    B->handle(tBCH);

    sleep(timestamp(0.5));
    //B->handle(tBCD);

    sleep(timestamp(3));
    //C->handle(tCD);
#endif


#if 0
    //create and send a bunch of really small packets
    train_message::ptr sp = new train_message(32);
    packettrain::ptr s1 = packettrain::construct(sp); set_trajectory_ABCD(s1);
    packettrain::ptr s2 = packettrain::construct(sp); set_trajectory_ABCD(s2);
    packettrain::ptr s3 = packettrain::construct(sp); set_trajectory_ABCD(s3);
    packettrain::ptr s4 = packettrain::construct(sp); set_trajectory_ABCD(s4);
    A->handle(s1);
    A->handle(s2);
    A->handle(s3);
    A->handle(s4);
#endif
}


    SSTRegisterApp("sstmac_test_packet_train", test_packet_train,
                   "test packet train interface");
#endif

  }
} //end of namespace sstmac
