#include <sstmac/common/test/test.h>
#include <sstmac/software/process/app.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/software/api/simpmsg/simpmsg.h>
#include <sstmac/software/api/simpmsg/sstmac_simpmsg_api.h>
#include <sstmac/hardware/network/topology/traffic/traffic.h>

using namespace sstmac;
using namespace sstmac::hw;
using namespace sstmac::sw;

namespace simpmsg {

sstmac_register_app(test_traffic_pattern)

    static long done = 0;

void run_test(
    traffic_pattern::type_t ty,
    long num_bytes
)
{
    std::vector<nodeaddress::ptr> node_partners;
    simpmsg_api::ptr simp = simp_api();
    simp->get_topology()->get_send_partners(
        ty,
        simp->my_addr(),
        node_partners);
    int num_partners = node_partners.size();
    std::vector<int> send_partners(num_partners, 0);
    for (int i=0; i < node_partners.size(); ++i){
        send_partners[i] = simp->get_partner(node_partners[i]);
    }

    node_partners.clear();
    std::vector<int> recv_partners(num_partners, 0);
    simp->get_topology()->get_recv_partners(
        ty,
        simp->my_addr(),
        node_partners);
    for (int i=0; i < num_partners; ++i){
        recv_partners[i] = simp->get_partner(node_partners[i]);
    }


    for (int i=0; i < num_partners; ++i){
        simple_message::ptr msg = new simple_message(num_bytes);
        simp_rdma_put(send_partners[i], msg);
    }

    int num_recved = 0;
    while (num_recved < num_partners)
    {
        simple_message::ptr msg = simp_poll();
        if (msg->event() == simple_message::rdma_put_ack)
        {
            //ignore
        }
        else if (msg->event() == simple_message::rdma_put_start)
        {
            ++num_recved;
        }
        else
        {
            sst_throw(illformederror,
                "got unexpected message %s",
                simple_message::tostr(msg->event()));
        }
    }
    done++;
    std::cout << "traffic(" << simp->my_addr() << "): done, count is " << done << "\n";
}

int
test_traffic_pattern_main(int argc, char** argv)
{
    simp_init();

    std::string pattern = sstmac_env::params->get_param("traffic_pattern");
    traffic_pattern::type_t ty;
    if (pattern == "NN" || pattern == "nearest_neighbor"){
        ty = traffic_pattern::nearest_neighbor;
    }
    else if (pattern == "BC" || pattern == "bit_complement"){
        ty = traffic_pattern::bit_complement;
    }
    else if (pattern == "TOR" || pattern == "tornado"){
        ty = traffic_pattern::tornado;
    }
    else {
        sst_throw(inputerror,
            "invalid traffic pattern %s",
            pattern.c_str());
    }

    long num_bytes_per_msg = sstmac_env::params->get_byte_length_param("message_length");

    run_test(ty, num_bytes_per_msg);



    simp_finalize();
}

}
