/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_THREADING_STACKALLOC_CHUNK_H_INCLUDED
#define SSTMAC_SOFTWARE_THREADING_STACKALLOC_CHUNK_H_INCLUDED


#include <sstmac/software/threading/stackalloc.h>
#include <algorithm>
#include <iostream>
#include <unistd.h>


namespace sstmac { namespace sw {
  /**
   * A chunk of allocated memory to be divided into fixed-size stacks.
   */
  class stackalloc::chunk{
    /// The base address of my memory region.
    char *addr_;
    /// The total size of my allocation.
    size_t size_;
    /// The target size of each open (unprotected) stack region.
    size_t stacksize_;
    /// Are we putting mprot pages between allocations or not?
    bool use_mprot_;
    /// Next stack (for get_next_stack).
    size_t next_stack_;

  public:
    /// Make a new chunk.
    chunk(size_t stacksize, size_t suggested_chunk_size, bool use_mprot);

    /// Goodbye.
    virtual ~chunk();

    /// Put a pointer to each open region onto the given output iterator.
    /// This chunk makes no effort to monitor allocated vs. unallocated stacks.
    //template <typename OutputIterator>
    //void get_stacks(OutputIterator result) {
    //  const size_t guard = (use_mprot_ ? sysconf(_SC_PAGESIZE) : 0);
    //  const size_t stride = stacksize_ + guard;
    //  // The memory region begins and ends with an mprotect page.
    //  for(size_t pos = guard; pos < size_; pos += stride) {
    //    //std::cerr << "Providing access to memory rooted at " << pos << "\n";
    //    result = addr_ + pos;
    //    ++result;
    //  }
    //}

    //void get_stacks2(std::vector<void*> &result) {
    //  const size_t guard = (use_mprot_ ? sysconf(_SC_PAGESIZE) : 0);
    //  const size_t stride = stacksize_ + guard;
    //  //std::cerr << "Resizing result buffer\n";
    //  result.reserve(result.size() + (size_ - guard) / stride);
    //  //std::cerr << "Result buffer resized\n";
    //  for(size_t pos = guard;  pos <= size_-stride; pos += stride) {
    //    //std::cerr << "Providing access to memory rooted at " << pos << "\n";
    //    result.push_back(addr_ + pos);
    //  }
    //}

    void* get_next_stack() {
        size_t unit = stacksize_ + (use_mprot_ ? sysconf(_SC_PAGESIZE) : 0);
        if(next_stack_+unit >= size_) {
         return 0;
        }
        else {
          //fprintf(stderr, "stackallock::chunk:  rv = %p+%llu = %p\n", addr_, next_stack_, (addr_+next_stack_));
          void* rv = addr_ + next_stack_;
          next_stack_ += unit;
          return rv;
        }
    }
  };

} } // end of namespace sstmac


#endif
