#include <sstmac/sstmacro.h>

using namespace sstmac;

enum TEST_MODE
{

  FEAT_C_ACCESS = 0,
  FEAT_C_ATOMICS = 1,
  FEAT_C_BARRIER = 2,
  FEAT_C_BROADCAST = 3,
  FEAT_C_COLLECTS = 4,
  FEAT_C_GET_GLOBALS = 5,
  FEAT_C_GET_SHMALLOC = 6,
  FEAT_C_GET = 7,
  FEAT_C_LOCK = 8,
  FEAT_C_PUT_GLOBALS = 9,
  FEAT_C_PUT_SHMALLOC = 10,
  FEAT_C_REDUCTION = 11,
  FEAT_C_SYNCH = 12,
  EXAMPLE_ADJACENT = 13,
  EXAMPLE_CPI = 14,
  EXAMPLE_HELLO = 15,
  EXAMPLE_PTP = 16,
  EXAMPLE_2DHEAT = 17,
  EXAMPLE_DAXPY = 18,
  EXAMPLE_HEAT_IMAGE = 19,
  EXAMPLE_MATRIX = 20,
  EXAMPLE_SPING = 21,
  FEAT_C_PUT = 22,
  TEST_MODE_END = 23

};

//-------- feature tests---------//
#include "feature_tests/C/test_shmem_accessible.cc"
#include "feature_tests/C/test_shmem_atomics.cc"
#include "feature_tests/C/test_shmem_barrier.cc"
#include "feature_tests/C/test_shmem_broadcast.cc"
#include "feature_tests/C/test_shmem_collects.cc"
#include "feature_tests/C/test_shmem_get_globals.cc"
#include "feature_tests/C/test_shmem_get_shmalloc.cc"
#include "feature_tests/C/test_shmem_get.cc"
#include "feature_tests/C/test_shmem_lock.cc"
#include "feature_tests/C/test_shmem_put_globals.cc"
#include "feature_tests/C/test_shmem_put_shmalloc.cc"
#include "feature_tests/C/test_shmem_put.cc"
#include "feature_tests/C/test_shmem_reduction.cc"
#include "feature_tests/C/test_shmem_synchronization.cc"

//----------- examples ----------------//
#include "examples/adjacent.h"
#include "examples/cpi.cc"
#include "examples/hello.cc"
#include "examples/ptp.cc"
#include "examples/2dheat.h"
#include "examples/shmem_daxpy.cc"
#include "examples/heat_image.h"
#include "examples/shmem_matrix.cc"
#include "examples/sping.cc"

int testmode_ = -1;
bool justprint = false;
bool printnumtests = false;

std::string
print_test_name(int testmode);

void
consume_parameters(const sstmac::sim_parameters::ptr &params)
{
  testmode_ = params->get_int_param("testsuite_testmode");
  justprint = params->get_optional_bool_param("testsuite_print", false);
  printnumtests = params->get_optional_bool_param("testsuite_numtests", false);
}

int USER_MAIN(int argc, char *argv[])
{
  if (justprint)
  {
    std::cout << "name: " << print_test_name(testmode_) << "\n";
    return 0;
  }
  else if (printnumtests)
  {
    std::cout << (TEST_MODE_END - 1) << "\n";
    return 0;
  }

  switch (testmode_)
  {

  case FEAT_C_ACCESS:
    feat_c_access::feat_c_access(argc, argv);
    break;

  case FEAT_C_ATOMICS:
    feat_c_atomics::feat_c_atomics(argc, argv);
    break;

  case FEAT_C_BARRIER:
    feat_c_barrier::feat_c_barrier(argc, argv);
    break;

  case FEAT_C_BROADCAST:
    feat_c_broadcast::feat_c_broadcast(argc, argv);
    break;

  case FEAT_C_COLLECTS:
    feat_c_collects::feat_c_collects(argc, argv);
    break;

  case FEAT_C_GET_GLOBALS:
    feat_c_get_globals::feat_c_get_globals(argc, argv);
    break;

  case FEAT_C_GET_SHMALLOC:
    feat_c_get_shmalloc::feat_c_get_shmalloc(argc, argv);
    break;

  case FEAT_C_GET:
    feat_c_get::feat_c_get(argc, argv);
    break;

  case FEAT_C_LOCK:
    feat_c_lock::feat_c_lock(argc, argv);
    break;

  case FEAT_C_PUT_GLOBALS:
    feat_c_put_globals::feat_c_put_globals(argc, argv);
    break;

  case FEAT_C_PUT_SHMALLOC:
    feat_c_put_shmalloc::feat_c_put_shmalloc(argc, argv);
    break;

  case FEAT_C_REDUCTION:
    feat_c_reduction::feat_c_reduction(argc, argv);
    break;

  case FEAT_C_SYNCH:
    feat_c_synch::feat_c_synch(argc, argv);
    break;

  case EXAMPLE_ADJACENT:
    example_adjacent(argc, argv);
    break;

  case EXAMPLE_CPI:
    example_cpi::example_cpi(argc, argv);
    break;

  case EXAMPLE_HELLO:
    example_hello::example_hello(argc, argv);
    break;

  case EXAMPLE_PTP:
    example_ptp::example_ptp(argc, argv);
    break;

  case EXAMPLE_2DHEAT:
    example_2dheat(argc, argv);
    break;

  case EXAMPLE_DAXPY:
    example_daxpy::example_daxpy(argc, argv);
    break;

  case EXAMPLE_HEAT_IMAGE:
    example_heat_image(argc, argv);
    break;

  case EXAMPLE_MATRIX:
    example_matrix::example_matrix(argc, argv);
    break;

  case EXAMPLE_SPING:
    example_sping::example_sping(argc, argv);
    break;

  case FEAT_C_PUT:
    feat_c_put::feat_c_put(argc, argv);
    break;

  default:
    sst_throw(ssterror, "testmpi: unknown test mode %d", testmode_);
    return 1;
  }
  return 0;
}

std::string
print_test_name(int testmode)
{
  switch (testmode)
  {
  case FEAT_C_ACCESS:
    return "FEAT_C_ACCESS";

  case FEAT_C_ATOMICS:
    return "FEAT_C_ATOMICS";

  case FEAT_C_BARRIER:
    return "FEAT_C_BARRIER";

  case FEAT_C_BROADCAST:
    return "FEAT_C_BROADCAST";

  case FEAT_C_COLLECTS:
    return "FEAT_C_COLLECTS";

  case FEAT_C_GET_GLOBALS:
    return "FEAT_C_GET_GLOBALS";

  case FEAT_C_GET_SHMALLOC:
    return "FEAT_C_GET_SHMALLOC";

  case FEAT_C_GET:
    return "FEAT_C_GET";

  case FEAT_C_LOCK:
    return "FEAT_C_LOCK";

  case FEAT_C_PUT_GLOBALS:
    return "FEAT_C_PUT_GLOBALS";

  case FEAT_C_PUT_SHMALLOC:
    return "FEAT_C_PUT_SHMALLOC";

  case FEAT_C_REDUCTION:
    return "FEAT_C_REDUCTION";

  case FEAT_C_SYNCH:
    return "FEAT_C_SYNCH";

  case EXAMPLE_ADJACENT:
    return "EXAMPLE_ADJACENT";

  case EXAMPLE_CPI:
    return "EXAMPLE_CPI";

  case EXAMPLE_HELLO:
    return "EXAMPLE_HELLO";

  case EXAMPLE_PTP:
    return "EXAMPLE_PTP";

  case EXAMPLE_2DHEAT:
    return "EXAMPLE_2DHEAT";

  case EXAMPLE_DAXPY:
    return "EXAMPLE_DAXPY";

  case EXAMPLE_HEAT_IMAGE:
    return "EXAMPLE_HEAT_IMAGE";

  case EXAMPLE_MATRIX:
    return "EXAMPLE_MATRIX";

  case EXAMPLE_SPING:
    return "EXAMPLE_SPING";

  case FEAT_C_PUT:
    return "FEAT_C_PUT";

  default:
    return "not a valid test mode";

  }
}

