#include <tests/unittests/util/util.h>
#include <sstmac/hardware/network/topology/dragonfly.h>


using namespace sstmac;
using namespace sstmac::hw;


void
test_dragonfly_v1(UnitTest& unit)
{
    topology::clear_rng();

    unittest_params params;
    sstmac_env::params = params;
    params["topology_geometry"] = "4 4 4";
    params["network_nodes_per_switch"] = "2";
    params["optical_link_weight"] = "2.0";
    params["topology_group_connections"] = "4";
    topology::ptr top = topology_factory::get_value("dragonfly", params);
    regular_topology::ptr dfly = boost::dynamic_pointer_cast<regular_topology>(top);
    assertTrue(unit, "dragonfly cast topology", dfly);

    topology::maptype switches;
    init_switches(switches, params, top);

    {
        coordinates coords = get_vector(3,0,1);
        long nid = dfly->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = dfly->get_node_id(get_vector(3,2,1,0));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        assertEqual(unit, "num productive ports", ports.size(), 1);

        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(1, 2));
    }

    {
        coordinates coords = get_vector(3,0,1);
        long nid = dfly->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = dfly->get_node_id(get_vector(0,2,1,0));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        assertEqual(unit, "num productive ports", ports.size(), 2);

        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(0, 0));
        if (ports.size() > 1)
            assertEqual(unit, "productive port 1", ports[1], router->convert_to_outport(1, 2));
    }

    {
        coordinates coords = get_vector(3,0,1);
        long nid = dfly->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = dfly->get_node_id(get_vector(0,2,0,0));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        assertEqual(unit, "num productive ports", ports.size(), 3);

        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(0, 0));
        if (ports.size() > 1)
            assertEqual(unit, "productive port 1", ports[1], router->convert_to_outport(1, 2));
        if (ports.size() > 2)
            assertEqual(unit, "productive port 2", ports[2], router->convert_to_outport(2, 0));
    }

    {
        coordinates c1 = get_vector(0,1,2);
        coordinates c2 = get_vector(1,0,2);
        int dist = dfly->minimal_distance(c1,c2);
        int correct = 2;
        assertEqual(unit, "min distance", dist, correct);
    }

    {
        coordinates c1 = get_vector(1,1,2);
        coordinates c2 = get_vector(1,0,2);
        int dist = dfly->minimal_distance(c1,c2);
        int correct = 1;
        assertEqual(unit, "min distance", dist, correct);
    }

    {
        coordinates c1 = get_vector(0,1,1);
        coordinates c2 = get_vector(1,0,2);
        int dist = dfly->minimal_distance(c1,c2);
        int correct = 3;
        assertEqual(unit, "min distance", dist, correct);
    }

}


void
test_dragonfly_v2(UnitTest& unit)
{
    topology::clear_rng();

    unittest_params params;
    sstmac_env::params = params;
    params["topology_geometry"] = "4 4 8";
    params["network_nodes_per_switch"] = "2";
    params["optical_link_weight"] = "2.0";
    params["topology_group_connections"] = "3";
    params["topology_seed"] = "14";
    topology::ptr top = topology_factory::get_value("dragonfly", params);
    regular_topology::ptr dfly = boost::dynamic_pointer_cast<regular_topology>(top);
    assertTrue(unit, "dragonfly cast topology", dfly);

    topology::maptype switches;
    init_switches(switches, params, top);

    {
        coordinates coords = get_vector(3,0,1);
        long nid = dfly->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = dfly->get_node_id(get_vector(3,2,1,0));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        assertEqual(unit, "num productive ports", ports.size(), 1);

        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(1, 2));
    }

    {
        coordinates coords = get_vector(3,0,1);
        long nid = dfly->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = dfly->get_node_id(get_vector(0,2,1,0));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        assertEqual(unit, "num productive ports", ports.size(), 2);

        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(0, 0));
        if (ports.size() > 1)
            assertEqual(unit, "productive port 1", ports[1], router->convert_to_outport(1, 2));
    }

    {
        coordinates coords = get_vector(3,0,1);
        long nid = dfly->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = dfly->get_node_id(get_vector(0,2,0,0));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        assertEqual(unit, "num productive ports", ports.size(), 3);

        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(0, 0));
        if (ports.size() > 1)
            assertEqual(unit, "productive port 1", ports[1], router->convert_to_outport(1, 2));
        if (ports.size() > 2)
            assertEqual(unit, "productive port 2", ports[2], router->convert_to_outport(0, 0));
    }

    {
        coordinates coords = get_vector(3,0,1);
        long nid = dfly->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = dfly->get_node_id(get_vector(0,2,5,0));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        assertEqual(unit, "num productive ports", ports.size(), 3);

        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(0, 0));
        if (ports.size() > 1)
            assertEqual(unit, "productive port 1", ports[1], router->convert_to_outport(1, 2));
        if (ports.size() > 2)
            assertEqual(unit, "productive port 2", ports[2], router->convert_to_outport(2, 5));
    }

    {
        coordinates c1 = get_vector(0,1,2);
        coordinates c2 = get_vector(1,0,2);
        int dist = dfly->minimal_distance(c1,c2);
        int correct = 2;
        assertEqual(unit, "min distance", dist, correct);
    }


    {
        coordinates c1 = get_vector(0,1,2);
        coordinates c2 = get_vector(0,1,1);
        int dist = dfly->minimal_distance(c1,c2);
        //group hop plus at least two y hops
        assertTrue(unit, "min distance", dist >= 3);
    }
}
