#include "test_topology.h"
#include <sstmac/hardware/network/topology/butterfly.h>


using namespace sstmac;
using namespace sstmac::hw;



void test_butterfly(UnitTest& unit)
{
    unittest_params params;
    sstmac_env::params = params;
    params["topology_geometry"] = "4 3";
    //params["network_nodes_per_switch"] = "2";
    topology::ptr top = topology_factory::get_value("butterfly", params);
    regular_topology::ptr bfly = boost::dynamic_pointer_cast<regular_topology>(top);
    assertTrue(unit, "butterfly cast topology", bfly);

    topology::maptype switches;
    init_switches(switches, params, top);

    {
        long nids[] = {0, 17, 45, 46};
        for (int i=0; i < 4; ++i){
            long nid = nids[i];
            //make sure the functions work back and forth
            coordinates coords = bfly->get_switch_coords(nid);
            long test_id = bfly->get_switch_id(coords);
            coordinates test_coords = bfly->get_switch_coords(test_id);
            assertEqual(unit, "bfly switch id", test_id, nid);
            assertEqual(unit, "bfly coords", test_coords, coords);
        }
    }

    {
    networkswitch::ptr sw = switches[naddr(7)];
    int dim,dir;
    bfly->minimal_route_to_node(naddr(7), naddr(4), dim, dir);
    assertEqual(unit, "route butterfly dim", dim, butterfly::up_dimension);
    assertEqual(unit, "route butterfly dir", dir, 0);
    assert_dim_dir(unit, "butterfly dim/dir",
                   sw, dim, dir, 19);

    bfly->minimal_route_to_node(naddr(7), naddr(42), dim, dir);
    assertEqual(unit, "route butterfly dim", dim, butterfly::up_dimension);
    assertEqual(unit, "route butterfly dir", dir, 2);
    assert_dim_dir(unit, "butterfly dim/dir",
                   sw, dim, dir, 27);

    bfly->minimal_route_to_node(naddr(7), naddr(15), dim, dir);
    assertEqual(unit, "route butterfly dim", dim, butterfly::up_dimension);
    assertEqual(unit, "route butterfly dir", dir, 0);
    assert_dim_dir(unit, "butterfly dim/dir",
                   sw, dim, dir, 19);
    }


}
