#include "test_topology.h"

using namespace sstmac;
using namespace sstmac::hw;



void test_crossbar(UnitTest& unit)
{
    unittest_params params;
    sstmac_env::params = params;
    params["topology_geometry"] = "10";
    params["network_nodes_per_switch"] = "3";
    topology::ptr top = topology_factory::get_value("crossbar", params);
    regular_topology::ptr xbar = boost::dynamic_pointer_cast<regular_topology>(top);
    assertTrue(unit, "crossbar cast topology", xbar);

    topology::maptype switches;
    init_switches(switches, params, top);

    {
        long nids[] = {0, 3, 8, 14};
        for (int i=0; i < 4; ++i){
            long nid = nids[i];
            //make sure the functions work back and forth
            coordinates coords = xbar->get_switch_coords(nid);
            long test_id = xbar->get_switch_id(coords);
            coordinates test_coords = xbar->get_switch_coords(test_id);
            assertEqual(unit, "xbar switch id", test_id, nid);
            assertEqual(unit, "xbar coords", test_coords, coords);
        }
    }



    {
    networkswitch::ptr sw = switches[naddr(7)];
    int dim,dir;
    xbar->minimal_route_to_node(naddr(7), naddr(4), dim, dir);
    assertEqual(unit, "route crossbar", dim, 0);
    assert_dim_dir(unit, "crossbar dim/dir",
                   sw, dim, dir, 1);

    xbar->minimal_route_to_node(naddr(7), naddr(23), dim, dir);
    assertEqual(unit, "route crossbar", dim, (int) topology::EJECT);

    xbar->minimal_route_to_node(naddr(7), naddr(15), dim, dir);
    assertEqual(unit, "route crossbar", dim, 0);
    assert_dim_dir(unit, "crossbar dim/dir",
                   sw, dim, dir, 5);
    }

}
