#include "test_topology.h"
#include <sstmac/hardware/network/topology/flattenedbutterfly.h>


using namespace sstmac;
using namespace sstmac::hw;


void test_fbfly(UnitTest& unit)
{
    unittest_params params;
    sstmac_env::params = params;
    params["topology_geometry"] = "4 3";
    //params["network_nodes_per_switch"] = "2";
    topology::ptr top = topology_factory::get_value("fbfly", params);
    regular_topology::ptr bfly = boost::dynamic_pointer_cast<regular_topology>(top);
    assertTrue(unit, "fbfly cast topology", bfly);

    topology::maptype switches;
    init_switches(switches, params, top);

    {
        long nids[] = {0, 3, 8, 14};
        for (int i=0; i < 4; ++i){
            long nid = nids[i];
            //make sure the functions work back and forth
            coordinates coords = bfly->get_switch_coords(nid);
            long test_id = bfly->get_switch_id(coords);
            coordinates test_coords = bfly->get_switch_coords(test_id);
            assertEqual(unit, "fbfly switch id", test_id, nid);
            assertEqual(unit, "fbfly coords", test_coords, coords);
        }
    }

    
    {
        coordinates coords = bfly->get_switch_coords(14);
        assertEqual(unit, "fbfly switch coords", coords, 3, 2);
    }

    {
        coordinates coords = bfly->get_switch_coords(3);
        assertEqual(unit, "fbfly switch coords", coords, 0, 3);
    }

    {
        coordinates coords = bfly->get_switch_coords(11);
        assertEqual(unit, "fbfly switch coords", coords, 2, 3);
    }

    {
        coordinates coords = bfly->get_switch_coords(7);
        assertEqual(unit, "fbfly switch coords", coords, 1, 3);
    }

    {
    networkswitch::ptr sw = switches[naddr(7)];
    int dim,dir;
    bfly->minimal_route_to_node(naddr(7), naddr(4), dim, dir);
    assertEqual(unit, "route fbfly dim", dim, flattenedbutterfly::down_dimension);
    assertEqual(unit, "route fbfly dir", dir, 0);
    assert_dim_dir(unit, "fbfly dim/dir",
                   sw, dim, dir, 3);
    }

    {
    networkswitch::ptr sw = switches[naddr(3)];
    int dim,dir;
    bfly->minimal_route_to_node(naddr(3), naddr(4), dim, dir);
    assertEqual(unit, "route fbfly dim", dim, flattenedbutterfly::down_dimension);
    assertEqual(unit, "route fbfly dir", dir, 5);
    assert_dim_dir(unit, "fbfly dim/dir",
                   sw, dim, dir, 1);
    }
}

