#include <tests/unittests/util/util.h>
#include <sstmac/hardware/network/interconnect/switchinterconnect.h>

using namespace sstmac;
using namespace sstmac::hw;

coordinates
get_vector(int a){
    coordinates vec(1);
    vec[0] = a;
    return vec;
}

coordinates
get_vector(int a, int b){
    coordinates vec(2);
    vec[0] = a;
    vec[1] = b;
    return vec;
}

coordinates
get_vector(int a, int b, int c){
    coordinates vec(3);
    vec[0] = a;
    vec[1] = b;
    vec[2] = c;
    return vec;
}

coordinates
get_vector(int a, int b, int c, int d){
    coordinates vec(4);
    vec[0] = a;
    vec[1] = b;
    vec[2] = c;
    vec[3] = d;
    return vec;
}

coordinates
get_vector(int a, int b, int c, int d, int e){
    coordinates vec(5);
    vec[0] = a;
    vec[1] = b;
    vec[2] = c;
    vec[3] = d;
    vec[4] = e;
    return vec;
}

coordinates
get_vector(int a, int b, int c, int d, int e, int f){
    coordinates vec(6);
    vec[0] = a;
    vec[1] = b;
    vec[2] = c;
    vec[3] = d;
    vec[4] = e;
    vec[5] = f;
    return vec;
}

coordinates
get_vector(int a, int b, int c, int d, int e, int f, int g){
    coordinates vec(7);
    vec[0] = a;
    vec[1] = b;
    vec[2] = c;
    vec[3] = d;
    vec[4] = e;
    vec[5] = f;
    vec[6] = g;
    return vec;
}


sstmac::nodeaddress::ptr
naddr(long nid)
{
    return sstmac::nodeaddress::convert_from_uniqueid(nid);
}

interconnect_message::ptr
msg(long nid)
{
    network_message::ptr new_msg = network_message::construct();
    new_msg->set_toaddr(naddr(nid));
    return interconnect_message::construct(new_msg);
}

void
init_switches(topology::maptype &switches,
              sstmac::unittest_params& params,
              const topology::ptr &top)
{
    params["network_bandwidth_link"] = "1.0GB/s";
    params["packetswitch_latency_r2r"] = "100ns";
    params["packetswitch_bandwidth_n2r"] = "1.0GB/s";
    params["network_switchtype"] = "packet";
  long num_switches = top->num_switches();

  // create all the switches
  networkswitch::ptr switchtemp = switchinterconnect::get_template_switch(params, top);
  for (long i=0; i < num_switches; i++)
  {
      nodeaddress::ptr sw_addr = nodeaddress::convert_from_uniqueid(i);
      networkswitch::ptr sw = switchtemp->clone(
          top->create_router(sw_addr, params), sw_addr);
     // sw->init_factory_params(params);
      switches[sw_addr] = sw;
  }
  top->connect_switches(switches);
}

void _assert_dim_dir(UnitTest& unit, const char* descr,
                    const char* file, int line,
                    networkswitch::ptr sw, int dim, int dir,
                    long outport_sw_id)
{
    discrete_network_switch::ptr dsw
        = boost::dynamic_pointer_cast<discrete_network_switch>(sw);
    if (!dsw){
        assertTrue(unit, descr, false);
    }
    long port = sw->get_router()->convert_to_outport(dim, dir);
    networkswitch::ptr outport_sw
        = boost::dynamic_pointer_cast<networkswitch>
          (dsw->get_outport_handler(port));
    if (outport_sw){
        long test_id = outport_sw->get_addr()->unique_id();
        _assertEqual(unit, descr, truncate_file(file), line, test_id, outport_sw_id);
    }
    else {
        _assertTrue(unit, descr, truncate_file(file), line, false);
    }
}

nodeaddress::ptr
get_addr_at_dim_dir(const networkswitch::ptr &sw, int dim, int dir)
{
    discrete_network_switch::ptr outport_sw =
        boost::dynamic_pointer_cast<discrete_network_switch>(
            get_switch_at_dim_dir(sw, dim, dir));

    if (outport_sw){
        return outport_sw->get_addr();
    }
    else {
        return nodeaddress::convert_from_uniqueid(-1);
    }
}


networkswitch::ptr
get_switch_at_dim_dir(const networkswitch::ptr &sw, int dim, int dir)
{
    discrete_network_switch::ptr dsw
        = boost::dynamic_pointer_cast<discrete_network_switch>(sw);
    if (!dsw){
        return networkswitch::ptr();
    }
    long port = sw->get_router()->convert_to_outport(dim, dir);
    networkswitch::ptr outport_sw
        = boost::dynamic_pointer_cast<networkswitch>
          (dsw->get_outport_handler(port));

    return outport_sw;
}

