#include <getopt.h>
#include <stdio.h>
#include "analyzer/setting/setting.h"
#include "common/log/logger.h"

namespace sstmac { namespace tools { namespace analyzer {

bool Setting::Load(int argc, char* argv[])
{
    //Not enough arugments provided
    if (argc <= 1)
    {
        OUTPUT_INFO("Usage:\t%s [options] MetaFile\n\t%s -h for more information\n", argv[0], argv[0]);
        return false;
    }

    bool isValid = true;
    int optVal;

    while((optVal = getopt(argc, argv, ":hvo:")) != -1)
    {
        switch(optVal)
        {
        case 'h':
            OUTPUT_INFO( 
                "Trace Analysis Tool generate statistics report from dumpi traces.\n"
	            "Usage: %s [options] MetaFile\n"
                "   Options:\n"
                "       -h              Print this help\n"
                "       -v              Verbose status output\n"
                "       -o  filename    Write report into file\n",
                argv[0]);
            isValid = false;
            break;
        case 'o':
            m_reportPath = optarg;
            break;
        case 'v':
            SetVerbose(true);
            break;
        case ':':
            OUTPUT_ERROR("Missing arugment for option \"%c\"\n", optopt);
            isValid = false;
            break;
        case '?':
        default:
            OUTPUT_ERROR("Unknown option \"%c\"\n", optopt);
            isValid = false;
            break;
        }
    }

    if (optind < argc)
    {
        //Expect one more agument as input metafile name
        if(optind + 1 == argc)    
        {
            m_metaPath = argv[optind];
        }
        else
        {
            OUTPUT_ERROR("Too many arguments while expecting only one\n");
            isValid = false;
        }
    }
    
    return isValid;
}

const std::string& Setting::GetMetaPath() const
{
    return m_metaPath;
}

const std::string& Setting::GetReportPath() const
{
    return m_reportPath;
}

} } } //namespace
