#ifndef SSTMAC_TOOLS_ANALYZER_CALLRECORDBASE
#define SSTMAC_TOOLS_ANALYZER_CALLRECORDBASE

#include <utility>
#include <vector>

namespace sstmac { namespace tools { namespace analyzer {

class MPIContext;

typedef std::pair<double, double> TimeRange;

// CallRecord encapsulate the information of one MPI call in the trace
class CallRecordBase 
{
public:
    CallRecordBase (MPIContext* context, int function, TimeRange cpuTime, TimeRange wallTime);
    virtual ~CallRecordBase ();

    // Recover MPI context info 
    virtual bool RecoverContext() const = 0;
    
    // Getters used by statistics module
    int GetNodeRank() const;
    int GetFunction() const;

    TimeRange GetCpuTime() const;
    TimeRange GetWallTime() const;

    virtual int GetBytesSend() const = 0;
    virtual int GetBytesSend(int nodeRank) const = 0;

    virtual void GetDest(std::vector<int>& nodeVec, bool local = false) const = 0;
    virtual void GetSource(std::vector<int>& nodeVec, bool local = false) const = 0;

    virtual bool IsCollective() const = 0;
    virtual bool IsInit() const = 0;
    virtual bool IsFinalize() const = 0;

protected:
    MPIContext* m_context;
    int m_function;
    TimeRange m_cpuTime;
    TimeRange m_wallTime;
};

} } } //namespace 

#endif //SSTMAC_TOOLS_ANALYZER_CALLRECORDBASE
