//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include "analyzer/setting/setting.h"
#include "analyzer/trace/trace.h"
#include "analyzer/stats/statsgenerate.h"
#include "common/report/reportstats.h"
#include "common/utilities/time.h"
#include "common/commoninitializer.h"

using namespace sstmac::tools;
using namespace sstmac::tools::analyzer;

enum ErrorCode
{
    Success = 0,
    WrongSetting,
    InvalidTrace,
    InvalidOutput
};

int main(int argc, char* argv[])
{
    CommonInitializer commonInitializer;
    ScopedTimer timeTrace("Statistics collected done in ");

    // 1. Parse and fill settings
    Setting setting;

    if (!setting.Load(argc, argv)) 
    {
        return WrongSetting;
    }

    // 2. Load trace files
    Trace trace;

    if (!trace.Init(setting.GetMetaPath()))
    {
        return InvalidTrace;
    }

    // 3. Run collectors to generate report
    StatsGenerate stats(trace);

    if (!stats.Generate())
    {
        return InvalidTrace; 
    }

    // 4. Output report data to file
    ReportStats report(stats);

    if(!report.Write(setting.GetReportPath()))
    {
        return InvalidOutput; 
    }

    return Success;
}
