//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include <cmath>
#include "common/numeric/accumulatorimpl.h"
#include "common/utilities/assert.h"

namespace sstmac { namespace tools {

////////////////////////////////////////////////////////////////////////////////
// Sum
Sum::Sum()
    : m_sum(0.0)
{}

void Sum::Accumulate(double val)
{
    m_sum += val;
}

double Sum::Evaluate() const
{
    return m_sum;
}

////////////////////////////////////////////////////////////////////////////////
// Average
Average::Average()
    : m_sum(0.0)
    , m_count(0)
{}

void Average::Accumulate(double val)
{
    m_sum += val;

    ++m_count;
}

double Average::Evaluate() const
{
    return m_count ? m_sum / m_count : 0.0;
}

////////////////////////////////////////////////////////////////////////////////
// Deviation
Deviation::Deviation()
    : m_mean(0.0)
    , m_m2(0.0)
    , m_count(0)
{}

void Deviation::Accumulate(double val)
{
    ++m_count;

    double delta = val - m_mean;
    m_mean = m_mean + delta / m_count;
    m_m2 = m_m2 + delta * (val - m_mean);
}

double Deviation::Evaluate() const
{
    return m_count ? sqrt(m_m2 / m_count) : 0.0;
}

} } //namespace
