//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#ifndef SSTMAC_TOOLS_COMMON_REGISTRY_SEQUENCEKEY_INCLUDED_
#define SSTMAC_TOOLS_COMMON_REGISTRY_SEQUENCEKEY_INCLUDED_

#include <map>
#include "common/utilities/assert.h"

namespace sstmac { namespace tools {

template<typename StubType>
class SequenceKey 
{
public:
    typedef int KeyType;

public:
    SequenceKey() : m_sequenceId(0) {}
    
    const KeyType& AllocateKey(const StubType& stub) {
        typename KeyStorageType::const_iterator iter = m_keyStorage.find(stub);
        if(iter == m_keyStorage.end())
        {
            return m_keyStorage[stub] = m_sequenceId++;
        }
        return iter->second;
    }

    const KeyType& GetKey(const StubType& stub) const {
        typename KeyStorageType::const_iterator iter = m_keyStorage.find(stub);
        if(iter == m_keyStorage.end())
        {
            return INVALID_KEY;
        }
        return iter->second;
    }
    
    void FreeKey(const StubType& stub) {
        m_keyStorage.erase(stub);
    }

private:
    typedef std::map<StubType, KeyType> KeyStorageType;
    KeyStorageType m_keyStorage;

    KeyType m_sequenceId;
    static const KeyType INVALID_KEY;
};

template<typename StubType>
const typename SequenceKey<StubType>::KeyType SequenceKey<StubType>::INVALID_KEY = -1;

} } //namespace

#endif //SSTMAC_TOOLS_COMMON_REGISTRY_SEQUENCEKEY_INCLUDED_
