//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include "common/utilities/containerhelper.h"
#include "common/log/logger.h"
#include "comparator/comparison/comparisongenerate.h"
#include "comparator/comparison/criterion/criteriongenerateimpl.h"

namespace sstmac { namespace tools { namespace comparator {

#define ADD_CRITERION(TYPE) \
    Register(criterion::TYPE::StaticClassId(), CriterionDataBase::Ptr(new CriterionGenerate<criterion::TYPE>()))

ComparisonGenerate::ComparisonGenerate()
{
    ADD_CRITERION(Overall);
    ADD_CRITERION(ByNode);
    ADD_CRITERION(MpiCall);
    ADD_CRITERION(Collective);
    ADD_CRITERION(TransmissionBytes);
}

bool ComparisonGenerate::Compare(const Stats& refStats, const Stats& simStats)
{
    bool ret = true;

    m_meta[0] = refStats.GetMetaData();
    m_meta[1] = simStats.GetMetaData();

    for (Iterator iter = Begin(); iter != End(); ++iter) 
    {
        CriterionDataBase::Ptr criterionData = iter->second;
        CriterionGenerateBase::Ptr criterionGen = boost::dynamic_pointer_cast<CriterionGenerateBase>(criterionData);

        bool result = criterionGen->Compare(refStats, simStats);
        ret &= result;

        OUTPUT_LOG("Comparing trace reports section [%s]...%s\n", criterionData->GetClassId(), result ? "Passed" : "Failed");
    }

    return ret;
}

} } } //namespace
