//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include "comparator/setting/setting.h"
#include "comparator/comparison/comparisongenerate.h"
#include "common/commoninitializer.h"
#include "common/stats/stats.h"
#include "common/report/reportstats.h"
#include "common/report/reportcomparison.h"
#include "common/utilities/time.h"

using namespace sstmac::tools;
using namespace sstmac::tools::comparator;

enum ErrorCode
{
    Success = 0,
    WrongSetting,
    InvalidTrace,
    ErrorComparing,
    InvalidOutput
};

int main(int argc, char *argv[])
{
    CommonInitializer commonInitializer;
    ScopedTimer timeTrace("Comparing done in ");

    // 1. Parse and fill settings
    Setting setting;

    if (!setting.Load(argc, argv)) 
    {
        return WrongSetting;
    }

    // 2. Load stats from report file
    Stats refStats;
    Stats simStats;

    ReportStats refReport(refStats);
    ReportStats simReport(simStats); 

    if(!refReport.Read(setting.GetReferencePath()) || !simReport.Read(setting.GetSimulationPath()))
    {
        return InvalidTrace;
    }

    // 3. Run criteria through stats
    ComparisonGenerate comparison;

    if(!comparison.Compare(refStats, simStats))
    {
        return ErrorComparing;
    }

    // 4. Output report
    ReportComparison report(comparison);

    if(!report.Write(setting.GetReportPath()))
    {
        return InvalidOutput;
    }

    return Success;
}
