//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include <wx/menu.h>
#include <wx/log.h>
#include <wx/sizer.h>
#include <wx/layout.h>
#include <wx/msgdlg.h>
#include <wx/textctrl.h>
#include "visualizer/ui/mainframe.h"
#include "visualizer/ui/plotframe.h"
#include "visualizer/visualizerapp.h"

namespace sstmac { namespace tools { namespace visualizer {

IMPLEMENT_CLASS(MainFrame, wxDocParentFrame)

BEGIN_EVENT_TABLE(MainFrame, wxDocParentFrame)
    EVT_MENU(wxID_ABOUT, MainFrame::OnMenuHelpAbout)
END_EVENT_TABLE()

MainFrame::MainFrame(wxDocManager* docManager, const wxString& title)
    : wxDocParentFrame(docManager, NULL, wxID_ANY, title, wxDefaultPosition, wxDefaultSize)
{
    //CreateToolBar(); 
    CreateStatusBar(); 
    CreateMenuBar(docManager);

    //Create log window
    wxTextCtrl* textCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
    textCtrl->SetMinSize(wxSize(640, 300));
    m_oldLog = wxLog::SetActiveTarget(new wxLogTextCtrl(textCtrl));
    wxLog::SetTimestamp((wxChar*)NULL);

    //Place log and plot windows
    wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
    sizer->Add(textCtrl, 1, wxEXPAND);
    SetSizerAndFit(sizer);
    Centre();

    wxLogMessage(wxT("Open an report xml file to begin..."));
}

MainFrame::~MainFrame()
{
    delete wxLog::SetActiveTarget(m_oldLog);
}

void MainFrame::OnMenuHelpAbout(wxCommandEvent& WXUNUSED(cmd))
{
    wxMessageBox(wxT("DUMPI trace visualization"), wxT("About"), wxOK| wxICON_INFORMATION, this);
}

void MainFrame::CreateMenuBar(wxDocManager* docManager)
{
    wxMenuBar* menuBar = new wxMenuBar();
    SetMenuBar(menuBar);

    wxMenu* fileMenu = new wxMenu();
    menuBar->Append(fileMenu, wxT("&File"));
    fileMenu->Append(wxID_OPEN, wxT("&Open"));
    fileMenu->Append(wxID_CLOSE, wxT("&Close"));
    fileMenu->AppendSeparator();
    fileMenu->Append(wxID_EXIT, wxT("&Exit"));
    docManager->FileHistoryUseMenu(fileMenu);

    wxMenu* aboutMenu = new wxMenu();
    menuBar->Append(aboutMenu, wxT("&Help"));
    aboutMenu->Append(wxID_ABOUT, wxT("&About..."));

#ifdef __WXMAC__
    wxMenuBar::MacSetCommonMenuBar(menuBar);
#endif
}

MainFrame* GetMainFrame()
{
    return dynamic_cast<MainFrame*>(wxGetApp().GetTopWindow());
}

} } } //namespace
