//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include <wx/cmdline.h>
#include <wx/docview.h>

#ifdef __WXMAC__
#include <Carbon/Carbon.h>
extern "C" { void CPSEnableForegroundOperation( ProcessSerialNumber* psn ); }
#endif

#include "visualizer/visualizerapp.h"
#include "visualizer/ui/mainframe.h"
#include "visualizer/document/statsdocument.h"
#include "visualizer/view/statsview.h"

namespace sstmac { namespace tools { namespace visualizer {

bool VisualizerApp::OnInit()
{
#ifdef __WXMAC__
    // this hack enables to have a GUI on Mac OSX even if the
    // program was called from the command line (and isn't a bundle)
    ProcessSerialNumber psn;

    GetCurrentProcess( &psn );
    CPSEnableForegroundOperation( &psn );
    SetFrontProcess( &psn );
#endif

    m_docManager = new wxDocManager(wxDOC_MDI);

    new wxDocTemplate(m_docManager, wxT("DUMPI trace report"), wxT("*.xml"), wxT("~/"), wxT("xml"),wxT("stats Doc"), wxT("stats View"), CLASSINFO(StatsDocument), CLASSINFO(StatsView));

    MainFrame* frame = new MainFrame(m_docManager, wxT("Trace Visualizer"));
    frame->Show(true);
    SetTopWindow(frame);

    return true;
}

int VisualizerApp::OnExit()
{
    delete m_docManager;
    return 0; 
}

void VisualizerApp::OnInitCmdLine(wxCmdLineParser& parser)
{
    wxApp::OnInitCmdLine(parser);
}

bool VisualizerApp::OnCmdLineParsed(wxCmdLineParser& parser)
{
    wxArrayString files;

    for (unsigned int i = 0; i < parser.GetParamCount(); i++) 
    {
        files.Add(parser.GetParam(i));
    }

    return true;
}

} } } //namespace

IMPLEMENT_APP(sstmac::tools::visualizer::VisualizerApp);
