/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef mem_bw_test_H_
#define mem_bw_test_H_

#include <sstmac/software/api/mpi/mpiapp.h>
#include <sstmac/software/api/mpi/sstmac_mpi.h>
#include <sstmac/software/libraries/compute/lib_compute_memmove.h>

namespace validation
{

  /**
   * App that can test out the memory bandwidth of the node
   */
  class mem_bw_test : virtual public sstmac::sw::mpiapp
  {

  protected:
    /// Hi.
    mem_bw_test(sstmac::sw::softwareid id) :
      sstmac::sw::mpiapp(id, to_string())
    {
      lib_ = sstmac::sw::lib_compute_memmove::construct(id);
      register_lib(lib_);
    }

    mem_bw_test(const sstmac::sim_parameters::ptr& params) : sstmac::sw::mpiapp(params) {}

  public:
    typedef boost::intrusive_ptr<mem_bw_test> ptr;

    /// Hello.
    static ptr
    construct(sstmac::sw::softwareid id)
    {
      return ptr(new mem_bw_test(id));
    }

    static ptr
    construct(const sstmac::sim_parameters::ptr& params)
    {
        return new mem_bw_test(params);
    }

    /// Goodbye.
    virtual
    ~mem_bw_test() throw ()
    {
    }

    /// Get a copy.
    virtual sstmac::sw::app::ptr
    clone(sstmac::sw::softwareid id)
    {
      return construct(id);
    }

    /// Go.
    void
    skeleton_main();

    virtual void
    consume_params(const sstmac::sim_parameters::ptr &params);

    virtual std::string
    to_string() const
    {
      return "mem_bw_test";
    }

  protected:

    sstmac::sw::lib_compute_memmove::ptr lib_;

    long numiter_;
    long data_size_;
    timestamp sleep_;
    bool do_remote_;
    int sockets_;

  };
}

#endif
