#include <sstmac/sstmacro.h>
#include "mpi_bw_test.h"
#include <mpi/sstmac_mpi.h>

using namespace sstmac;
using namespace sstmac::sw;

namespace validation
{
  SSTRegisterApp("mpi_bw_test", mpi_bw_test);

  void
  mpi_bw_test::consume_params(const sim_parameters::ptr &params)
  {

    numiter_ = params->get_long_param("mpi_bw_test_iterations");
    data_size_ = params->get_long_param("mpi_bw_test_size");
    sleep_ = params->get_time_param("mpi_bw_test_sleep");
    do_remote_ = params->get_optional_bool_param("mpi_bw_test_remote", false);
    sockets_ = params->get_optional_int_param("node_num_procs", -1);
  }

  void
  mpi_bw_test::skeleton_main()
  {
    int rank, size;

    MPI_Init(NULL, NULL);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);

    timestamp begin = now();

    long total = 0;

    double totallat = 0;

    for (long i = 0; i < numiter_; i++)
      {
        //SSTMAC_DEBUG << "mpi_bw_test[" << rank << "]: starting iteration " << i
        //      << "out of " << numiter_ << "\n";
        timestamp start = now();
        if(rank % 2 == 0){
            MPI_Send(NULL, data_size_, MPI_DOUBLE, rank + 1, 0, MPI_COMM_WORLD);
        }else{
            MPI_Status stat;
            MPI_Recv(NULL, data_size_, MPI_DOUBLE, rank - 1, 0, MPI_COMM_WORLD, &stat);
        }
        timestamp done = now();
        totallat += (done.sec() - start.sec());
        total += data_size_;

        sleep(sleep_);
      }

    double totaltime = now().sec() - begin.sec();
    SSTMAC_DEBUG << "rank " << rank << ": mem bw, avg lat: " << ((double) total
          / totaltime) << ", " << (totallat / (double) numiter_) << "\n";

    MPI_Finalize();
  }

}
